proc configure_fpga_with_bitstream { } {
  # Get the directory containing this script. Source file paths are relative to this.
  set origin_dir [file dirname [file normalize [info script]]]
  set this_script [file tail [info script]]

  # User can set use_remote_hw_server in Tcl console before sourceing this script
  global use_remote_hw_server
  set url {localhost:3121}
  if { [info exists use_remote_hw_server] } {
    set url $use_remote_hw_server
  }
  
  set design {ddr4_test}
  set top {ddr4_test}
  # Extract the design configuration and device from the name of this script
  set dir [regsub -nocase {^debug-} $this_script {}]
  set dir [regsub -nocase {.tcl$} $dir {}]
  set device [lindex [split $dir {-}] end]
  set device [lindex [split $device {_}] 0]
  if { [info exists ::env(ADMXRC3_PKG_TEST)] } {
    append dir {-} [version -short]
  }
  set bit_path [file normalize "${origin_dir}/${dir}/${design}.runs/impl_1/${top}.bit"]

  # The logic for this script is that we look for a .bit file in the same directory as this script; if it exists, we use it.
  # Otherwise, expect the .bit file in 'bit_relpath', as set above.
  set bit_filename [file tail $bit_path]
  if { [file exists "${origin_dir}/${bit_filename}" ] } {
    set bit_dirpath $origin_dir
  } else {
    set bit_dirpath [file dirname $bit_path]
  }

  set ltx_filename [file rootname $bit_filename]
  append ltx_filename {.ltx}

  load_features labtools
  # open_hw deprecated in Vivado 2019.1 in favour of open_hw_manager
  if { [llength [info commands open_hw_manager]] > 0 } {
    open_hw_manager
  } else {
    open_hw
  }

  set old_hw_server [ get_hw_server ]
  if { $old_hw_server ne {} } {
    disconnect_hw_server $old_hw_server
  }

  connect_hw_server -url $url
  
  set target [lindex [get_hw_targets] 0]
  open_hw_target $target

  set device [lindex [get_hw_devices -filter "PART =~ xc${device}"] 0]
  set_property PROBES.FILE "${bit_dirpath}/${ltx_filename}" $device
  set_property PROGRAM.FILE "${bit_dirpath}/${bit_filename}" $device
  program_hw_devices $device
  refresh_hw_device $device
}

configure_fpga_with_bitstream
