library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

entity dma_demo is
generic(
  sim_axi    : boolean := false);
port(
  -- MPTL
  mptl_clk_p : in    std_logic;
  mptl_clk_n : in    std_logic;
  mptl_b2t_p : in    std_logic_vector(3 downto 0);
  mptl_b2t_n : in    std_logic_vector(3 downto 0);
  mptl_t2b_p : out   std_logic_vector(3 downto 0);
  mptl_t2b_n : out   std_logic_vector(3 downto 0);
  -- MPTL sideband
  mptl_bridge_gtp_online_l : in    std_logic;
  mptl_target_configured_l : out   std_logic;
  mptl_target_gtp_online_l : out   std_logic);
end entity;

architecture admxrcku1 of dma_demo is

  -- Wrapper for Host Slave (AXI) IP
  component mptl_tgt_ku_axi_wrap
  generic(
    sim_axi : boolean := false);
  port(
    -- Reset input
    sys_rst           : in    std_logic;
    -- AXI4
    aclk              : out   std_logic;
    aresetn           : out   std_logic;
    --
    ds_axi_awaddr     : out   std_logic_vector(63 downto 0);
    ds_axi_awlen      : out   std_logic_vector(7 downto 0);
    ds_axi_awsize     : out   std_logic_vector(2 downto 0);
    ds_axi_awburst    : out   std_logic_vector(1 downto 0);
    ds_axi_awcache    : out   std_logic_vector(3 downto 0);
    ds_axi_awprot     : out   std_logic_vector(2 downto 0);
    ds_axi_awvalid    : out   std_logic;
    ds_axi_wdata      : out   std_logic_vector(127 downto 0);
    ds_axi_wstrb      : out   std_logic_vector(15 downto 0);
    ds_axi_wlast      : out   std_logic;
    ds_axi_wvalid     : out   std_logic;
    ds_axi_bready     : out   std_logic;
    ds_axi_araddr     : out   std_logic_vector(63 downto 0);
    ds_axi_arlen      : out   std_logic_vector(7 downto 0);
    ds_axi_arsize     : out   std_logic_vector(2 downto 0);
    ds_axi_arburst    : out   std_logic_vector(1 downto 0);
    ds_axi_arcache    : out   std_logic_vector(3 downto 0);
    ds_axi_arprot     : out   std_logic_vector(2 downto 0);
    ds_axi_arvalid    : out   std_logic;
    ds_axi_rready     : out   std_logic;
    ds_axi_awready    : in    std_logic;
    ds_axi_wready     : in    std_logic;
    ds_axi_bresp      : in    std_logic_vector(1 downto 0);
    ds_axi_bvalid     : in    std_logic;
    ds_axi_arready    : in    std_logic;
    ds_axi_rdata      : in    std_logic_vector(127 downto 0);
    ds_axi_rresp      : in    std_logic_vector(1 downto 0);
    ds_axi_rlast      : in    std_logic;
    ds_axi_rvalid     : in    std_logic;
    ds_axi_awlock     : out   std_logic_vector(0 downto 0);
    ds_axi_awqos      : out   std_logic_vector(3 downto 0);
    ds_axi_arlock     : out   std_logic_vector(0 downto 0);
    ds_axi_arqos      : out   std_logic_vector(3 downto 0);
    ds_axi_awregion   : out   std_logic_vector(3 downto 0);
    ds_axi_arregion   : out   std_logic_vector(3 downto 0);
    --
    dma0_axi_awaddr   : out   std_logic_vector(38 downto 0);
    dma0_axi_awlen    : out   std_logic_vector(7 downto 0);
    dma0_axi_awsize   : out   std_logic_vector(2 downto 0);
    dma0_axi_awburst  : out   std_logic_vector(1 downto 0);
    dma0_axi_awcache  : out   std_logic_vector(3 downto 0);
    dma0_axi_awprot   : out   std_logic_vector(2 downto 0);
    dma0_axi_awvalid  : out   std_logic;
    dma0_axi_wdata    : out   std_logic_vector(127 downto 0);
    dma0_axi_wstrb    : out   std_logic_vector(15 downto 0);
    dma0_axi_wlast    : out   std_logic;
    dma0_axi_wvalid   : out   std_logic;
    dma0_axi_bready   : out   std_logic;
    dma0_axi_araddr   : out   std_logic_vector(38 downto 0);
    dma0_axi_arlen    : out   std_logic_vector(7 downto 0);
    dma0_axi_arsize   : out   std_logic_vector(2 downto 0);
    dma0_axi_arburst  : out   std_logic_vector(1 downto 0);
    dma0_axi_arcache  : out   std_logic_vector(3 downto 0);
    dma0_axi_arprot   : out   std_logic_vector(2 downto 0);
    dma0_axi_arvalid  : out   std_logic;
    dma0_axi_rready   : out   std_logic;
    dma0_axi_awready  : in    std_logic;
    dma0_axi_wready   : in    std_logic;
    dma0_axi_bresp    : in    std_logic_vector(1 downto 0);
    dma0_axi_bvalid   : in    std_logic;
    dma0_axi_arready  : in    std_logic;
    dma0_axi_rdata    : in    std_logic_vector(127 downto 0);
    dma0_axi_rresp    : in    std_logic_vector(1 downto 0);
    dma0_axi_rlast    : in    std_logic;
    dma0_axi_rvalid   : in    std_logic;
    dma0_axi_awlock   : out   std_logic_vector(0 downto 0);
    dma0_axi_awqos    : out   std_logic_vector(3 downto 0);
    dma0_axi_arlock   : out   std_logic_vector(0 downto 0);
    dma0_axi_arqos    : out   std_logic_vector(3 downto 0);
    dma0_axi_awregion : out   std_logic_vector(3 downto 0);
    dma0_axi_arregion : out   std_logic_vector(3 downto 0);
    --
    dma1_axi_awaddr   : out   std_logic_vector(38 downto 0);
    dma1_axi_awlen    : out   std_logic_vector(7 downto 0);
    dma1_axi_awsize   : out   std_logic_vector(2 downto 0);
    dma1_axi_awburst  : out   std_logic_vector(1 downto 0);
    dma1_axi_awcache  : out   std_logic_vector(3 downto 0);
    dma1_axi_awprot   : out   std_logic_vector(2 downto 0);
    dma1_axi_awvalid  : out   std_logic;
    dma1_axi_wdata    : out   std_logic_vector(127 downto 0);
    dma1_axi_wstrb    : out   std_logic_vector(15 downto 0);
    dma1_axi_wlast    : out   std_logic;
    dma1_axi_wvalid   : out   std_logic;
    dma1_axi_bready   : out   std_logic;
    dma1_axi_araddr   : out   std_logic_vector(38 downto 0);
    dma1_axi_arlen    : out   std_logic_vector(7 downto 0);
    dma1_axi_arsize   : out   std_logic_vector(2 downto 0);
    dma1_axi_arburst  : out   std_logic_vector(1 downto 0);
    dma1_axi_arcache  : out   std_logic_vector(3 downto 0);
    dma1_axi_arprot   : out   std_logic_vector(2 downto 0);
    dma1_axi_arvalid  : out   std_logic;
    dma1_axi_rready   : out   std_logic;
    dma1_axi_awready  : in    std_logic;
    dma1_axi_wready   : in    std_logic;
    dma1_axi_bresp    : in    std_logic_vector(1 downto 0);
    dma1_axi_bvalid   : in    std_logic;
    dma1_axi_arready  : in    std_logic;
    dma1_axi_rdata    : in    std_logic_vector(127 downto 0);
    dma1_axi_rresp    : in    std_logic_vector(1 downto 0);
    dma1_axi_rlast    : in    std_logic;
    dma1_axi_rvalid   : in    std_logic;
    dma1_axi_awlock   : out   std_logic_vector(0 downto 0);
    dma1_axi_awqos    : out   std_logic_vector(3 downto 0);
    dma1_axi_arlock   : out   std_logic_vector(0 downto 0);
    dma1_axi_arqos    : out   std_logic_vector(3 downto 0);
    dma1_axi_awregion : out   std_logic_vector(3 downto 0);
    dma1_axi_arregion : out   std_logic_vector(3 downto 0);
    -- MPTL
    mptl_t2b_p               : out   std_logic_vector(3 downto 0);
    mptl_t2b_n               : out   std_logic_vector(3 downto 0);
    mptl_b2t_p               : in    std_logic_vector(3 downto 0);
    mptl_b2t_n               : in    std_logic_vector(3 downto 0);
    mptl_clk_p               : in    std_logic;
    mptl_clk_n               : in    std_logic;
    -- MPTL initialisation
    mptl_target_configured_l : out   std_logic;
    mptl_target_gtp_online_l : out   std_logic;
    mptl_bridge_gtp_online_l : in    std_logic;
    -- MPTL sideband
    mptl_gpio_t2b            : in    std_logic_vector(15 downto 0);
    mptl_gpio_b2t            : out   std_logic_vector(15 downto 0));
  end component;

  -- AXI4 crossbar switch IP
  component axi_crossbar_ip
  port(
    aclk    : in    std_logic;
    aresetn : in    std_logic;
    --
    s_axi_awaddr  : in    std_logic_vector(63 downto 0);
    s_axi_awlen   : in    std_logic_vector(7 downto 0);
    s_axi_awsize  : in    std_logic_vector(2 downto 0);
    s_axi_awburst : in    std_logic_vector(1 downto 0);
    s_axi_awlock  : in    std_logic_vector(0 downto 0);
    s_axi_awcache : in    std_logic_vector(3 downto 0);
    s_axi_awprot  : in    std_logic_vector(2 downto 0);
    s_axi_awqos   : in    std_logic_vector(3 downto 0);
    s_axi_awvalid : in    std_logic_vector(0 downto 0);
    s_axi_awready : out   std_logic_vector(0 downto 0);
    s_axi_wdata   : in    std_logic_vector(127 downto 0);
    s_axi_wstrb   : in    std_logic_vector(15 downto 0);
    s_axi_wlast   : in    std_logic_vector(0 downto 0);
    s_axi_wvalid  : in    std_logic_vector(0 downto 0);
    s_axi_wready  : out   std_logic_vector(0 downto 0);
    s_axi_bresp   : out   std_logic_vector(1 downto 0);
    s_axi_bvalid  : out   std_logic_vector(0 downto 0);
    s_axi_bready  : in    std_logic_vector(0 downto 0);
    s_axi_araddr  : in    std_logic_vector(63 downto 0);
    s_axi_arlen   : in    std_logic_vector(7 downto 0);
    s_axi_arsize  : in    std_logic_vector(2 downto 0);
    s_axi_arburst : in    std_logic_vector(1 downto 0);
    s_axi_arlock  : in    std_logic_vector(0 downto 0);
    s_axi_arcache : in    std_logic_vector(3 downto 0);
    s_axi_arprot  : in    std_logic_vector(2 downto 0);
    s_axi_arqos   : in    std_logic_vector(3 downto 0);
    s_axi_arvalid : in    std_logic_vector(0 downto 0);
    s_axi_arready : out   std_logic_vector(0 downto 0);
    s_axi_rdata   : out   std_logic_vector(127 downto 0);
    s_axi_rresp   : out   std_logic_vector(1 downto 0);
    s_axi_rlast   : out   std_logic_vector(0 downto 0);
    s_axi_rvalid  : out   std_logic_vector(0 downto 0);
    s_axi_rready  : in    std_logic_vector(0 downto 0);
    --
    m_axi_awaddr   : out   std_logic_vector(127 downto 0);
    m_axi_awlen    : out   std_logic_vector(15 downto 0);
    m_axi_awsize   : out   std_logic_vector(5 downto 0);
    m_axi_awburst  : out   std_logic_vector(3 downto 0);
    m_axi_awlock   : out   std_logic_vector(1 downto 0);
    m_axi_awcache  : out   std_logic_vector(7 downto 0);
    m_axi_awprot   : out   std_logic_vector(5 downto 0);
    m_axi_awregion : out   std_logic_vector(7 downto 0);
    m_axi_awqos    : out   std_logic_vector(7 downto 0);
    m_axi_awvalid  : out   std_logic_vector(1 downto 0);
    m_axi_awready  : in    std_logic_vector(1 downto 0);
    m_axi_wdata    : out   std_logic_vector(255 downto 0);
    m_axi_wstrb    : out   std_logic_vector(31 downto 0);
    m_axi_wlast    : out   std_logic_vector(1 downto 0);
    m_axi_wvalid   : out   std_logic_vector(1 downto 0);
    m_axi_wready   : in    std_logic_vector(1 downto 0);
    m_axi_bresp    : in    std_logic_vector(3 downto 0);
    m_axi_bvalid   : in    std_logic_vector(1 downto 0);
    m_axi_bready   : out   std_logic_vector(1 downto 0);
    m_axi_araddr   : out   std_logic_vector(127 downto 0);
    m_axi_arlen    : out   std_logic_vector(15 downto 0);
    m_axi_arsize   : out   std_logic_vector(5 downto 0);
    m_axi_arburst  : out   std_logic_vector(3 downto 0);
    m_axi_arlock   : out   std_logic_vector(1 downto 0);
    m_axi_arcache  : out   std_logic_vector(7 downto 0);
    m_axi_arprot   : out   std_logic_vector(5 downto 0);
    m_axi_arregion : out   std_logic_vector(7 downto 0);
    m_axi_arqos    : out   std_logic_vector(7 downto 0);
    m_axi_arvalid  : out   std_logic_vector(1 downto 0);
    m_axi_arready  : in    std_logic_vector(1 downto 0);
    m_axi_rdata    : in    std_logic_vector(255 downto 0);
    m_axi_rresp    : in    std_logic_vector(3 downto 0);
    m_axi_rlast    : in    std_logic_vector(1 downto 0);
    m_axi_rvalid   : in    std_logic_vector(1 downto 0);
    m_axi_rready   : out   std_logic_vector(1 downto 0));
  end component;

  -- AXI4 BRAM interface IP
  component ram_interface_axi4_ip
  port(
    aresetn : in    std_logic;
    aclk    : in    std_logic;
    --
    s_axi_araddr   : in    std_logic_vector(18 downto 0);
    s_axi_arburst  : in    std_logic_vector(1 downto 0);
    s_axi_arcache  : in    std_logic_vector(3 downto 0);
    s_axi_arlen    : in    std_logic_vector(7 downto 0);
    s_axi_arlock   : in    std_logic_vector(0 downto 0);
    s_axi_arprot   : in    std_logic_vector(2 downto 0);
    s_axi_arqos    : in    std_logic_vector(3 downto 0);
    s_axi_arready  : out   std_logic;
    s_axi_arregion : in    std_logic_vector(3 downto 0);
    s_axi_arsize   : in    std_logic_vector(2 downto 0);
    s_axi_arvalid  : in    std_logic;
    s_axi_awaddr   : in    std_logic_vector(18 downto 0);
    s_axi_awburst  : in    std_logic_vector(1 downto 0);
    s_axi_awcache  : in    std_logic_vector(3 downto 0);
    s_axi_awlen    : in    std_logic_vector(7 downto 0);
    s_axi_awlock   : in    std_logic_vector(0 downto 0);
    s_axi_awprot   : in    std_logic_vector(2 downto 0);
    s_axi_awqos    : in    std_logic_vector(3 downto 0);
    s_axi_awready  : out   std_logic;
    s_axi_awregion : in    std_logic_vector(3 downto 0);
    s_axi_awsize   : in    std_logic_vector(2 downto 0);
    s_axi_awvalid  : in    std_logic;
    s_axi_wdata    : in    std_logic_vector(127 downto 0);
    s_axi_wlast    : in    std_logic;
    s_axi_wstrb    : in    std_logic_vector(15 downto 0);
    s_axi_wready   : out   std_logic;
    s_axi_wvalid   : in    std_logic;
    s_axi_bready   : in    std_logic;
    s_axi_bresp    : out   std_logic_vector(1 downto 0);
    s_axi_bvalid   : out   std_logic;
    s_axi_rdata    : out   std_logic_vector(127 downto 0);
    s_axi_rlast    : out   std_logic;
    s_axi_rready   : in    std_logic;
    s_axi_rresp    : out   std_logic_vector(1 downto 0);
    s_axi_rvalid   : out   std_logic;
    --
    ram_clk_a  : out   std_logic;
    ram_rst_a  : out   std_logic;
    ram_en_a   : out   std_logic;
    ram_addr_a : out   std_logic_vector(14 downto 0);
    ram_we_a   : out   std_logic_vector(15 downto 0);
    ram_din_a  : out   std_logic_vector(127 downto 0);
    ram_dout_a : in    std_logic_vector(127 downto 0));
  end component;

  -- BRAM IP
  component blk_mem_gen_ip
  port(
    clka  : in    std_logic;
    ena   : in    std_logic;
    wea   : in    std_logic_vector(15 downto 0);
    addra : in    std_logic_vector(14 downto 0);
    dina  : in    std_logic_vector(127 downto 0);
    douta : out   std_logic_vector(127 downto 0);
    clkb  : in    std_logic;
    enb   : in    std_logic;
    web   : in    std_logic_vector(15 downto 0);
    addrb : in    std_logic_vector(14 downto 0);
    dinb  : in    std_logic_vector(127 downto 0);
    doutb : out   std_logic_vector(127 downto 0));
  end component;


  -----------------------------
  -- mptl_tgt_ku_axi signals --
  -----------------------------
  signal aclk           : std_logic;
  signal aresetn        : std_logic;

  signal mptl_gpio_b2t : std_logic_vector(15 downto 0);
  signal mptl_gpio_t2b : std_logic_vector(15 downto 0) := (others => '0');

  -- Host Slave DS
  signal ds_axi_awaddr   : std_logic_vector(63 downto 0);
  signal ds_axi_awlen    : std_logic_vector(7 downto 0);
  signal ds_axi_awsize   : std_logic_vector(2 downto 0);
  signal ds_axi_awburst  : std_logic_vector(1 downto 0);
  signal ds_axi_awcache  : std_logic_vector(3 downto 0);
  signal ds_axi_awprot   : std_logic_vector(2 downto 0);
  signal ds_axi_awvalid  : std_logic_vector(0 downto 0);
  signal ds_axi_wdata    : std_logic_vector(127 downto 0);
  signal ds_axi_wstrb    : std_logic_vector(15 downto 0);
  signal ds_axi_wlast    : std_logic_vector(0 downto 0);
  signal ds_axi_wvalid   : std_logic_vector(0 downto 0);
  signal ds_axi_bready   : std_logic_vector(0 downto 0);
  signal ds_axi_araddr   : std_logic_vector(63 downto 0);
  signal ds_axi_arlen    : std_logic_vector(7 downto 0);
  signal ds_axi_arsize   : std_logic_vector(2 downto 0);
  signal ds_axi_arburst  : std_logic_vector(1 downto 0);
  signal ds_axi_arcache  : std_logic_vector(3 downto 0);
  signal ds_axi_arprot   : std_logic_vector(2 downto 0);
  signal ds_axi_arvalid  : std_logic_vector(0 downto 0);
  signal ds_axi_rready   : std_logic_vector(0 downto 0);
  signal ds_axi_awready  : std_logic_vector(0 downto 0);
  signal ds_axi_wready   : std_logic_vector(0 downto 0);
  signal ds_axi_bresp    : std_logic_vector(1 downto 0);
  signal ds_axi_bvalid   : std_logic_vector(0 downto 0);
  signal ds_axi_arready  : std_logic_vector(0 downto 0);
  signal ds_axi_rdata    : std_logic_vector(127 downto 0);
  signal ds_axi_rresp    : std_logic_vector(1 downto 0);
  signal ds_axi_rlast    : std_logic_vector(0 downto 0);
  signal ds_axi_rvalid   : std_logic_vector(0 downto 0);
  signal ds_axi_awlock   : std_logic_vector(0 downto 0);
  signal ds_axi_awqos    : std_logic_vector(3 downto 0);
  signal ds_axi_arlock   : std_logic_vector(0 downto 0);
  signal ds_axi_arqos    : std_logic_vector(3 downto 0);
  signal ds_axi_awregion : std_logic_vector(3 downto 0);
  signal ds_axi_arregion : std_logic_vector(3 downto 0);

  -- Host Slave DMA0
  signal dma0_axi_awaddr   : std_logic_vector(38 downto 0);
  signal dma0_axi_awlen    : std_logic_vector(7 downto 0);
  signal dma0_axi_awsize   : std_logic_vector(2 downto 0);
  signal dma0_axi_awburst  : std_logic_vector(1 downto 0);
  signal dma0_axi_awcache  : std_logic_vector(3 downto 0);
  signal dma0_axi_awprot   : std_logic_vector(2 downto 0);
  signal dma0_axi_awvalid  : std_logic;
  signal dma0_axi_wdata    : std_logic_vector(127 downto 0);
  signal dma0_axi_wstrb    : std_logic_vector(15 downto 0);
  signal dma0_axi_wlast    : std_logic;
  signal dma0_axi_wvalid   : std_logic;
  signal dma0_axi_bready   : std_logic;
  signal dma0_axi_araddr   : std_logic_vector(38 downto 0);
  signal dma0_axi_arlen    : std_logic_vector(7 downto 0);
  signal dma0_axi_arsize   : std_logic_vector(2 downto 0);
  signal dma0_axi_arburst  : std_logic_vector(1 downto 0);
  signal dma0_axi_arcache  : std_logic_vector(3 downto 0);
  signal dma0_axi_arprot   : std_logic_vector(2 downto 0);
  signal dma0_axi_arvalid  : std_logic;
  signal dma0_axi_rready   : std_logic;
  signal dma0_axi_awready  : std_logic;
  signal dma0_axi_wready   : std_logic;
  signal dma0_axi_bresp    : std_logic_vector(1 downto 0);
  signal dma0_axi_bvalid   : std_logic;
  signal dma0_axi_arready  : std_logic;
  signal dma0_axi_rdata    : std_logic_vector(127 downto 0);
  signal dma0_axi_rresp    : std_logic_vector(1 downto 0);
  signal dma0_axi_rlast    : std_logic;
  signal dma0_axi_rvalid   : std_logic;
  signal dma0_axi_awlock   : std_logic_vector(0 downto 0);
  signal dma0_axi_awqos    : std_logic_vector(3 downto 0);
  signal dma0_axi_arlock   : std_logic_vector(0 downto 0);
  signal dma0_axi_arqos    : std_logic_vector(3 downto 0);
  signal dma0_axi_awregion : std_logic_vector(3 downto 0);
  signal dma0_axi_arregion : std_logic_vector(3 downto 0);

  -- Host Slave DMA1
  signal dma1_axi_awaddr   : std_logic_vector(38 downto 0);
  signal dma1_axi_awlen    : std_logic_vector(7 downto 0);
  signal dma1_axi_awsize   : std_logic_vector(2 downto 0);
  signal dma1_axi_awburst  : std_logic_vector(1 downto 0);
  signal dma1_axi_awcache  : std_logic_vector(3 downto 0);
  signal dma1_axi_awprot   : std_logic_vector(2 downto 0);
  signal dma1_axi_awvalid  : std_logic;
  signal dma1_axi_wdata    : std_logic_vector(127 downto 0);
  signal dma1_axi_wstrb    : std_logic_vector(15 downto 0);
  signal dma1_axi_wlast    : std_logic;
  signal dma1_axi_wvalid   : std_logic;
  signal dma1_axi_bready   : std_logic;
  signal dma1_axi_araddr   : std_logic_vector(38 downto 0);
  signal dma1_axi_arlen    : std_logic_vector(7 downto 0);
  signal dma1_axi_arsize   : std_logic_vector(2 downto 0);
  signal dma1_axi_arburst  : std_logic_vector(1 downto 0);
  signal dma1_axi_arcache  : std_logic_vector(3 downto 0);
  signal dma1_axi_arprot   : std_logic_vector(2 downto 0);
  signal dma1_axi_arvalid  : std_logic;
  signal dma1_axi_rready   : std_logic;
  signal dma1_axi_awready  : std_logic;
  signal dma1_axi_wready   : std_logic;
  signal dma1_axi_bresp    : std_logic_vector(1 downto 0);
  signal dma1_axi_bvalid   : std_logic;
  signal dma1_axi_arready  : std_logic;
  signal dma1_axi_rdata    : std_logic_vector(127 downto 0);
  signal dma1_axi_rresp    : std_logic_vector(1 downto 0);
  signal dma1_axi_rlast    : std_logic;
  signal dma1_axi_rvalid   : std_logic;
  signal dma1_axi_awlock   : std_logic_vector(0 downto 0);
  signal dma1_axi_awqos    : std_logic_vector(3 downto 0);
  signal dma1_axi_arlock   : std_logic_vector(0 downto 0);
  signal dma1_axi_arqos    : std_logic_vector(3 downto 0);
  signal dma1_axi_awregion : std_logic_vector(3 downto 0);
  signal dma1_axi_arregion : std_logic_vector(3 downto 0);

  -- Crossbar outputs
  signal x0_axi_awaddr   : std_logic_vector(63 downto 0);
  signal x0_axi_awlen    : std_logic_vector(7 downto 0);
  signal x0_axi_awsize   : std_logic_vector(2 downto 0);
  signal x0_axi_awburst  : std_logic_vector(1 downto 0);
  signal x0_axi_awcache  : std_logic_vector(3 downto 0);
  signal x0_axi_awprot   : std_logic_vector(2 downto 0);
  signal x0_axi_awvalid  : std_logic;
  signal x0_axi_wdata    : std_logic_vector(127 downto 0);
  signal x0_axi_wstrb    : std_logic_vector(15 downto 0);
  signal x0_axi_wlast    : std_logic;
  signal x0_axi_wvalid   : std_logic;
  signal x0_axi_bready   : std_logic;
  signal x0_axi_araddr   : std_logic_vector(63 downto 0);
  signal x0_axi_arlen    : std_logic_vector(7 downto 0);
  signal x0_axi_arsize   : std_logic_vector(2 downto 0);
  signal x0_axi_arburst  : std_logic_vector(1 downto 0);
  signal x0_axi_arcache  : std_logic_vector(3 downto 0);
  signal x0_axi_arprot   : std_logic_vector(2 downto 0);
  signal x0_axi_arvalid  : std_logic;
  signal x0_axi_rready   : std_logic;
  signal x0_axi_awready  : std_logic;
  signal x0_axi_wready   : std_logic;
  signal x0_axi_bresp    : std_logic_vector(1 downto 0);
  signal x0_axi_bvalid   : std_logic;
  signal x0_axi_arready  : std_logic;
  signal x0_axi_rdata    : std_logic_vector(127 downto 0);
  signal x0_axi_rresp    : std_logic_vector(1 downto 0);
  signal x0_axi_rlast    : std_logic;
  signal x0_axi_rvalid   : std_logic;
  signal x0_axi_awlock   : std_logic_vector(0 downto 0);
  signal x0_axi_awqos    : std_logic_vector(3 downto 0);
  signal x0_axi_arlock   : std_logic_vector(0 downto 0);
  signal x0_axi_arqos    : std_logic_vector(3 downto 0);
  signal x0_axi_awregion : std_logic_vector(3 downto 0);
  signal x0_axi_arregion : std_logic_vector(3 downto 0);

  signal x1_axi_awaddr   : std_logic_vector(63 downto 0);
  signal x1_axi_awlen    : std_logic_vector(7 downto 0);
  signal x1_axi_awsize   : std_logic_vector(2 downto 0);
  signal x1_axi_awburst  : std_logic_vector(1 downto 0);
  signal x1_axi_awcache  : std_logic_vector(3 downto 0);
  signal x1_axi_awprot   : std_logic_vector(2 downto 0);
  signal x1_axi_awvalid  : std_logic;
  signal x1_axi_wdata    : std_logic_vector(127 downto 0);
  signal x1_axi_wstrb    : std_logic_vector(15 downto 0);
  signal x1_axi_wlast    : std_logic;
  signal x1_axi_wvalid   : std_logic;
  signal x1_axi_bready   : std_logic;
  signal x1_axi_araddr   : std_logic_vector(63 downto 0);
  signal x1_axi_arlen    : std_logic_vector(7 downto 0);
  signal x1_axi_arsize   : std_logic_vector(2 downto 0);
  signal x1_axi_arburst  : std_logic_vector(1 downto 0);
  signal x1_axi_arcache  : std_logic_vector(3 downto 0);
  signal x1_axi_arprot   : std_logic_vector(2 downto 0);
  signal x1_axi_arvalid  : std_logic;
  signal x1_axi_rready   : std_logic;
  signal x1_axi_awready  : std_logic;
  signal x1_axi_wready   : std_logic;
  signal x1_axi_bresp    : std_logic_vector(1 downto 0);
  signal x1_axi_bvalid   : std_logic;
  signal x1_axi_arready  : std_logic;
  signal x1_axi_rdata    : std_logic_vector(127 downto 0);
  signal x1_axi_rresp    : std_logic_vector(1 downto 0);
  signal x1_axi_rlast    : std_logic;
  signal x1_axi_rvalid   : std_logic;
  signal x1_axi_awlock   : std_logic_vector(0 downto 0);
  signal x1_axi_awqos    : std_logic_vector(3 downto 0);
  signal x1_axi_arlock   : std_logic_vector(0 downto 0);
  signal x1_axi_arqos    : std_logic_vector(3 downto 0);
  signal x1_axi_awregion : std_logic_vector(3 downto 0);
  signal x1_axi_arregion : std_logic_vector(3 downto 0);

  signal x0_ram_clk_a  : std_logic;
  signal x0_ram_rst_a  : std_logic;
  signal x0_ram_en_a   : std_logic;
  signal x0_ram_addr_a : std_logic_vector(14 downto 0);
  signal x0_ram_we_a   : std_logic_vector(15 downto 0);
  signal x0_ram_din_a  : std_logic_vector(127 downto 0);
  signal x0_ram_dout_a : std_logic_vector(127 downto 0);

  signal x1_ram_clk_a  : std_logic;
  signal x1_ram_rst_a  : std_logic;
  signal x1_ram_en_a   : std_logic;
  signal x1_ram_addr_a : std_logic_vector(14 downto 0);
  signal x1_ram_we_a   : std_logic_vector(15 downto 0);
  signal x1_ram_din_a  : std_logic_vector(127 downto 0);
  signal x1_ram_dout_a : std_logic_vector(127 downto 0);

  signal dma0_ram_clk_a  : std_logic;
  signal dma0_ram_rst_a  : std_logic;
  signal dma0_ram_en_a   : std_logic;
  signal dma0_ram_addr_a : std_logic_vector(14 downto 0);
  signal dma0_ram_we_a   : std_logic_vector(15 downto 0);
  signal dma0_ram_din_a  : std_logic_vector(127 downto 0);
  signal dma0_ram_dout_a : std_logic_vector(127 downto 0);

  signal dma1_ram_clk_a  : std_logic;
  signal dma1_ram_rst_a  : std_logic;
  signal dma1_ram_en_a   : std_logic;
  signal dma1_ram_addr_a : std_logic_vector(14 downto 0);
  signal dma1_ram_we_a   : std_logic_vector(15 downto 0);
  signal dma1_ram_din_a  : std_logic_vector(127 downto 0);
  signal dma1_ram_dout_a : std_logic_vector(127 downto 0);

  signal logic0 : std_logic;
  
begin

  logic0 <= '0';
  

  -----------------------------------
  -- MPTL interface and AXI blocks --
  -----------------------------------

  -- Instantiate Host Slave (AXI) Interface
  mptl_tgt_ku_axi_wrap_i : mptl_tgt_ku_axi_wrap
  generic map(
    sim_axi => sim_axi)
  port map(
    -- Reset input
    sys_rst => logic0,  -- in  std_logic;
    -- AXI4 Interfaces
    aclk    => aclk,    -- out std_logic;
    aresetn => aresetn, -- out std_logic;
    -- Direct Slave
    ds_axi_awaddr   => ds_axi_awaddr,      -- out   std_logic_vector(63 downto 0);
    ds_axi_awlen    => ds_axi_awlen,       -- out   std_logic_vector(7 downto 0);
    ds_axi_awsize   => ds_axi_awsize,      -- out   std_logic_vector(2 downto 0);
    ds_axi_awburst  => ds_axi_awburst,     -- out   std_logic_vector(1 downto 0);
    ds_axi_awcache  => ds_axi_awcache,     -- out   std_logic_vector(3 downto 0);
    ds_axi_awprot   => ds_axi_awprot,      -- out   std_logic_vector(2 downto 0);
    ds_axi_awvalid  => ds_axi_awvalid(0),  -- out   std_logic;
    ds_axi_wdata    => ds_axi_wdata,       -- out   std_logic_vector(127 downto 0);
    ds_axi_wstrb    => ds_axi_wstrb,       -- out   std_logic_vector(15 downto 0);
    ds_axi_wlast    => ds_axi_wlast(0),    -- out   std_logic;
    ds_axi_wvalid   => ds_axi_wvalid(0),   -- out   std_logic;
    ds_axi_bready   => ds_axi_bready(0),   -- out   std_logic;
    ds_axi_araddr   => ds_axi_araddr,      -- out   std_logic_vector(63 downto 0);
    ds_axi_arlen    => ds_axi_arlen,       -- out   std_logic_vector(7 downto 0);
    ds_axi_arsize   => ds_axi_arsize,      -- out   std_logic_vector(2 downto 0);
    ds_axi_arburst  => ds_axi_arburst,     -- out   std_logic_vector(1 downto 0);
    ds_axi_arcache  => ds_axi_arcache,     -- out   std_logic_vector(3 downto 0);
    ds_axi_arprot   => ds_axi_arprot,      -- out   std_logic_vector(2 downto 0);
    ds_axi_arvalid  => ds_axi_arvalid(0),  -- out   std_logic;
    ds_axi_rready   => ds_axi_rready(0),   -- out   std_logic;
    ds_axi_awready  => ds_axi_awready(0),  -- in    std_logic;
    ds_axi_wready   => ds_axi_wready(0),   -- in    std_logic;
    ds_axi_bresp    => ds_axi_bresp,       -- in    std_logic_vector(1 downto 0);
    ds_axi_bvalid   => ds_axi_bvalid(0),   -- in    std_logic;
    ds_axi_arready  => ds_axi_arready(0),  -- in    std_logic;
    ds_axi_rdata    => ds_axi_rdata,       -- in    std_logic_vector(127 downto 0);
    ds_axi_rresp    => ds_axi_rresp,       -- in    std_logic_vector(1 downto 0);
    ds_axi_rlast    => ds_axi_rlast(0),    -- in    std_logic;
    ds_axi_rvalid   => ds_axi_rvalid(0),   -- in    std_logic;
    ds_axi_awlock   => ds_axi_awlock,      -- out   std_logic_vector(0 downto 0);
    ds_axi_awqos    => ds_axi_awqos,       -- out   std_logic_vector(3 downto 0);
    ds_axi_arlock   => ds_axi_arlock,      -- out   std_logic_vector(0 downto 0);
    ds_axi_arqos    => ds_axi_arqos,       -- out   std_logic_vector(3 downto 0);
    ds_axi_awregion => ds_axi_awregion,    -- out   std_logic_vector(3 downto 0);
    ds_axi_arregion => ds_axi_arregion,    -- out   std_logic_vector(3 downto 0);
    --
    dma0_axi_awaddr   => dma0_axi_awaddr,
    dma0_axi_awlen    => dma0_axi_awlen,
    dma0_axi_awsize   => dma0_axi_awsize,
    dma0_axi_awburst  => dma0_axi_awburst,
    dma0_axi_awcache  => dma0_axi_awcache,
    dma0_axi_awprot   => dma0_axi_awprot,
    dma0_axi_awvalid  => dma0_axi_awvalid,
    dma0_axi_wdata    => dma0_axi_wdata,
    dma0_axi_wstrb    => dma0_axi_wstrb,
    dma0_axi_wlast    => dma0_axi_wlast,
    dma0_axi_wvalid   => dma0_axi_wvalid,
    dma0_axi_bready   => dma0_axi_bready,
    dma0_axi_araddr   => dma0_axi_araddr,
    dma0_axi_arlen    => dma0_axi_arlen,
    dma0_axi_arsize   => dma0_axi_arsize,
    dma0_axi_arburst  => dma0_axi_arburst,
    dma0_axi_arcache  => dma0_axi_arcache,
    dma0_axi_arprot   => dma0_axi_arprot,
    dma0_axi_arvalid  => dma0_axi_arvalid,
    dma0_axi_rready   => dma0_axi_rready,
    dma0_axi_awready  => dma0_axi_awready,
    dma0_axi_wready   => dma0_axi_wready,
    dma0_axi_bresp    => dma0_axi_bresp,
    dma0_axi_bvalid   => dma0_axi_bvalid,
    dma0_axi_arready  => dma0_axi_arready,
    dma0_axi_rdata    => dma0_axi_rdata,
    dma0_axi_rresp    => dma0_axi_rresp,
    dma0_axi_rlast    => dma0_axi_rlast,
    dma0_axi_rvalid   => dma0_axi_rvalid,
    dma0_axi_awlock   => dma0_axi_awlock,
    dma0_axi_awqos    => dma0_axi_awqos,
    dma0_axi_arlock   => dma0_axi_arlock,
    dma0_axi_arqos    => dma0_axi_arqos,
    dma0_axi_awregion => dma0_axi_awregion,
    dma0_axi_arregion => dma0_axi_arregion,
    --
    dma1_axi_awaddr   => dma1_axi_awaddr,
    dma1_axi_awlen    => dma1_axi_awlen,
    dma1_axi_awsize   => dma1_axi_awsize,
    dma1_axi_awburst  => dma1_axi_awburst,
    dma1_axi_awcache  => dma1_axi_awcache,
    dma1_axi_awprot   => dma1_axi_awprot,
    dma1_axi_awvalid  => dma1_axi_awvalid,
    dma1_axi_wdata    => dma1_axi_wdata,
    dma1_axi_wstrb    => dma1_axi_wstrb,
    dma1_axi_wlast    => dma1_axi_wlast,
    dma1_axi_wvalid   => dma1_axi_wvalid,
    dma1_axi_bready   => dma1_axi_bready,
    dma1_axi_araddr   => dma1_axi_araddr,
    dma1_axi_arlen    => dma1_axi_arlen,
    dma1_axi_arsize   => dma1_axi_arsize,
    dma1_axi_arburst  => dma1_axi_arburst,
    dma1_axi_arcache  => dma1_axi_arcache,
    dma1_axi_arprot   => dma1_axi_arprot,
    dma1_axi_arvalid  => dma1_axi_arvalid,
    dma1_axi_rready   => dma1_axi_rready,
    dma1_axi_awready  => dma1_axi_awready,
    dma1_axi_wready   => dma1_axi_wready,
    dma1_axi_bresp    => dma1_axi_bresp,
    dma1_axi_bvalid   => dma1_axi_bvalid,
    dma1_axi_arready  => dma1_axi_arready,
    dma1_axi_rdata    => dma1_axi_rdata,
    dma1_axi_rresp    => dma1_axi_rresp,
    dma1_axi_rlast    => dma1_axi_rlast,
    dma1_axi_rvalid   => dma1_axi_rvalid,
    dma1_axi_awlock   => dma1_axi_awlock,
    dma1_axi_awqos    => dma1_axi_awqos,
    dma1_axi_arlock   => dma1_axi_arlock,
    dma1_axi_arqos    => dma1_axi_arqos,
    dma1_axi_awregion => dma1_axi_awregion,
    dma1_axi_arregion => dma1_axi_arregion,
    -- MPTL
    mptl_t2b_p => mptl_t2b_p, -- out std_logic_vector(3 downto 0);
    mptl_t2b_n => mptl_t2b_n, -- out std_logic_vector(3 downto 0);
    mptl_b2t_p => mptl_b2t_p, -- in  std_logic_vector(3 downto 0);
    mptl_b2t_n => mptl_b2t_n, -- in  std_logic_vector(3 downto 0);
    mptl_clk_p => mptl_clk_p, -- in  std_logic;
    mptl_clk_n => mptl_clk_n, -- in  std_logic;
    -- MPTL initialisation
    mptl_target_configured_l => mptl_target_configured_l, -- out std_logic;
    mptl_target_gtp_online_l => mptl_target_gtp_online_l, -- out std_logic;
    mptl_bridge_gtp_online_l => mptl_bridge_gtp_online_l, -- in  std_logic;
    -- MPTL sideband
    mptl_gpio_t2b => mptl_gpio_t2b,  -- in  std_logic_vector(15 downto 0);
    mptl_gpio_b2t => mptl_gpio_b2t); -- out std_logic_vector(15 downto 0));


  axi_crossbar_i : axi_crossbar_ip
  port map(
    aclk    => aclk,
    aresetn => aresetn,
    --
    s_axi_awaddr  => ds_axi_awaddr,
    s_axi_awlen   => ds_axi_awlen,
    s_axi_awsize  => ds_axi_awsize,
    s_axi_awburst => ds_axi_awburst,
    s_axi_awlock  => ds_axi_awlock,
    s_axi_awcache => ds_axi_awcache,
    s_axi_awprot  => ds_axi_awprot,
    s_axi_awqos   => ds_axi_awqos,
    s_axi_awvalid => ds_axi_awvalid,
    s_axi_awready => ds_axi_awready,
    s_axi_wdata   => ds_axi_wdata,
    s_axi_wstrb   => ds_axi_wstrb,
    s_axi_wlast   => ds_axi_wlast,
    s_axi_wvalid  => ds_axi_wvalid,
    s_axi_wready  => ds_axi_wready,
    s_axi_bresp   => ds_axi_bresp,
    s_axi_bvalid  => ds_axi_bvalid,
    s_axi_bready  => ds_axi_bready,
    s_axi_araddr  => ds_axi_araddr,
    s_axi_arlen   => ds_axi_arlen,
    s_axi_arsize  => ds_axi_arsize,
    s_axi_arburst => ds_axi_arburst,
    s_axi_arlock  => ds_axi_arlock,
    s_axi_arcache => ds_axi_arcache,
    s_axi_arprot  => ds_axi_arprot,
    s_axi_arqos   => ds_axi_arqos,
    s_axi_arvalid => ds_axi_arvalid,
    s_axi_arready => ds_axi_arready,
    s_axi_rdata   => ds_axi_rdata,
    s_axi_rresp   => ds_axi_rresp,
    s_axi_rlast   => ds_axi_rlast,
    s_axi_rvalid  => ds_axi_rvalid,
    s_axi_rready  => ds_axi_rready,
    --
    m_axi_awaddr(63 downto 0)    => x0_axi_awaddr,   -- out   std_logic_vector(127 downto 0);
    m_axi_awaddr(127 downto 64)  => x1_axi_awaddr,
    m_axi_awlen(7 downto 0)      => x0_axi_awlen,    -- out   std_logic_vector(15 downto 0);
    m_axi_awlen(15 downto 8)     => x1_axi_awlen,
    m_axi_awsize(2 downto 0)     => x0_axi_awsize,   -- out   std_logic_vector(5 downto 0);
    m_axi_awsize(5 downto 3)     => x1_axi_awsize,
    m_axi_awburst(1 downto 0)    => x0_axi_awburst,  -- out   std_logic_vector(3 downto 0);
    m_axi_awburst(3 downto 2)    => x1_axi_awburst,
    m_axi_awlock(0 downto 0)     => x0_axi_awlock,   -- out   std_logic_vector(1 downto 0);
    m_axi_awlock(1 downto 1)     => x1_axi_awlock,
    m_axi_awcache(3 downto 0)    => x0_axi_awcache,  -- out   std_logic_vector(7 downto 0);
    m_axi_awcache(7 downto 4)    => x1_axi_awcache,
    m_axi_awprot(2 downto 0)     => x0_axi_awprot,   -- out   std_logic_vector(5 downto 0);
    m_axi_awprot(5 downto 3)     => x1_axi_awprot,
    m_axi_awregion(3 downto 0)   => x0_axi_awregion, -- out   std_logic_vector(7 downto 0);
    m_axi_awregion(7 downto 4)   => x1_axi_awregion,
    m_axi_awqos(3 downto 0)      => x0_axi_awqos,    -- out   std_logic_vector(7 downto 0);
    m_axi_awqos(7 downto 4)      => x1_axi_awqos,
    m_axi_awvalid(0)             => x0_axi_awvalid,  -- out   std_logic_vector(1 downto 0);
    m_axi_awvalid(1)             => x1_axi_awvalid,
    m_axi_awready(0)             => x0_axi_awready,  -- in    std_logic_vector(1 downto 0);
    m_axi_awready(1)             => x1_axi_awready,
    m_axi_wdata(127 downto 0)    => x0_axi_wdata,    -- out   std_logic_vector(255 downto 0);
    m_axi_wdata(255 downto 128)  => x1_axi_wdata,
    m_axi_wstrb(15 downto 0)     => x0_axi_wstrb,    -- out   std_logic_vector(31 downto 0);
    m_axi_wstrb(31 downto 16)    => x1_axi_wstrb,
    m_axi_wlast(0)               => x0_axi_wlast,    -- out   std_logic_vector(1 downto 0);
    m_axi_wlast(1)               => x1_axi_wlast,
    m_axi_wvalid(0)              => x0_axi_wvalid,   -- out   std_logic_vector(1 downto 0);
    m_axi_wvalid(1)              => x1_axi_wvalid,
    m_axi_wready(0)              => x0_axi_wready,   -- in    std_logic_vector(1 downto 0);
    m_axi_wready(1)              => x1_axi_wready, 
    m_axi_bresp(1 downto 0)      => x0_axi_bresp,    -- in    std_logic_vector(3 downto 0);
    m_axi_bresp(3 downto 2)      => x0_axi_bresp,
    m_axi_bvalid(0)              => x0_axi_bvalid,   -- in    std_logic_vector(1 downto 0);
    m_axi_bvalid(1)              => x1_axi_bvalid,
    m_axi_bready(0)              => x0_axi_bready,   -- out   std_logic_vector(1 downto 0);
    m_axi_bready(1)              => x1_axi_bready,
    m_axi_araddr(63 downto 0)    => x0_axi_araddr,   -- out   std_logic_vector(127 downto 0);
    m_axi_araddr(127 downto 64)  => x1_axi_araddr,
    m_axi_arlen(7 downto 0)      => x0_axi_arlen,    -- out   std_logic_vector(15 downto 0);
    m_axi_arlen(15 downto 8)     => x1_axi_arlen,
    m_axi_arsize(2 downto 0)     => x0_axi_arsize,   -- out   std_logic_vector(5 downto 0);
    m_axi_arsize(5 downto 3)     => x1_axi_arsize,
    m_axi_arburst(1 downto 0)    => x0_axi_arburst,  -- out   std_logic_vector(3 downto 0);
    m_axi_arburst(3 downto 2)    => x1_axi_arburst,
    m_axi_arlock(0 downto 0)     => x0_axi_arlock,   -- out   std_logic_vector(1 downto 0);
    m_axi_arlock(1 downto 1)     => x1_axi_arlock,
    m_axi_arcache(3 downto 0)    => x0_axi_arcache,  -- out   std_logic_vector(7 downto 0);
    m_axi_arcache(7 downto 4)    => x1_axi_arcache,
    m_axi_arprot(2 downto 0)     => x0_axi_arprot,   -- out   std_logic_vector(5 downto 0);
    m_axi_arprot(5 downto 3)     => x1_axi_arprot,
    m_axi_arregion(3 downto 0)   => x0_axi_arregion, -- out   std_logic_vector(7 downto 0);
    m_axi_arregion(7 downto 4)   => x1_axi_arregion,
    m_axi_arqos(3 downto 0)      => x0_axi_arqos,    -- out   std_logic_vector(7 downto 0);
    m_axi_arqos(7 downto 4)      => x1_axi_arqos,
    m_axi_arvalid(0)             => x0_axi_arvalid,  -- out   std_logic_vector(1 downto 0);
    m_axi_arvalid(1)             => x1_axi_arvalid,
    m_axi_arready(0)             => x0_axi_arready,  -- in    std_logic_vector(1 downto 0);
    m_axi_arready(1)             => x1_axi_arready,
    m_axi_rdata(127 downto 0)    => x0_axi_rdata,    -- in    std_logic_vector(255 downto 0);
    m_axi_rdata(255 downto 128)  => x1_axi_rdata,
    m_axi_rresp(1 downto 0)      => x0_axi_rresp,    -- in    std_logic_vector(3 downto 0);
    m_axi_rresp(3 downto 2)      => x1_axi_rresp,
    m_axi_rlast(0)               => x0_axi_rlast,    -- in    std_logic_vector(1 downto 0);
    m_axi_rlast(1)               => x1_axi_rlast,
    m_axi_rvalid(0)              => x0_axi_rvalid,   -- in    std_logic_vector(1 downto 0);
    m_axi_rvalid(1)              => x1_axi_rvalid,
    m_axi_rready(0)              => x0_axi_rready,   -- out   std_logic_vector(1 downto 0));
    m_axi_rready(1)              => x1_axi_rready);


  x0_ram_if_i : ram_interface_axi4_ip
  port map(
    aresetn => aresetn, -- in    std_logic;
    aclk    => aclk,    -- in    std_logic;
    --
    s_axi_araddr   => x0_axi_araddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_arburst  => x0_axi_arburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_arcache  => x0_axi_arcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_arlen    => x0_axi_arlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_arlock   => x0_axi_arlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_arprot   => x0_axi_arprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_arqos    => x0_axi_arqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_arready  => x0_axi_arready,             -- out   std_logic;
    s_axi_arregion => x0_axi_arregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_arsize   => x0_axi_arsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_arvalid  => x0_axi_arvalid,             -- in    std_logic;
    s_axi_awaddr   => x0_axi_awaddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_awburst  => x0_axi_awburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_awcache  => x0_axi_awcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_awlen    => x0_axi_awlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_awlock   => x0_axi_awlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_awprot   => x0_axi_awprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_awqos    => x0_axi_awqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_awready  => x0_axi_awready,             -- out   std_logic;
    s_axi_awregion => x0_axi_awregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_awsize   => x0_axi_awsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_awvalid  => x0_axi_awvalid,             -- in    std_logic;
    s_axi_wdata    => x0_axi_wdata,               -- in    std_logic_vector(127 downto 0);
    s_axi_wlast    => x0_axi_wlast,               -- in    std_logic;
    s_axi_wstrb    => x0_axi_wstrb,               -- in    std_logic_vector(15 downto 0);
    s_axi_wready   => x0_axi_wready,              -- out   std_logic;
    s_axi_wvalid   => x0_axi_wvalid,              -- in    std_logic;
    s_axi_bready   => x0_axi_bready,              -- in    std_logic;
    s_axi_bresp    => x0_axi_bresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_bvalid   => x0_axi_bvalid,              -- out   std_logic;
    s_axi_rdata    => x0_axi_rdata,               -- out   std_logic_vector(127 downto 0);
    s_axi_rlast    => x0_axi_rlast,               -- out   std_logic;
    s_axi_rready   => x0_axi_rready,              -- in    std_logic;
    s_axi_rresp    => x0_axi_rresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_rvalid   => x0_axi_rvalid,              -- out   std_logic;
    --
    ram_clk_a  => x0_ram_clk_a,   -- out   std_logic;
    ram_rst_a  => x0_ram_rst_a,   -- out   std_logic;
    ram_en_a   => x0_ram_en_a,    -- out   std_logic;
    ram_addr_a => x0_ram_addr_a,  -- out   std_logic_vector(14 downto 0);
    ram_we_a   => x0_ram_we_a,    -- out   std_logic_vector(15 downto 0);
    ram_din_a  => x0_ram_din_a,   -- out   std_logic_vector(127 downto 0);
    ram_dout_a => x0_ram_dout_a); -- in    std_logic_vector(127 downto 0);

  x1_ram_if_i : ram_interface_axi4_ip
  port map(
    aresetn => aresetn, -- in    std_logic;
    aclk    => aclk,    -- in    std_logic;
    --
    s_axi_araddr   => x1_axi_araddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_arburst  => x1_axi_arburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_arcache  => x1_axi_arcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_arlen    => x1_axi_arlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_arlock   => x1_axi_arlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_arprot   => x1_axi_arprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_arqos    => x1_axi_arqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_arready  => x1_axi_arready,             -- out   std_logic;
    s_axi_arregion => x1_axi_arregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_arsize   => x1_axi_arsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_arvalid  => x1_axi_arvalid,             -- in    std_logic;
    s_axi_awaddr   => x1_axi_awaddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_awburst  => x1_axi_awburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_awcache  => x1_axi_awcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_awlen    => x1_axi_awlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_awlock   => x1_axi_awlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_awprot   => x1_axi_awprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_awqos    => x1_axi_awqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_awready  => x1_axi_awready,             -- out   std_logic;
    s_axi_awregion => x1_axi_awregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_awsize   => x1_axi_awsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_awvalid  => x1_axi_awvalid,             -- in    std_logic;
    s_axi_wdata    => x1_axi_wdata,               -- in    std_logic_vector(127 downto 0);
    s_axi_wlast    => x1_axi_wlast,               -- in    std_logic;
    s_axi_wstrb    => x1_axi_wstrb,               -- in    std_logic_vector(15 downto 0);
    s_axi_wready   => x1_axi_wready,              -- out   std_logic;
    s_axi_wvalid   => x1_axi_wvalid,              -- in    std_logic;
    s_axi_bready   => x1_axi_bready,              -- in    std_logic;
    s_axi_bresp    => x1_axi_bresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_bvalid   => x1_axi_bvalid,              -- out   std_logic;
    s_axi_rdata    => x1_axi_rdata,               -- out   std_logic_vector(127 downto 0);
    s_axi_rlast    => x1_axi_rlast,               -- out   std_logic;
    s_axi_rready   => x1_axi_rready,              -- in    std_logic;
    s_axi_rresp    => x1_axi_rresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_rvalid   => x1_axi_rvalid,              -- out   std_logic;
    --
    ram_clk_a  => x1_ram_clk_a,   -- out   std_logic;
    ram_rst_a  => x1_ram_rst_a,   -- out   std_logic;
    ram_en_a   => x1_ram_en_a,    -- out   std_logic;
    ram_addr_a => x1_ram_addr_a,  -- out   std_logic_vector(14 downto 0);
    ram_we_a   => x1_ram_we_a,    -- out   std_logic_vector(15 downto 0);
    ram_din_a  => x1_ram_din_a,   -- out   std_logic_vector(127 downto 0);
    ram_dout_a => x1_ram_dout_a); -- in    std_logic_vector(127 downto 0);

  dma0_ram_if_i : ram_interface_axi4_ip
  port map(
    aresetn => aresetn, -- in    std_logic;
    aclk    => aclk,    -- in    std_logic;
    --
    s_axi_araddr   => dma0_axi_araddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_arburst  => dma0_axi_arburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_arcache  => dma0_axi_arcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_arlen    => dma0_axi_arlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_arlock   => dma0_axi_arlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_arprot   => dma0_axi_arprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_arqos    => dma0_axi_arqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_arready  => dma0_axi_arready,             -- out   std_logic;
    s_axi_arregion => dma0_axi_arregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_arsize   => dma0_axi_arsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_arvalid  => dma0_axi_arvalid,             -- in    std_logic;
    s_axi_awaddr   => dma0_axi_awaddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_awburst  => dma0_axi_awburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_awcache  => dma0_axi_awcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_awlen    => dma0_axi_awlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_awlock   => dma0_axi_awlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_awprot   => dma0_axi_awprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_awqos    => dma0_axi_awqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_awready  => dma0_axi_awready,             -- out   std_logic;
    s_axi_awregion => dma0_axi_awregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_awsize   => dma0_axi_awsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_awvalid  => dma0_axi_awvalid,             -- in    std_logic;
    s_axi_wdata    => dma0_axi_wdata,               -- in    std_logic_vector(127 downto 0);
    s_axi_wlast    => dma0_axi_wlast,               -- in    std_logic;
    s_axi_wstrb    => dma0_axi_wstrb,               -- in    std_logic_vector(15 downto 0);
    s_axi_wready   => dma0_axi_wready,              -- out   std_logic;
    s_axi_wvalid   => dma0_axi_wvalid,              -- in    std_logic;
    s_axi_bready   => dma0_axi_bready,              -- in    std_logic;
    s_axi_bresp    => dma0_axi_bresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_bvalid   => dma0_axi_bvalid,              -- out   std_logic;
    s_axi_rdata    => dma0_axi_rdata,               -- out   std_logic_vector(127 downto 0);
    s_axi_rlast    => dma0_axi_rlast,               -- out   std_logic;
    s_axi_rready   => dma0_axi_rready,              -- in    std_logic;
    s_axi_rresp    => dma0_axi_rresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_rvalid   => dma0_axi_rvalid,              -- out   std_logic;
    --
    ram_clk_a  => dma0_ram_clk_a,   -- out   std_logic;
    ram_rst_a  => dma0_ram_rst_a,   -- out   std_logic;
    ram_en_a   => dma0_ram_en_a,    -- out   std_logic;
    ram_addr_a => dma0_ram_addr_a,  -- out   std_logic_vector(14 downto 0);
    ram_we_a   => dma0_ram_we_a,    -- out   std_logic_vector(15 downto 0);
    ram_din_a  => dma0_ram_din_a,   -- out   std_logic_vector(127 downto 0);
    ram_dout_a => dma0_ram_dout_a); -- in    std_logic_vector(127 downto 0);

  dma1_ram_if_i : ram_interface_axi4_ip
  port map(
    aresetn => aresetn, -- in    std_logic;
    aclk    => aclk,    -- in    std_logic;
    --
    s_axi_araddr   => dma1_axi_araddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_arburst  => dma1_axi_arburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_arcache  => dma1_axi_arcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_arlen    => dma1_axi_arlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_arlock   => dma1_axi_arlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_arprot   => dma1_axi_arprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_arqos    => dma1_axi_arqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_arready  => dma1_axi_arready,             -- out   std_logic;
    s_axi_arregion => dma1_axi_arregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_arsize   => dma1_axi_arsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_arvalid  => dma1_axi_arvalid,             -- in    std_logic;
    s_axi_awaddr   => dma1_axi_awaddr(18 downto 0), -- in    std_logic_vector(18 downto 0);
    s_axi_awburst  => dma1_axi_awburst,             -- in    std_logic_vector(1 downto 0);
    s_axi_awcache  => dma1_axi_awcache,             -- in    std_logic_vector(3 downto 0);
    s_axi_awlen    => dma1_axi_awlen,               -- in    std_logic_vector(7 downto 0);
    s_axi_awlock   => dma1_axi_awlock,              -- in    std_logic_vector(0 downto 0);
    s_axi_awprot   => dma1_axi_awprot,              -- in    std_logic_vector(2 downto 0);
    s_axi_awqos    => dma1_axi_awqos,               -- in    std_logic_vector(3 downto 0);
    s_axi_awready  => dma1_axi_awready,             -- out   std_logic;
    s_axi_awregion => dma1_axi_awregion,            -- in    std_logic_vector(3 downto 0);
    s_axi_awsize   => dma1_axi_awsize,              -- in    std_logic_vector(2 downto 0);
    s_axi_awvalid  => dma1_axi_awvalid,             -- in    std_logic;
    s_axi_wdata    => dma1_axi_wdata,               -- in    std_logic_vector(127 downto 0);
    s_axi_wlast    => dma1_axi_wlast,               -- in    std_logic;
    s_axi_wstrb    => dma1_axi_wstrb,               -- in    std_logic_vector(15 downto 0);
    s_axi_wready   => dma1_axi_wready,              -- out   std_logic;
    s_axi_wvalid   => dma1_axi_wvalid,              -- in    std_logic;
    s_axi_bready   => dma1_axi_bready,              -- in    std_logic;
    s_axi_bresp    => dma1_axi_bresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_bvalid   => dma1_axi_bvalid,              -- out   std_logic;
    s_axi_rdata    => dma1_axi_rdata,               -- out   std_logic_vector(127 downto 0);
    s_axi_rlast    => dma1_axi_rlast,               -- out   std_logic;
    s_axi_rready   => dma1_axi_rready,              -- in    std_logic;
    s_axi_rresp    => dma1_axi_rresp,               -- out   std_logic_vector(1 downto 0);
    s_axi_rvalid   => dma1_axi_rvalid,              -- out   std_logic;
    --
    ram_clk_a  => dma1_ram_clk_a,   -- out   std_logic;
    ram_rst_a  => dma1_ram_rst_a,   -- out   std_logic;
    ram_en_a   => dma1_ram_en_a,    -- out   std_logic;
    ram_addr_a => dma1_ram_addr_a,  -- out   std_logic_vector(14 downto 0);
    ram_we_a   => dma1_ram_we_a,    -- out   std_logic_vector(15 downto 0);
    ram_din_a  => dma1_ram_din_a,   -- out   std_logic_vector(127 downto 0);
    ram_dout_a => dma1_ram_dout_a); -- in    std_logic_vector(127 downto 0);


  dma0_blk_mem_gen_i : blk_mem_gen_ip
  port map(
    clka  => x0_ram_clk_a,     -- in    std_logic;
    ena   => x0_ram_en_a,      -- in    std_logic;
    wea   => x0_ram_we_a,      -- in    std_logic_vector(15 downto 0);
    addra => x0_ram_addr_a,    -- in    std_logic_vector(14 downto 0);
    dina  => x0_ram_din_a,     -- in    std_logic_vector(127 downto 0);
    douta => x0_ram_dout_a,    -- out   std_logic_vector(127 downto 0);
    --
    clkb  => dma0_ram_clk_a,   -- in    std_logic;
    enb   => dma0_ram_en_a,    -- in    std_logic;
    web   => dma0_ram_we_a,    -- in    std_logic_vector(15 downto 0);
    addrb => dma0_ram_addr_a,  -- in    std_logic_vector(14 downto 0); 
    dinb  => dma0_ram_din_a,   -- in    std_logic_vector(127 downto 0); 
    doutb => dma0_ram_dout_a); -- out   std_logic_vector(127 downto 0)); 

  dma1_blk_mem_gen_i : blk_mem_gen_ip
  port map(
    clka  => x1_ram_clk_a,     -- in    std_logic;
    ena   => x1_ram_en_a,      -- in    std_logic;
    wea   => x1_ram_we_a,      -- in    std_logic_vector(15 downto 0);
    addra => x1_ram_addr_a,    -- in    std_logic_vector(14 downto 0);
    dina  => x1_ram_din_a,     -- in    std_logic_vector(127 downto 0);
    douta => x1_ram_dout_a,    -- out   std_logic_vector(127 downto 0);
    --
    clkb  => dma1_ram_clk_a,   -- in    std_logic;
    enb   => dma1_ram_en_a,    -- in    std_logic;
    web   => dma1_ram_we_a,    -- in    std_logic_vector(15 downto 0);
    addrb => dma1_ram_addr_a,  -- in    std_logic_vector(14 downto 0); 
    dinb  => dma1_ram_din_a,   -- in    std_logic_vector(127 downto 0); 
    doutb => dma1_ram_dout_a); -- out   std_logic_vector(127 downto 0)); 

end architecture;