library ieee;
use ieee.std_logic_1164.all;

entity mptl_tgt_ku_axi_wrap is
generic(
  sim_axi : boolean := false);
port(
  -- Reset in
  sys_rst                  : in    std_logic;
  -- AXI4
  aclk                     : out   std_logic;
  aresetn                  : out   std_logic;
  -- Direct slave
  ds_axi_awaddr            : out   std_logic_vector(63 downto 0);
  ds_axi_awlen             : out   std_logic_vector(7 downto 0);
  ds_axi_awsize            : out   std_logic_vector(2 downto 0);
  ds_axi_awburst           : out   std_logic_vector(1 downto 0);
  ds_axi_awcache           : out   std_logic_vector(3 downto 0);
  ds_axi_awprot            : out   std_logic_vector(2 downto 0);
  ds_axi_awvalid           : out   std_logic;
  ds_axi_wdata             : out   std_logic_vector(127 downto 0);
  ds_axi_wstrb             : out   std_logic_vector(15 downto 0);
  ds_axi_wlast             : out   std_logic;
  ds_axi_wvalid            : out   std_logic;
  ds_axi_bready            : out   std_logic;
  ds_axi_araddr            : out   std_logic_vector(63 downto 0);
  ds_axi_arlen             : out   std_logic_vector(7 downto 0);
  ds_axi_arsize            : out   std_logic_vector(2 downto 0);
  ds_axi_arburst           : out   std_logic_vector(1 downto 0);
  ds_axi_arcache           : out   std_logic_vector(3 downto 0);
  ds_axi_arprot            : out   std_logic_vector(2 downto 0);
  ds_axi_arvalid           : out   std_logic;
  ds_axi_rready            : out   std_logic;
  ds_axi_awready           : in    std_logic;
  ds_axi_wready            : in    std_logic;
  ds_axi_bresp             : in    std_logic_vector(1 downto 0);
  ds_axi_bvalid            : in    std_logic;
  ds_axi_arready           : in    std_logic;
  ds_axi_rdata             : in    std_logic_vector(127 downto 0);
  ds_axi_rresp             : in    std_logic_vector(1 downto 0);
  ds_axi_rlast             : in    std_logic;
  ds_axi_rvalid            : in    std_logic;
  ds_axi_awlock            : out   std_logic_vector(0 downto 0);
  ds_axi_awqos             : out   std_logic_vector(3 downto 0);
  ds_axi_arlock            : out   std_logic_vector(0 downto 0);
  ds_axi_arqos             : out   std_logic_vector(3 downto 0);
  ds_axi_awregion          : out   std_logic_vector(3 downto 0);
  ds_axi_arregion          : out   std_logic_vector(3 downto 0);
  -- MPTL
  mptl_clk_p               : in    std_logic;
  mptl_clk_n               : in    std_logic;
  mptl_b2t_p               : in    std_logic_vector(3 downto 0);
  mptl_b2t_n               : in    std_logic_vector(3 downto 0);
  mptl_t2b_p               : out   std_logic_vector(3 downto 0);
  mptl_t2b_n               : out   std_logic_vector(3 downto 0);
  -- MPTL initialisation
  mptl_bridge_gtp_online_l : in    std_logic;
  mptl_target_configured_l : out   std_logic;
  mptl_target_gtp_online_l : out   std_logic;
  -- MPTL sideband
  mptl_gpio_t2b            : in    std_logic_vector(15 downto 0);
  mptl_gpio_b2t            : out   std_logic_vector(15 downto 0));
end;


architecture synth of mptl_tgt_ku_axi_wrap is

  -- Unused IO
  signal mptl_clk_out : std_logic;

begin

  mptl_tgt_ku_axi_syn_g : if (sim_axi = false) generate
    component mptl_tgt_ku_axi_ip
    port(
      -- Reset in
      async_sys_rst            : in    std_logic;    
      -- AXI4
      aclk                     : out   std_logic;
      aresetn                  : out   std_logic;
      -- Direct slave
      ds_axi_awaddr            : out   std_logic_vector(63 downto 0);
      ds_axi_awlen             : out   std_logic_vector(7 downto 0);
      ds_axi_awsize            : out   std_logic_vector(2 downto 0);
      ds_axi_awburst           : out   std_logic_vector(1 downto 0);
      ds_axi_awcache           : out   std_logic_vector(3 downto 0);
      ds_axi_awprot            : out   std_logic_vector(2 downto 0);
      ds_axi_awvalid           : out   std_logic;
      ds_axi_wdata             : out   std_logic_vector(127 downto 0);
      ds_axi_wstrb             : out   std_logic_vector(15 downto 0);
      ds_axi_wlast             : out   std_logic;
      ds_axi_wvalid            : out   std_logic;
      ds_axi_bready            : out   std_logic;
      ds_axi_araddr            : out   std_logic_vector(63 downto 0);
      ds_axi_arlen             : out   std_logic_vector(7 downto 0);
      ds_axi_arsize            : out   std_logic_vector(2 downto 0);
      ds_axi_arburst           : out   std_logic_vector(1 downto 0);
      ds_axi_arcache           : out   std_logic_vector(3 downto 0);
      ds_axi_arprot            : out   std_logic_vector(2 downto 0);
      ds_axi_arvalid           : out   std_logic;
      ds_axi_rready            : out   std_logic;
      ds_axi_awready           : in    std_logic;
      ds_axi_wready            : in    std_logic;
      ds_axi_bresp             : in    std_logic_vector(1 downto 0);
      ds_axi_bvalid            : in    std_logic;
      ds_axi_arready           : in    std_logic;
      ds_axi_rdata             : in    std_logic_vector(127 downto 0);
      ds_axi_rresp             : in    std_logic_vector(1 downto 0);
      ds_axi_rlast             : in    std_logic;
      ds_axi_rvalid            : in    std_logic;
      ds_axi_awlock            : out   std_logic_vector(0 downto 0);
      ds_axi_awqos             : out   std_logic_vector(3 downto 0);
      ds_axi_arlock            : out   std_logic_vector(0 downto 0);
      ds_axi_arqos             : out   std_logic_vector(3 downto 0);
      ds_axi_awregion          : out   std_logic_vector(3 downto 0);
      ds_axi_arregion          : out   std_logic_vector(3 downto 0);
      -- MPTL
      mptl_tx_p                : out   std_logic_vector(3 downto 0);
      mptl_tx_n                : out   std_logic_vector(3 downto 0);
      mptl_rx_p                : in    std_logic_vector(3 downto 0);
      mptl_rx_n                : in    std_logic_vector(3 downto 0);
      mptl_clk_p               : in    std_logic;
      mptl_clk_n               : in    std_logic;
      mptl_clk_out             : out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n : out   std_logic;
      mptl_target_gtp_online_n : out   std_logic;
      mptl_bridge_gtp_online_n : in    std_logic;
      -- MPTL sideband
      gpio_t2b                 : in    std_logic_vector(15 downto 0);
      gpio_b2t                 : out   std_logic_vector(15 downto 0));
    end component;
  begin
    -- MPTL simulation or IP synthesis
    mptl_tgt_ku_axi_i : mptl_tgt_ku_axi_ip
    port map(
      -- Reset in
      async_sys_rst            => sys_rst,                  -- in    std_logic;
      -- AXI4
      aclk                     => aclk,                     -- out   std_logic;
      aresetn                  => aresetn,                  -- out   std_logic;
      -- Direct slave
      ds_axi_awaddr            => ds_axi_awaddr,            -- out   std_logic_vector(63 downto 0);
      ds_axi_awlen             => ds_axi_awlen,             -- out   std_logic_vector(7 downto 0);
      ds_axi_awsize            => ds_axi_awsize,            -- out   std_logic_vector(2 downto 0);
      ds_axi_awburst           => ds_axi_awburst,           -- out   std_logic_vector(1 downto 0);
      ds_axi_awcache           => ds_axi_awcache,           -- out   std_logic_vector(3 downto 0);
      ds_axi_awprot            => ds_axi_awprot,            -- out   std_logic_vector(2 downto 0);
      ds_axi_awvalid           => ds_axi_awvalid,           -- out   std_logic;
      ds_axi_wdata             => ds_axi_wdata,             -- out   std_logic_vector(127 downto 0);
      ds_axi_wstrb             => ds_axi_wstrb,             -- out   std_logic_vector(15 downto 0);
      ds_axi_wlast             => ds_axi_wlast,             -- out   std_logic;
      ds_axi_wvalid            => ds_axi_wvalid,            -- out   std_logic;
      ds_axi_bready            => ds_axi_bready,            -- out   std_logic;
      ds_axi_araddr            => ds_axi_araddr,            -- out   std_logic_vector(63 downto 0);
      ds_axi_arlen             => ds_axi_arlen,             -- out   std_logic_vector(7 downto 0);
      ds_axi_arsize            => ds_axi_arsize,            -- out   std_logic_vector(2 downto 0);
      ds_axi_arburst           => ds_axi_arburst,           -- out   std_logic_vector(1 downto 0);
      ds_axi_arcache           => ds_axi_arcache,           -- out   std_logic_vector(3 downto 0);
      ds_axi_arprot            => ds_axi_arprot,            -- out   std_logic_vector(2 downto 0);
      ds_axi_arvalid           => ds_axi_arvalid,           -- out   std_logic;
      ds_axi_rready            => ds_axi_rready,            -- out   std_logic;
      ds_axi_awready           => ds_axi_awready,           -- in    std_logic;
      ds_axi_wready            => ds_axi_wready,            -- in    std_logic;
      ds_axi_bresp             => ds_axi_bresp,             -- in    std_logic_vector(1 downto 0);
      ds_axi_bvalid            => ds_axi_bvalid,            -- in    std_logic;
      ds_axi_arready           => ds_axi_arready,           -- in    std_logic;
      ds_axi_rdata             => ds_axi_rdata,             -- in    std_logic_vector(127 downto 0);
      ds_axi_rresp             => ds_axi_rresp,             -- in    std_logic_vector(1 downto 0);
      ds_axi_rlast             => ds_axi_rlast,             -- in    std_logic;
      ds_axi_rvalid            => ds_axi_rvalid,            -- in    std_logic;
      ds_axi_awlock            => ds_axi_awlock,            -- out   std_logic_vector(0 downto 0);
      ds_axi_awqos             => ds_axi_awqos,             -- out   std_logic_vector(3 downto 0);
      ds_axi_arlock            => ds_axi_arlock,            -- out   std_logic_vector(0 downto 0);
      ds_axi_arqos             => ds_axi_arqos,             -- out   std_logic_vector(3 downto 0);
      ds_axi_awregion          => ds_axi_awregion,          -- out   std_logic_vector(3 downto 0);
      ds_axi_arregion          => ds_axi_arregion,          -- out   std_logic_vector(3 downto 0);
      -- MPTL
      mptl_tx_p                => mptl_t2b_p,               -- out   std_logic_vector(3 downto 0);
      mptl_tx_n                => mptl_t2b_n,               -- out   std_logic_vector(3 downto 0);
      mptl_rx_p                => mptl_b2t_p,               -- in    std_logic_vector(3 downto 0);
      mptl_rx_n                => mptl_b2t_n,               -- in    std_logic_vector(3 downto 0);
      mptl_clk_p               => mptl_clk_p,               -- in    std_logic;
      mptl_clk_n               => mptl_clk_n,               -- in    std_logic;
      mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n => mptl_target_configured_l, -- out   std_logic;
      mptl_target_gtp_online_n => mptl_target_gtp_online_l, -- out   std_logic;
      mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_l, -- in    std_logic;
      -- MPTL sideband
      gpio_t2b                 => mptl_gpio_t2b,            -- in    std_logic_vector(15 downto 0);
      gpio_b2t                 => mptl_gpio_b2t);           -- out   std_logic_vector(15 downto 0));
  end generate;


  mptl_tgt_ku_axi_sim_g : if (sim_axi = true) generate
    component mptl_tgt_ku_axi_sim
    port(
      -- Reset in
      async_sys_rst            : in    std_logic;    
      -- AXI4
      aclk                     : out   std_logic;
      aresetn                  : out   std_logic;
      -- Direct slave
      ds_axi_awaddr            : out   std_logic_vector(63 downto 0);
      ds_axi_awlen             : out   std_logic_vector(7 downto 0);
      ds_axi_awsize            : out   std_logic_vector(2 downto 0);
      ds_axi_awburst           : out   std_logic_vector(1 downto 0);
      ds_axi_awcache           : out   std_logic_vector(3 downto 0);
      ds_axi_awprot            : out   std_logic_vector(2 downto 0);
      ds_axi_awvalid           : out   std_logic;
      ds_axi_wdata             : out   std_logic_vector(127 downto 0);
      ds_axi_wstrb             : out   std_logic_vector(15 downto 0);
      ds_axi_wlast             : out   std_logic;
      ds_axi_wvalid            : out   std_logic;
      ds_axi_bready            : out   std_logic;
      ds_axi_araddr            : out   std_logic_vector(63 downto 0);
      ds_axi_arlen             : out   std_logic_vector(7 downto 0);
      ds_axi_arsize            : out   std_logic_vector(2 downto 0);
      ds_axi_arburst           : out   std_logic_vector(1 downto 0);
      ds_axi_arcache           : out   std_logic_vector(3 downto 0);
      ds_axi_arprot            : out   std_logic_vector(2 downto 0);
      ds_axi_arvalid           : out   std_logic;
      ds_axi_rready            : out   std_logic;
      ds_axi_awready           : in    std_logic;
      ds_axi_wready            : in    std_logic;
      ds_axi_bresp             : in    std_logic_vector(1 downto 0);
      ds_axi_bvalid            : in    std_logic;
      ds_axi_arready           : in    std_logic;
      ds_axi_rdata             : in    std_logic_vector(127 downto 0);
      ds_axi_rresp             : in    std_logic_vector(1 downto 0);
      ds_axi_rlast             : in    std_logic;
      ds_axi_rvalid            : in    std_logic;
      ds_axi_awlock            : out   std_logic_vector(0 downto 0);
      ds_axi_awqos             : out   std_logic_vector(3 downto 0);
      ds_axi_arlock            : out   std_logic_vector(0 downto 0);
      ds_axi_arqos             : out   std_logic_vector(3 downto 0);
      ds_axi_awregion          : out   std_logic_vector(3 downto 0);
      ds_axi_arregion          : out   std_logic_vector(3 downto 0);
      -- MPTL
      mptl_tx_p                : out   std_logic_vector(3 downto 0);
      mptl_tx_n                : out   std_logic_vector(3 downto 0);
      mptl_rx_p                : in    std_logic_vector(3 downto 0);
      mptl_rx_n                : in    std_logic_vector(3 downto 0);
      mptl_clk_p               : in    std_logic;
      mptl_clk_n               : in    std_logic;
      mptl_clk_out             : out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n : out   std_logic;
      mptl_target_gtp_online_n : out   std_logic;
      mptl_bridge_gtp_online_n : in    std_logic;
      -- MPTL sideband
      gpio_t2b                 : in    std_logic_vector(15 downto 0);
      gpio_b2t                 : out   std_logic_vector(15 downto 0));
    end component;
  begin
    -- AXI simulation 
    mptl_tgt_ku_axi_sim_i : mptl_tgt_ku_axi_sim
    port map(
      -- Reset in
      async_sys_rst            => sys_rst,                  -- in    std_logic;
      -- AXI4
      aclk                     => aclk,                     -- out   std_logic;
      aresetn                  => aresetn,                  -- out   std_logic;
      -- Direct slave
      ds_axi_awaddr            => ds_axi_awaddr,            -- out   std_logic_vector(63 downto 0);
      ds_axi_awlen             => ds_axi_awlen,             -- out   std_logic_vector(7 downto 0);
      ds_axi_awsize            => ds_axi_awsize,            -- out   std_logic_vector(2 downto 0);
      ds_axi_awburst           => ds_axi_awburst,           -- out   std_logic_vector(1 downto 0);
      ds_axi_awcache           => ds_axi_awcache,           -- out   std_logic_vector(3 downto 0);
      ds_axi_awprot            => ds_axi_awprot,            -- out   std_logic_vector(2 downto 0);
      ds_axi_awvalid           => ds_axi_awvalid,           -- out   std_logic;
      ds_axi_wdata             => ds_axi_wdata,             -- out   std_logic_vector(127 downto 0);
      ds_axi_wstrb             => ds_axi_wstrb,             -- out   std_logic_vector(15 downto 0);
      ds_axi_wlast             => ds_axi_wlast,             -- out   std_logic;
      ds_axi_wvalid            => ds_axi_wvalid,            -- out   std_logic;
      ds_axi_bready            => ds_axi_bready,            -- out   std_logic;
      ds_axi_araddr            => ds_axi_araddr,            -- out   std_logic_vector(63 downto 0);
      ds_axi_arlen             => ds_axi_arlen,             -- out   std_logic_vector(7 downto 0);
      ds_axi_arsize            => ds_axi_arsize,            -- out   std_logic_vector(2 downto 0);
      ds_axi_arburst           => ds_axi_arburst,           -- out   std_logic_vector(1 downto 0);
      ds_axi_arcache           => ds_axi_arcache,           -- out   std_logic_vector(3 downto 0);
      ds_axi_arprot            => ds_axi_arprot,            -- out   std_logic_vector(2 downto 0);
      ds_axi_arvalid           => ds_axi_arvalid,           -- out   std_logic;
      ds_axi_rready            => ds_axi_rready,            -- out   std_logic;
      ds_axi_awready           => ds_axi_awready,           -- in    std_logic;
      ds_axi_wready            => ds_axi_wready,            -- in    std_logic;
      ds_axi_bresp             => ds_axi_bresp,             -- in    std_logic_vector(1 downto 0);
      ds_axi_bvalid            => ds_axi_bvalid,            -- in    std_logic;
      ds_axi_arready           => ds_axi_arready,           -- in    std_logic;
      ds_axi_rdata             => ds_axi_rdata,             -- in    std_logic_vector(127 downto 0);
      ds_axi_rresp             => ds_axi_rresp,             -- in    std_logic_vector(1 downto 0);
      ds_axi_rlast             => ds_axi_rlast,             -- in    std_logic;
      ds_axi_rvalid            => ds_axi_rvalid,            -- in    std_logic;
      ds_axi_awlock            => ds_axi_awlock,            -- out   std_logic_vector(0 downto 0);
      ds_axi_awqos             => ds_axi_awqos,             -- out   std_logic_vector(3 downto 0);
      ds_axi_arlock            => ds_axi_arlock,            -- out   std_logic_vector(0 downto 0);
      ds_axi_arqos             => ds_axi_arqos,             -- out   std_logic_vector(3 downto 0);
      ds_axi_awregion          => ds_axi_awregion,          -- out   std_logic_vector(3 downto 0);
      ds_axi_arregion          => ds_axi_arregion,          -- out   std_logic_vector(3 downto 0);
      -- MPTL
      mptl_tx_p                => mptl_t2b_p,               -- out   std_logic_vector(3 downto 0);
      mptl_tx_n                => mptl_t2b_n,               -- out   std_logic_vector(3 downto 0);
      mptl_rx_p                => mptl_b2t_p,               -- in    std_logic_vector(3 downto 0);
      mptl_rx_n                => mptl_b2t_n,               -- in    std_logic_vector(3 downto 0);
      mptl_clk_p               => mptl_clk_p,               -- in    std_logic;
      mptl_clk_n               => mptl_clk_n,               -- in    std_logic;
      mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n => mptl_target_configured_l, -- out   std_logic;
      mptl_target_gtp_online_n => mptl_target_gtp_online_l, -- out   std_logic;
      mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_l, -- in    std_logic;
      -- MPTL sideband
      gpio_t2b                 => mptl_gpio_t2b,            -- in    std_logic_vector(15 downto 0);
      gpio_b2t                 => mptl_gpio_b2t);           -- out   std_logic_vector(15 downto 0));
  end generate;

end;