library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

entity simple is
generic(
  sim_axi : boolean := false);
port(
  -- MPTL
  mptl_clk_p               : in    std_logic;
  mptl_clk_n               : in    std_logic;
  mptl_b2t_p               : in    std_logic_vector(3 downto 0);
  mptl_b2t_n               : in    std_logic_vector(3 downto 0);
  mptl_t2b_p               : out   std_logic_vector(3 downto 0);
  mptl_t2b_n               : out   std_logic_vector(3 downto 0);
  -- MPTL initialisation
  mptl_bridge_gtp_online_l : in    std_logic;
  mptl_target_configured_l : out   std_logic;
  mptl_target_gtp_online_l : out   std_logic);
end entity;

architecture admxrcku1 of simple is

  component mptl_tgt_ku_axi_wrap
  generic(
    sim_axi : boolean := false);
  port(
    -- Reset in
    sys_rst                  : in    std_logic;
    -- AXI4
    aclk                     : out   std_logic;
    aresetn                  : out   std_logic;
    -- Direct slave
    ds_axi_awaddr            : out   std_logic_vector(63 downto 0);
    ds_axi_awlen             : out   std_logic_vector(7 downto 0);
    ds_axi_awsize            : out   std_logic_vector(2 downto 0);
    ds_axi_awburst           : out   std_logic_vector(1 downto 0);
    ds_axi_awcache           : out   std_logic_vector(3 downto 0);
    ds_axi_awprot            : out   std_logic_vector(2 downto 0);
    ds_axi_awvalid           : out   std_logic;
    ds_axi_wdata             : out   std_logic_vector(127 downto 0);
    ds_axi_wstrb             : out   std_logic_vector(15 downto 0);
    ds_axi_wlast             : out   std_logic;
    ds_axi_wvalid            : out   std_logic;
    ds_axi_bready            : out   std_logic;
    ds_axi_araddr            : out   std_logic_vector(63 downto 0);
    ds_axi_arlen             : out   std_logic_vector(7 downto 0);
    ds_axi_arsize            : out   std_logic_vector(2 downto 0);
    ds_axi_arburst           : out   std_logic_vector(1 downto 0);
    ds_axi_arcache           : out   std_logic_vector(3 downto 0);
    ds_axi_arprot            : out   std_logic_vector(2 downto 0);
    ds_axi_arvalid           : out   std_logic;
    ds_axi_rready            : out   std_logic;
    ds_axi_awready           : in    std_logic;
    ds_axi_wready            : in    std_logic;
    ds_axi_bresp             : in    std_logic_vector(1 downto 0);
    ds_axi_bvalid            : in    std_logic;
    ds_axi_arready           : in    std_logic;
    ds_axi_rdata             : in    std_logic_vector(127 downto 0);
    ds_axi_rresp             : in    std_logic_vector(1 downto 0);
    ds_axi_rlast             : in    std_logic;
    ds_axi_rvalid            : in    std_logic;
    ds_axi_awlock            : out   std_logic_vector(0 downto 0);
    ds_axi_awqos             : out   std_logic_vector(3 downto 0);
    ds_axi_arlock            : out   std_logic_vector(0 downto 0);
    ds_axi_arqos             : out   std_logic_vector(3 downto 0);
    ds_axi_awregion          : out   std_logic_vector(3 downto 0);
    ds_axi_arregion          : out   std_logic_vector(3 downto 0);
    -- MPTL
    mptl_t2b_p               : out   std_logic_vector(3 downto 0);
    mptl_t2b_n               : out   std_logic_vector(3 downto 0);
    mptl_b2t_p               : in    std_logic_vector(3 downto 0);
    mptl_b2t_n               : in    std_logic_vector(3 downto 0);
    mptl_clk_p               : in    std_logic;
    mptl_clk_n               : in    std_logic;
    -- MPTL initialisation
    mptl_target_configured_l : out   std_logic;
    mptl_target_gtp_online_l : out   std_logic;
    mptl_bridge_gtp_online_l : in    std_logic;
    -- MPTL sideband
    mptl_gpio_t2b            : in    std_logic_vector(15 downto 0);
    mptl_gpio_b2t            : out   std_logic_vector(15 downto 0));
  end component;

  component axi_dwidth_converter_ip
  port(
    s_axi_aclk     : in    std_logic;
    s_axi_aresetn  : in    std_logic;
    --
    s_axi_awaddr   : in    std_logic_vector(63 downto 0);
    s_axi_awlen    : in    std_logic_vector(7 downto 0);
    s_axi_awsize   : in    std_logic_vector(2 downto 0);
    s_axi_awburst  : in    std_logic_vector(1 downto 0);
    s_axi_awlock   : in    std_logic_vector(0 downto 0);
    s_axi_awcache  : in    std_logic_vector(3 downto 0);
    s_axi_awprot   : in    std_logic_vector(2 downto 0);
    s_axi_awregion : in    std_logic_vector(3 downto 0);
    s_axi_awqos    : in    std_logic_vector(3 downto 0);
    s_axi_awvalid  : in    std_logic;
    s_axi_awready  : out   std_logic;
    s_axi_wdata    : in    std_logic_vector(127 downto 0);
    s_axi_wstrb    : in    std_logic_vector(15 downto 0);
    s_axi_wlast    : in    std_logic;
    s_axi_wvalid   : in    std_logic;
    s_axi_wready   : out   std_logic;
    s_axi_bresp    : out   std_logic_vector(1 downto 0);
    s_axi_bvalid   : out   std_logic;
    s_axi_bready   : in    std_logic;
    s_axi_araddr   : in    std_logic_vector(63 downto 0);
    s_axi_arlen    : in    std_logic_vector(7 downto 0);
    s_axi_arsize   : in    std_logic_vector(2 downto 0);
    s_axi_arburst  : in    std_logic_vector(1 downto 0);
    s_axi_arlock   : in    std_logic_vector(0 downto 0);
    s_axi_arcache  : in    std_logic_vector(3 downto 0);
    s_axi_arprot   : in    std_logic_vector(2 downto 0);
    s_axi_arregion : in    std_logic_vector(3 downto 0);
    s_axi_arqos    : in    std_logic_vector(3 downto 0);
    s_axi_arvalid  : in    std_logic;
    s_axi_arready  : out   std_logic;
    s_axi_rdata    : out   std_logic_vector(127 downto 0);
    s_axi_rresp    : out   std_logic_vector(1 downto 0);
    s_axi_rlast    : out   std_logic;
    s_axi_rvalid   : out   std_logic;
    s_axi_rready   : in    std_logic;
    --
    m_axi_awaddr   : out   std_logic_vector(63 downto 0);
    m_axi_awlen    : out   std_logic_vector(7 downto 0);
    m_axi_awsize   : out   std_logic_vector(2 downto 0);
    m_axi_awburst  : out   std_logic_vector(1 downto 0);
    m_axi_awlock   : out   std_logic_vector(0 downto 0);
    m_axi_awcache  : out   std_logic_vector(3 downto 0);
    m_axi_awprot   : out   std_logic_vector(2 downto 0);
    m_axi_awregion : out   std_logic_vector(3 downto 0);
    m_axi_awqos    : out   std_logic_vector(3 downto 0);
    m_axi_awvalid  : out   std_logic;
    m_axi_awready  : in    std_logic;
    m_axi_wdata    : out   std_logic_vector(31 downto 0);
    m_axi_wstrb    : out   std_logic_vector(3 downto 0);
    m_axi_wlast    : out   std_logic;
    m_axi_wvalid   : out   std_logic;
    m_axi_wready   : in    std_logic;
    m_axi_bresp    : in    std_logic_vector(1 downto 0);
    m_axi_bvalid   : in    std_logic;
    m_axi_bready   : out   std_logic;
    m_axi_araddr   : out   std_logic_vector(63 downto 0);
    m_axi_arlen    : out   std_logic_vector(7 downto 0);
    m_axi_arsize   : out   std_logic_vector(2 downto 0);
    m_axi_arburst  : out   std_logic_vector(1 downto 0);
    m_axi_arlock   : out   std_logic_vector(0 downto 0);
    m_axi_arcache  : out   std_logic_vector(3 downto 0);
    m_axi_arprot   : out   std_logic_vector(2 downto 0);
    m_axi_arregion : out   std_logic_vector(3 downto 0);
    m_axi_arqos    : out   std_logic_vector(3 downto 0);
    m_axi_arvalid  : out   std_logic;
    m_axi_arready  : in    std_logic;
    m_axi_rdata    : in    std_logic_vector(31 downto 0);
    m_axi_rresp    : in    std_logic_vector(1 downto 0);
    m_axi_rlast    : in    std_logic;
    m_axi_rvalid   : in    std_logic;
    m_axi_rready   : out   std_logic);
  end component;

  component axi_protocol_converter_ip
  port(
    aclk           : in    std_logic;
    aresetn        : in    std_logic;
    --
    s_axi_awaddr   : in    std_logic_vector(63 downto 0);
    s_axi_awlen    : in    std_logic_vector(7 downto 0);
    s_axi_awsize   : in    std_logic_vector(2 downto 0);
    s_axi_awburst  : in    std_logic_vector(1 downto 0);
    s_axi_awlock   : in    std_logic_vector(0 downto 0);
    s_axi_awcache  : in    std_logic_vector(3 downto 0);
    s_axi_awprot   : in    std_logic_vector(2 downto 0);
    s_axi_awregion : in    std_logic_vector(3 downto 0);
    s_axi_awqos    : in    std_logic_vector(3 downto 0);
    s_axi_awvalid  : in    std_logic;
    s_axi_awready  : out   std_logic;
    s_axi_wdata    : in    std_logic_vector(31 downto 0);
    s_axi_wstrb    : in    std_logic_vector(3 downto 0);
    s_axi_wlast    : in    std_logic;
    s_axi_wvalid   : in    std_logic;
    s_axi_wready   : out   std_logic;
    s_axi_bresp    : out   std_logic_vector(1 downto 0);
    s_axi_bvalid   : out   std_logic;
    s_axi_bready   : in    std_logic;
    s_axi_araddr   : in    std_logic_vector(63 downto 0);
    s_axi_arlen    : in    std_logic_vector(7 downto 0);
    s_axi_arsize   : in    std_logic_vector(2 downto 0);
    s_axi_arburst  : in    std_logic_vector(1 downto 0);
    s_axi_arlock   : in    std_logic_vector(0 downto 0);
    s_axi_arcache  : in    std_logic_vector(3 downto 0);
    s_axi_arprot   : in    std_logic_vector(2 downto 0);
    s_axi_arregion : in    std_logic_vector(3 downto 0);
    s_axi_arqos    : in    std_logic_vector(3 downto 0);
    s_axi_arvalid  : in    std_logic;
    s_axi_arready  : out   std_logic;
    s_axi_rdata    : out   std_logic_vector(31 downto 0);
    s_axi_rresp    : out   std_logic_vector(1 downto 0);
    s_axi_rlast    : out   std_logic;
    s_axi_rvalid   : out   std_logic;
    s_axi_rready   : in    std_logic;
    --
    m_axi_awaddr   : out   std_logic_vector(63 downto 0);
    m_axi_awprot   : out   std_logic_vector(2 downto 0);
    m_axi_awvalid  : out   std_logic;
    m_axi_awready  : in    std_logic;
    m_axi_wdata    : out   std_logic_vector(31 downto 0);
    m_axi_wstrb    : out   std_logic_vector(3 downto 0);
    m_axi_wvalid   : out   std_logic;
    m_axi_wready   : in    std_logic;
    m_axi_bresp    : in    std_logic_vector(1 downto 0);
    m_axi_bvalid   : in    std_logic;
    m_axi_bready   : out   std_logic;
    m_axi_araddr   : out   std_logic_vector(63 downto 0);
    m_axi_arprot   : out   std_logic_vector(2 downto 0);
    m_axi_arvalid  : out   std_logic;
    m_axi_arready  : in    std_logic;
    m_axi_rdata    : in    std_logic_vector(31 downto 0);
    m_axi_rresp    : in    std_logic_vector(1 downto 0);
    m_axi_rvalid   : in    std_logic;
    m_axi_rready   : out   std_logic);
  end component;

  component axi_bram_ctrl_ip
  port (
    s_axi_aclk    : in    std_logic;
    s_axi_aresetn : in    std_logic;
    --
    s_axi_awaddr  : in    std_logic_vector(11 downto 0);
    s_axi_awprot  : in    std_logic_vector(2 downto 0);
    s_axi_awvalid : in    std_logic;
    s_axi_awready : out   std_logic;
    s_axi_wdata   : in    std_logic_vector(31 downto 0);
    s_axi_wstrb   : in    std_logic_vector(3 downto 0);
    s_axi_wvalid  : in    std_logic;
    s_axi_wready  : out   std_logic;
    s_axi_bresp   : out   std_logic_vector(1 downto 0);
    s_axi_bvalid  : out   std_logic;
    s_axi_bready  : in    std_logic;
    s_axi_araddr  : in    std_logic_vector(11 downto 0);
    s_axi_arprot  : in    std_logic_vector(2 downto 0);
    s_axi_arvalid : in    std_logic;
    s_axi_arready : out   std_logic;
    s_axi_rdata   : out   std_logic_vector(31 downto 0);
    s_axi_rresp   : out   std_logic_vector(1 downto 0);
    s_axi_rvalid  : out   std_logic;
    s_axi_rready  : in    std_logic;
    --
    bram_rst_a    : out   std_logic;
    bram_clk_a    : out   std_logic;
    bram_en_a     : out   std_logic;
    bram_we_a     : out   std_logic_vector(3 downto 0);
    bram_addr_a   : out   std_logic_vector(11 downto 0);
    bram_wrdata_a : out   std_logic_vector(31 downto 0);
    bram_rddata_a : in    std_logic_vector(31 downto 0));
  end component;


  ---------------------
  -- Local constants --
  ---------------------
  constant REG_DATA_WIDTH    : natural := 32;
  constant REG_BE_WIDTH      : natural := 4;
  constant REG_BE_WIDTH_LOG2 : natural := 2;

  constant REG_ADDR_WIDTH    : natural range 1 to 22 := 3; -- Byte address width


  -----------------------------
  -- mptl_tgt_ku_axi signals --
  -----------------------------
  signal aclk           : std_logic;
  signal aresetn        : std_logic;

  signal mptl_gpio_b2t : std_logic_vector(15 downto 0);
  signal mptl_gpio_t2b : std_logic_vector(15 downto 0) := (others => '0');

  signal ds_axi_awaddr   : std_logic_vector(63 downto 0);
  signal ds_axi_awlen    : std_logic_vector(7 downto 0);
  signal ds_axi_awsize   : std_logic_vector(2 downto 0);
  signal ds_axi_awburst  : std_logic_vector(1 downto 0);
  signal ds_axi_awcache  : std_logic_vector(3 downto 0);
  signal ds_axi_awprot   : std_logic_vector(2 downto 0);
  signal ds_axi_awvalid  : std_logic;
  signal ds_axi_wdata    : std_logic_vector(127 downto 0);
  signal ds_axi_wstrb    : std_logic_vector(15 downto 0);
  signal ds_axi_wlast    : std_logic;
  signal ds_axi_wvalid   : std_logic;
  signal ds_axi_bready   : std_logic;
  signal ds_axi_araddr   : std_logic_vector(63 downto 0);
  signal ds_axi_arlen    : std_logic_vector(7 downto 0);
  signal ds_axi_arsize   : std_logic_vector(2 downto 0);
  signal ds_axi_arburst  : std_logic_vector(1 downto 0);
  signal ds_axi_arcache  : std_logic_vector(3 downto 0);
  signal ds_axi_arprot   : std_logic_vector(2 downto 0);
  signal ds_axi_arvalid  : std_logic;
  signal ds_axi_rready   : std_logic;
  signal ds_axi_awready  : std_logic;
  signal ds_axi_wready   : std_logic;
  signal ds_axi_bresp    : std_logic_vector(1 downto 0);
  signal ds_axi_bvalid   : std_logic;
  signal ds_axi_arready  : std_logic;
  signal ds_axi_rdata    : std_logic_vector(127 downto 0);
  signal ds_axi_rresp    : std_logic_vector(1 downto 0);
  signal ds_axi_rlast    : std_logic;
  signal ds_axi_rvalid   : std_logic;
  signal ds_axi_awlock   : std_logic_vector(0 downto 0);
  signal ds_axi_awqos    : std_logic_vector(3 downto 0);
  signal ds_axi_arlock   : std_logic_vector(0 downto 0);
  signal ds_axi_arqos    : std_logic_vector(3 downto 0);
  signal ds_axi_awregion : std_logic_vector(3 downto 0);
  signal ds_axi_arregion : std_logic_vector(3 downto 0);

  ---------------------------
  -- axi_128_to_32 signals --
  ---------------------------
  signal ds_axi32_awaddr   : std_logic_vector(63 downto 0);
  signal ds_axi32_awlen    : std_logic_vector(7 downto 0);
  signal ds_axi32_awsize   : std_logic_vector(2 downto 0);
  signal ds_axi32_awburst  : std_logic_vector(1 downto 0);
  signal ds_axi32_awcache  : std_logic_vector(3 downto 0);
  signal ds_axi32_awprot   : std_logic_vector(2 downto 0);
  signal ds_axi32_awvalid  : std_logic;
  signal ds_axi32_wdata    : std_logic_vector(31 downto 0);
  signal ds_axi32_wstrb    : std_logic_vector(3 downto 0);
  signal ds_axi32_wlast    : std_logic;
  signal ds_axi32_wvalid   : std_logic;
  signal ds_axi32_bready   : std_logic;
  signal ds_axi32_araddr   : std_logic_vector(63 downto 0);
  signal ds_axi32_arlen    : std_logic_vector(7 downto 0);
  signal ds_axi32_arsize   : std_logic_vector(2 downto 0);
  signal ds_axi32_arburst  : std_logic_vector(1 downto 0);
  signal ds_axi32_arcache  : std_logic_vector(3 downto 0);
  signal ds_axi32_arprot   : std_logic_vector(2 downto 0);
  signal ds_axi32_arvalid  : std_logic;
  signal ds_axi32_rready   : std_logic;
  signal ds_axi32_awready  : std_logic;
  signal ds_axi32_wready   : std_logic;
  signal ds_axi32_bresp    : std_logic_vector(1 downto 0);
  signal ds_axi32_bvalid   : std_logic;
  signal ds_axi32_arready  : std_logic;
  signal ds_axi32_rdata    : std_logic_vector(31 downto 0);
  signal ds_axi32_rresp    : std_logic_vector(1 downto 0);
  signal ds_axi32_rlast    : std_logic;
  signal ds_axi32_rvalid   : std_logic;
  signal ds_axi32_awlock   : std_logic_vector(0 downto 0);
  signal ds_axi32_awqos    : std_logic_vector(3 downto 0);
  signal ds_axi32_arlock   : std_logic_vector(0 downto 0);
  signal ds_axi32_arqos    : std_logic_vector(3 downto 0);
  signal ds_axi32_awregion : std_logic_vector(3 downto 0);
  signal ds_axi32_arregion : std_logic_vector(3 downto 0);

  -----------------------------
  -- axi_to_axi_lite signals --
  -----------------------------
  signal ds_axil_awaddr   : std_logic_vector(63 downto 0);
  signal ds_axil_awprot   : std_logic_vector(2 downto 0);
  signal ds_axil_awvalid  : std_logic;
  signal ds_axil_wdata    : std_logic_vector(31 downto 0);
  signal ds_axil_wstrb    : std_logic_vector(3 downto 0);
  signal ds_axil_wvalid   : std_logic;
  signal ds_axil_bready   : std_logic;
  signal ds_axil_araddr   : std_logic_vector(63 downto 0);
  signal ds_axil_arprot   : std_logic_vector(2 downto 0);
  signal ds_axil_arvalid  : std_logic;
  signal ds_axil_rready   : std_logic;
  signal ds_axil_awready  : std_logic;
  signal ds_axil_wready   : std_logic;
  signal ds_axil_bresp    : std_logic_vector(1 downto 0);
  signal ds_axil_bvalid   : std_logic;
  signal ds_axil_arready  : std_logic;
  signal ds_axil_rdata    : std_logic_vector(31 downto 0);
  signal ds_axil_rresp    : std_logic_vector(1 downto 0);
  signal ds_axil_rvalid   : std_logic;

  ---------------------
  -- axi_lite_to_reg --
  ---------------------
  signal bram_clk_a    : std_logic;
  signal bram_rst_a    : std_logic;
  signal bram_rddata_a : std_logic_vector(REG_DATA_WIDTH-1 downto 0);
  signal bram_wrdata_a : std_logic_vector(REG_DATA_WIDTH-1 downto 0);
  signal bram_we_a     : std_logic_vector(REG_BE_WIDTH-1 downto 0);
  signal bram_en_a     : std_logic;
  signal bram_addr_a   : std_logic_vector(11 downto 0); -- Byte address

  ----------------------
  -- Register signals --
  ----------------------
  signal reg_simple_wdata : std_logic_vector(REG_DATA_WIDTH-1 downto 0);
  signal reg_simple_rdata : std_logic_vector(REG_DATA_WIDTH-1 downto 0);

  signal logic0 : std_logic := '0';
  signal logic1 : std_logic := '1';
  
begin


  -----------------------------------
  -- MPTL interface and AXI blocks --
  -----------------------------------

  mptl_tgt_ku_axi_wrap_i : mptl_tgt_ku_axi_wrap
  generic map(
    sim_axi => sim_axi)
  port map(
    -- Reset in
    sys_rst => logic0,                  -- in  std_logic;
    -- AXI4 Interfaces
    aclk    => aclk,                    -- out std_logic;
    aresetn => aresetn,                 -- out std_logic;
    -- Direct Slave
    ds_axi_awaddr   => ds_axi_awaddr,   -- out std_logic_vector(63 downto 0);
    ds_axi_awlen    => ds_axi_awlen,    -- out std_logic_vector(7 downto 0);
    ds_axi_awsize   => ds_axi_awsize,   -- out std_logic_vector(2 downto 0);
    ds_axi_awburst  => ds_axi_awburst,  -- out std_logic_vector(1 downto 0);
    ds_axi_awcache  => ds_axi_awcache,  -- out std_logic_vector(3 downto 0);
    ds_axi_awprot   => ds_axi_awprot,   -- out std_logic_vector(2 downto 0);
    ds_axi_awvalid  => ds_axi_awvalid,  -- out std_logic;
    ds_axi_wdata    => ds_axi_wdata,    -- out std_logic_vector(127 downto 0);
    ds_axi_wstrb    => ds_axi_wstrb,    -- out std_logic_vector(15 downto 0);
    ds_axi_wlast    => ds_axi_wlast,    -- out std_logic;
    ds_axi_wvalid   => ds_axi_wvalid,   -- out std_logic;
    ds_axi_bready   => ds_axi_bready,   -- out std_logic;
    ds_axi_araddr   => ds_axi_araddr,   -- out std_logic_vector(63 downto 0);
    ds_axi_arlen    => ds_axi_arlen,    -- out std_logic_vector(7 downto 0);
    ds_axi_arsize   => ds_axi_arsize,   -- out std_logic_vector(2 downto 0);
    ds_axi_arburst  => ds_axi_arburst,  -- out std_logic_vector(1 downto 0);
    ds_axi_arcache  => ds_axi_arcache,  -- out std_logic_vector(3 downto 0);
    ds_axi_arprot   => ds_axi_arprot,   -- out std_logic_vector(2 downto 0);
    ds_axi_arvalid  => ds_axi_arvalid,  -- out std_logic;
    ds_axi_rready   => ds_axi_rready,   -- out std_logic;
    ds_axi_awready  => ds_axi_awready,  -- in  std_logic;
    ds_axi_wready   => ds_axi_wready,   -- in  std_logic;
    ds_axi_bresp    => ds_axi_bresp,    -- in  std_logic_vector(1 downto 0);
    ds_axi_bvalid   => ds_axi_bvalid,   -- in  std_logic;
    ds_axi_arready  => ds_axi_arready,  -- in  std_logic;
    ds_axi_rdata    => ds_axi_rdata,    -- in  std_logic_vector(127 downto 0);
    ds_axi_rresp    => ds_axi_rresp,    -- in  std_logic_vector(1 downto 0);
    ds_axi_rlast    => ds_axi_rlast,    -- in  std_logic;
    ds_axi_rvalid   => ds_axi_rvalid,   -- in  std_logic;
    ds_axi_awlock   => ds_axi_awlock,   -- out std_logic_vector(0 downto 0);
    ds_axi_awqos    => ds_axi_awqos,    -- out std_logic_vector(3 downto 0);
    ds_axi_arlock   => ds_axi_arlock,   -- out std_logic_vector(0 downto 0);
    ds_axi_arqos    => ds_axi_arqos,    -- out std_logic_vector(3 downto 0);
    ds_axi_awregion => ds_axi_awregion, -- out std_logic_vector(3 downto 0);
    ds_axi_arregion => ds_axi_arregion, -- out std_logic_vector(3 downto 0);
    -- MPTL Interface
    mptl_t2b_p => mptl_t2b_p,                             -- out std_logic_vector(3 downto 0);
    mptl_t2b_n => mptl_t2b_n,                             -- out std_logic_vector(3 downto 0);
    mptl_b2t_p => mptl_b2t_p,                             -- in  std_logic_vector(3 downto 0);
    mptl_b2t_n => mptl_b2t_n,                             -- in  std_logic_vector(3 downto 0);
    mptl_clk_p => mptl_clk_p,                             -- in  std_logic;
    mptl_clk_n => mptl_clk_n,                             -- in  std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_l => mptl_target_configured_l, -- out std_logic;
    mptl_target_gtp_online_l => mptl_target_gtp_online_l, -- out std_logic;
    mptl_bridge_gtp_online_l => mptl_bridge_gtp_online_l, -- in  std_logic;
    -- MPTL128 Sideband Interface
    mptl_gpio_t2b => mptl_gpio_t2b,                       -- in  std_logic_vector(15 downto 0);
    mptl_gpio_b2t => mptl_gpio_b2t);                      -- out std_logic_vector(15 downto 0));


  -- Convert from AXI4 128-bit data to AXI4 32-bit data
  axi_128_to_32_i : axi_dwidth_converter_ip
  port map(
    s_axi_aclk    => aclk,
    s_axi_aresetn => aresetn,
    --
    s_axi_awaddr   => ds_axi_awaddr,
    s_axi_awlen    => ds_axi_awlen,
    s_axi_awsize   => ds_axi_awsize,
    s_axi_awburst  => ds_axi_awburst,
    s_axi_awlock   => ds_axi_awlock,
    s_axi_awcache  => ds_axi_awcache,
    s_axi_awprot   => ds_axi_awprot,
    s_axi_awregion => ds_axi_awregion,
    s_axi_awqos    => ds_axi_awqos,
    s_axi_awvalid  => ds_axi_awvalid,
    s_axi_awready  => ds_axi_awready,
    s_axi_wdata    => ds_axi_wdata,
    s_axi_wstrb    => ds_axi_wstrb,
    s_axi_wlast    => ds_axi_wlast,
    s_axi_wvalid   => ds_axi_wvalid,
    s_axi_wready   => ds_axi_wready,
    s_axi_bresp    => ds_axi_bresp,
    s_axi_bvalid   => ds_axi_bvalid,
    s_axi_bready   => ds_axi_bready,
    s_axi_araddr   => ds_axi_araddr,
    s_axi_arlen    => ds_axi_arlen,
    s_axi_arsize   => ds_axi_arsize,
    s_axi_arburst  => ds_axi_arburst,
    s_axi_arlock   => ds_axi_arlock,
    s_axi_arcache  => ds_axi_arcache,
    s_axi_arprot   => ds_axi_arprot,
    s_axi_arregion => ds_axi_arregion,
    s_axi_arqos    => ds_axi_arqos,
    s_axi_arvalid  => ds_axi_arvalid,
    s_axi_arready  => ds_axi_arready,
    s_axi_rdata    => ds_axi_rdata,
    s_axi_rresp    => ds_axi_rresp,
    s_axi_rlast    => ds_axi_rlast,
    s_axi_rvalid   => ds_axi_rvalid,
    s_axi_rready   => ds_axi_rready,
    --
    m_axi_awaddr   => ds_axi32_awaddr,
    m_axi_awlen    => ds_axi32_awlen,
    m_axi_awsize   => ds_axi32_awsize,
    m_axi_awburst  => ds_axi32_awburst,
    m_axi_awlock   => ds_axi32_awlock,
    m_axi_awcache  => ds_axi32_awcache,
    m_axi_awprot   => ds_axi32_awprot,
    m_axi_awregion => ds_axi32_awregion,
    m_axi_awqos    => ds_axi32_awqos,
    m_axi_awvalid  => ds_axi32_awvalid,
    m_axi_awready  => ds_axi32_awready,
    m_axi_wdata    => ds_axi32_wdata,
    m_axi_wstrb    => ds_axi32_wstrb,
    m_axi_wlast    => ds_axi32_wlast,
    m_axi_wvalid   => ds_axi32_wvalid,
    m_axi_wready   => ds_axi32_wready,
    m_axi_bresp    => ds_axi32_bresp,
    m_axi_bvalid   => ds_axi32_bvalid,
    m_axi_bready   => ds_axi32_bready,
    m_axi_araddr   => ds_axi32_araddr,
    m_axi_arlen    => ds_axi32_arlen,
    m_axi_arsize   => ds_axi32_arsize,
    m_axi_arburst  => ds_axi32_arburst,
    m_axi_arlock   => ds_axi32_arlock,
    m_axi_arcache  => ds_axi32_arcache,
    m_axi_arprot   => ds_axi32_arprot,
    m_axi_arregion => ds_axi32_arregion,
    m_axi_arqos    => ds_axi32_arqos,
    m_axi_arvalid  => ds_axi32_arvalid,
    m_axi_arready  => ds_axi32_arready,
    m_axi_rdata    => ds_axi32_rdata,
    m_axi_rresp    => ds_axi32_rresp,
    m_axi_rlast    => ds_axi32_rlast,
    m_axi_rvalid   => ds_axi32_rvalid,
    m_axi_rready   => ds_axi32_rready);


  -- Convert from AXI4 32-bit data to AXI4-Lite 32-bit data
  axi_32_to_lite_i : axi_protocol_converter_ip
  port map (
    aclk    => aclk,
    aresetn => aresetn,
    --
    s_axi_awaddr   => ds_axi32_awaddr,
    s_axi_awlen    => ds_axi32_awlen,
    s_axi_awsize   => ds_axi32_awsize,
    s_axi_awburst  => ds_axi32_awburst,
    s_axi_awlock   => ds_axi32_awlock,
    s_axi_awcache  => ds_axi32_awcache,
    s_axi_awprot   => ds_axi32_awprot,
    s_axi_awregion => ds_axi32_awregion,
    s_axi_awqos    => ds_axi32_awqos,
    s_axi_awvalid  => ds_axi32_awvalid,
    s_axi_awready  => ds_axi32_awready,
    s_axi_wdata    => ds_axi32_wdata,
    s_axi_wstrb    => ds_axi32_wstrb,
    s_axi_wlast    => ds_axi32_wlast,
    s_axi_wvalid   => ds_axi32_wvalid,
    s_axi_wready   => ds_axi32_wready,
    s_axi_bresp    => ds_axi32_bresp,
    s_axi_bvalid   => ds_axi32_bvalid,
    s_axi_bready   => ds_axi32_bready,
    s_axi_araddr   => ds_axi32_araddr,
    s_axi_arlen    => ds_axi32_arlen,
    s_axi_arsize   => ds_axi32_arsize,
    s_axi_arburst  => ds_axi32_arburst,
    s_axi_arlock   => ds_axi32_arlock,
    s_axi_arcache  => ds_axi32_arcache,
    s_axi_arprot   => ds_axi32_arprot,
    s_axi_arregion => ds_axi32_arregion,
    s_axi_arqos    => ds_axi32_arqos,
    s_axi_arvalid  => ds_axi32_arvalid,
    s_axi_arready  => ds_axi32_arready,
    s_axi_rdata    => ds_axi32_rdata,
    s_axi_rresp    => ds_axi32_rresp,
    s_axi_rlast    => ds_axi32_rlast,
    s_axi_rvalid   => ds_axi32_rvalid,
    s_axi_rready   => ds_axi32_rready,
    --
    m_axi_awaddr  => ds_axil_awaddr,
    m_axi_awprot  => ds_axil_awprot,
    m_axi_awvalid => ds_axil_awvalid,
    m_axi_awready => ds_axil_awready,
    m_axi_wdata   => ds_axil_wdata,
    m_axi_wstrb   => ds_axil_wstrb,
    m_axi_wvalid  => ds_axil_wvalid,
    m_axi_wready  => ds_axil_wready,
    m_axi_bresp   => ds_axil_bresp,
    m_axi_bvalid  => ds_axil_bvalid,
    m_axi_bready  => ds_axil_bready,
    m_axi_araddr  => ds_axil_araddr,
    m_axi_arprot  => ds_axil_arprot,
    m_axi_arvalid => ds_axil_arvalid,
    m_axi_arready => ds_axil_arready,
    m_axi_rdata   => ds_axil_rdata,
    m_axi_rresp   => ds_axil_rresp,
    m_axi_rvalid  => ds_axil_rvalid,
    m_axi_rready  => ds_axil_rready);


  -- Convert AXI4-Lite 32-bit data to parallel interface
  axi_lite_to_reg_i : axi_bram_ctrl_ip
  port map(
    s_axi_aclk    => aclk,
    s_axi_aresetn => aresetn,
    --
    s_axi_awaddr  => ds_axil_awaddr(11 downto 0),
    s_axi_awprot  => ds_axil_awprot,
    s_axi_awvalid => ds_axil_awvalid,
    s_axi_awready => ds_axil_awready,
    s_axi_wdata   => ds_axil_wdata,
    s_axi_wstrb   => ds_axil_wstrb,
    s_axi_wvalid  => ds_axil_wvalid,
    s_axi_wready  => ds_axil_wready,
    s_axi_bresp   => ds_axil_bresp,
    s_axi_bvalid  => ds_axil_bvalid,
    s_axi_bready  => ds_axil_bready,
    s_axi_araddr  => ds_axil_araddr(11 downto 0),
    s_axi_arprot  => ds_axil_arprot,
    s_axi_arvalid => ds_axil_arvalid,
    s_axi_arready => ds_axil_arready,
    s_axi_rdata   => ds_axil_rdata,
    s_axi_rresp   => ds_axil_rresp,
    s_axi_rvalid  => ds_axil_rvalid,
    s_axi_rready  => ds_axil_rready,
    --
    bram_rst_a    => bram_rst_a,
    bram_clk_a    => bram_clk_a,
    bram_en_a     => bram_en_a,
    bram_we_a     => bram_we_a,
    bram_addr_a   => bram_addr_a,
    bram_wrdata_a => bram_wrdata_a,
    bram_rddata_a => bram_rddata_a);

  ---------------
  -- Registers --
  ---------------

  -- bram_addr_a(REG_BE_WIDTH_LOG2-1 downto 0) is unused
  -- bram_we_a bit is active during AXI write when AXI byte enable is also active
  write_regs : process(bram_rst_a, bram_clk_a)
  begin
    if bram_rst_a = '1' then
      reg_simple_wdata <= (others => '0');
    elsif rising_edge(bram_clk_a) then
      if bram_addr_a(REG_ADDR_WIDTH-1 downto REG_BE_WIDTH_LOG2) = "0" then
        for i in 0 to REG_BE_WIDTH-1 loop
          if bram_we_a(i) = '1' then
            reg_simple_wdata(8*(i+1)-1 downto 8*i) <= bram_wrdata_a(8*(i+1)-1 downto 8*i);
          end if;
        end loop;
      end if;
    end if;
  end process;

  -- bram_addr_a(REG_BE_WIDTH_LOG2-1 downto 0) is unused
  -- Read latency is 1 cycle
  read_regs : process(bram_rst_a, bram_clk_a)
  begin
    if bram_rst_a = '1' then
      bram_rddata_a <= (others => '0');
    elsif rising_edge(bram_clk_a) then
      if bram_addr_a(REG_ADDR_WIDTH-1 downto REG_BE_WIDTH_LOG2) = "0" then
        bram_rddata_a <= reg_simple_rdata;
      else
        bram_rddata_a <= (others => '-');
      end if;
    end if;
  end process;


  -- Nibble reverse data
  reg_simple_rdata <= reg_simple_wdata(3  downto 0)  &
                      reg_simple_wdata(7  downto 4)  &
                      reg_simple_wdata(11 downto 8)  &
                      reg_simple_wdata(15 downto 12) &
                      reg_simple_wdata(19 downto 16) &
                      reg_simple_wdata(23 downto 20) &
                      reg_simple_wdata(27 downto 24) &
                      reg_simple_wdata(31 downto 28);

end architecture;
