library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

entity mptl_tgt_ku_ocp_wrap is
generic(
  sim_ocp                  : boolean := false);
port(
  -- OCP
  ocp_ready                : in    std_logic;
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  mptl_ready               : out   std_logic;
  -- MPTL
  mptl_clk_p               : in    std_logic;
  mptl_clk_n               : in    std_logic;
  mptl_b2t_p               : in    std_logic_vector(3 downto 0);
  mptl_b2t_n               : in    std_logic_vector(3 downto 0);
  mptl_t2b_p               : out   std_logic_vector(3 downto 0);
  mptl_t2b_n               : out   std_logic_vector(3 downto 0);
  -- MPTL initialisation
  mptl_bridge_gtp_online_l : in    std_logic;
  mptl_target_configured_l : out   std_logic;
  mptl_target_gtp_online_l : out   std_logic;
  -- MPTL sideband
  mptl_gpio_t2b            : in    std_logic_vector(15 downto 0);
  mptl_gpio_b2t            : out   std_logic_vector(15 downto 0));
end;


architecture synth of mptl_tgt_ku_ocp_wrap is

begin

  mptl_tgt_ku_ocp_syn_g : if (sim_ocp = false) generate
    -- Internal MPTL
    signal mptl_t2b_int_p               : std_logic_vector(3 downto 0);
    signal mptl_t2b_int_n               : std_logic_vector(3 downto 0);
    signal mptl_target_config_n_int     : std_logic;
    signal mptl_target_gtp_online_n_int : std_logic;

    signal ds_m2s_slv : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
    signal ds_s2m_slv : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);

    component mptl_tgt_ku_slv_ip
    port(
      -- OCP
      ocp_ready                : in    std_logic;
      ocp_clk                  : in    std_logic;
      ds_m2s                   : out   std_logic_vector(232 downto 0);
      ds_s2m                   : in    std_logic_vector(139 downto 0);
      mptl_ready               : out   std_logic;
      -- MPTL
      mptl_tx_p                : out   std_logic_vector(3 downto 0);
      mptl_tx_n                : out   std_logic_vector(3 downto 0);
      mptl_rx_p                : in    std_logic_vector(3 downto 0);
      mptl_rx_n                : in    std_logic_vector(3 downto 0);
      mptl_clk_p               : in    std_logic;
      mptl_clk_n               : in    std_logic;
      mptl_clk_out             : out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n : out   std_logic;
      mptl_target_gtp_online_n : out   std_logic;
      mptl_bridge_gtp_online_n : in    std_logic;
      -- MPTL sideband
      gpio_t2b                 : in    std_logic_vector(15 downto 0);
      gpio_b2t                 : out   std_logic_vector(15 downto 0));
    end component;
  begin
    -- MPTL simulation or synthesis IP
    mptl_tgt_ku_slv_i : mptl_tgt_ku_slv_ip
    port map(
      -- OCP
      ocp_ready                => ocp_ready,                    -- in    std_logic
      ocp_clk                  => ocp_clk,                      -- in    std_logic
      ds_m2s                   => ds_m2s_slv,                   -- out   std_logic_vector(232 downto 0)
      ds_s2m                   => ds_s2m_slv,                   -- in    std_logic_vector(139 downto 0)
      mptl_ready               => mptl_ready,                   -- out   std_logic;
      -- MPTL
      mptl_tx_p                => mptl_t2b_int_p,               -- out   std_logic_vector(3 downto 0);
      mptl_tx_n                => mptl_t2b_int_n,               -- out   std_logic_vector(3 downto 0);
      mptl_rx_p                => mptl_b2t_p,                   -- in    std_logic_vector(3 downto 0);
      mptl_rx_n                => mptl_b2t_n,                   -- in    std_logic_vector(3 downto 0);
      mptl_clk_p               => mptl_clk_p,                   -- in    std_logic;
      mptl_clk_n               => mptl_clk_n,                   -- in    std_logic;
      mptl_clk_out             => open,                         -- out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n => mptl_target_config_n_int,     -- out   std_logic;
      mptl_target_gtp_online_n => mptl_target_gtp_online_n_int, -- out   std_logic;
      mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_l,     -- in    std_logic;
      -- MPTL sideband
      gpio_t2b                 => mptl_gpio_t2b,                -- in    std_logic_vector(15 downto 0);
      gpio_b2t                 => mptl_gpio_b2t);               -- out   std_logic_vector(15 downto 0));

    -- Define signals during full MPTL simulation
    direct_slave_m2s <= to_adb3_ocp_m2sT(ds_m2s_slv);
    ds_s2m_slv <= to_std_logic_vector(make_defined_s2m(direct_slave_s2m));

    mptl_t2b_p <= make_defined_slv(mptl_t2b_int_p);
    mptl_t2b_n <= make_defined_slv(mptl_t2b_int_n);

    mptl_target_configured_l <= make_defined_sl(mptl_target_config_n_int,'1');
    mptl_target_gtp_online_l <= make_defined_sl(mptl_target_gtp_online_n_int,'1');
  end generate;


  mptl_tgt_ku_ocp_sim_g : if (sim_ocp = true) generate
    component mptl_tgt_ku_ocp_sim
    port(
      -- OCP
      ocp_ready                : in    std_logic;
      ocp_clk                  : in    std_logic;
      direct_slave_m2s         : out   adb3_ocp_m2sT;
      direct_slave_s2m         : in    adb3_ocp_s2mT;
      mptl_ready               : out   std_logic;
      -- MPTL
      mptl_tx_p                : out   std_logic_vector(3 downto 0);
      mptl_tx_n                : out   std_logic_vector(3 downto 0);
      mptl_rx_p                : in    std_logic_vector(3 downto 0);
      mptl_rx_n                : in    std_logic_vector(3 downto 0);
      mptl_clk_p               : in    std_logic;
      mptl_clk_n               : in    std_logic;
      mptl_clk_out             : out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n : out   std_logic;
      mptl_target_gtp_online_n : out   std_logic;
      mptl_bridge_gtp_online_n : in    std_logic;
      -- MPTL sideband
      gpio_t2b                 : in    std_logic_vector(15 downto 0);
      gpio_b2t                 : out   std_logic_vector(15 downto 0));
    end component;
  begin
    -- OCP simulation 
    mptl_tgt_ku_ocp_sim_i : mptl_tgt_ku_ocp_sim
    port map(
      -- OCP
      ocp_ready                => ocp_ready,                -- in    std_logic;
      ocp_clk                  => ocp_clk,                  -- in    std_logic
      direct_slave_m2s         => direct_slave_m2s,         -- out   adb3_ocp_m2sT
      direct_slave_s2m         => direct_slave_s2m,         -- in    adb3_ocp_s2mT
      mptl_ready               => mptl_ready,               -- out   std_logic;
      -- MPTL
      mptl_tx_p                => mptl_t2b_p,               -- out   std_logic_vector(3 downto 0);
      mptl_tx_n                => mptl_t2b_n,               -- out   std_logic_vector(3 downto 0);
      mptl_rx_p                => mptl_b2t_p,               -- in    std_logic_vector(3 downto 0);
      mptl_rx_n                => mptl_b2t_n,               -- in    std_logic_vector(3 downto 0);
      mptl_clk_p               => mptl_clk_p,               -- in    std_logic;
      mptl_clk_n               => mptl_clk_n,               -- in    std_logic;
      mptl_clk_out             => open,                     -- out   std_logic;
      -- MPTL initialisation
      mptl_target_configured_n => mptl_target_configured_l, -- out   std_logic;
      mptl_target_gtp_online_n => mptl_target_gtp_online_l, -- out   std_logic;
      mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_l, -- in    std_logic;
      -- MPTL sideband
      gpio_t2b                 => mptl_gpio_t2b,            -- in    std_logic_vector(15 downto 0);
      gpio_b2t                 => mptl_gpio_b2t);           -- out   std_logic_vector(15 downto 0));
  end generate;

end;