--
-- This package defines parts for the K4A8G family, currently:
--
-- K4A8G045WB-??RC (DDR4-2400 17-17-17 2Gx4)
-- K4A8G085WB-??RC (DDR4-2400 17-17-17 1Gx8)
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.ddr4_sdram_model.all;

package ddr4_sdram_k4a8g is

  constant PINOUT_K4A8G_8G_X4 : pinout_t := (
    stack_type     => stack_none,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 1,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => false,
    tdqs_supported => false,
    row_width      => 17,
    col_width      => 10
  );

  constant PINOUT_K4A8G_8G_X8 : pinout_t := (
    stack_type     => stack_none,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 2,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => true,
    tdqs_supported => true,
    row_width      => 16,
    col_width      => 10
  );

  
  constant TIMING_K4A8G_RC : timing_t := (
    power_on => (
      t_reset_hold    => 700 us,
      t_cke_reset_off => 10 ns
    ),
    reset => (
      t_reset_hold    => 100 ns,
      t_cke_reset_off => 10 ns
    ),
    init => (
      t_reset_cke     => 500 us,
      n_ck_cke        => 5,
      t_ck_cke        => 10 ns,
      n_xpr           => 5,
      t_xs_extra      => 10 ns,
      n_dllk          => 768,
      n_zqinit        => 1024
    ),
    operational => (
      n_mrd           => 8,
      t_mrd           => 0 ns,
      t_is            => 162 ps,
      t_ih            => 162 ps,
      t_rcd           => 14.16 ns,
      t_rp            => 14.16 ns
    ),
    refresh => (
      t_rfc           => 350 ns,
      t_refi          => 7.8 us,
      t_refi_ext      => 3.9 us
    )
  );

  -- 8Gib density parts  

  -- K4A8G045WB-??RC (DDR4-2400 17-17-17 2Gx4)
  constant K4A8G045WB_RC : part_t := (
    timing => TIMING_K4A8G_RC,
    pinout => PINOUT_K4A8G_8G_X4
  );

  -- K4A8G085WB-??RC (DDR4-2400 17-17-17 1Gx8)  
  constant K4A8G085WB_RC : part_t := (
    timing => TIMING_K4A8G_RC,
    pinout => PINOUT_K4A8G_8G_X8
  );
  
end package;

package body ddr4_sdram_k4a8g is


end package body;
