library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

entity memory_exerciser_dearranger is
  generic(
    memory_data_width  : in    natural;         -- Logical data width
    pack_factor        : in    natural;         -- Number of physical data words per logical data word
    -- true  => Rearrange data
    -- false => Don't rearrange data
    pack_rearranged    : in    boolean
  );
  port(
    data_in            : in    std_logic_vector(memory_data_width - 1 downto 0);
    data_out           : out   std_logic_vector(memory_data_width - 1 downto 0)
  );
end entity;

architecture rtl of memory_exerciser_dearranger is

  -- This entity performs the inverse of memory_exerciser_rearranger.
  
  -- This function performs the inverse of rearrange_to_mig.
  function rearrange_from_mig(
    constant vec         : in std_logic_vector;
    constant pack_factor : in natural;
    constant lane_width  : in natural)
  return std_logic_vector is
    constant phys_word_width : natural := vec'length / pack_factor;
    constant num_lane_per_word : natural := phys_word_width / lane_width;
    constant lane_group_width : natural := pack_factor * lane_width;
    variable tmp_vec, tmp_rearranged : std_logic_vector(vec'length - 1 downto 0);
    variable returned : std_logic_vector(vec'range);
  begin
    tmp_vec := vec;
    for i in 0 to num_lane_per_word - 1 loop
      for j in 0 to pack_factor - 1 loop
        tmp_rearranged(phys_word_width * j + lane_width * (i + 1) - 1 downto phys_word_width * j + lane_width * i) :=
          tmp_vec(lane_group_width * i + lane_width * (j + 1) - 1 downto lane_group_width * i + lane_width * j);
      end loop;
    end loop;
    returned := tmp_rearranged;
    return returned;
  end;


begin

  data_out <= rearrange_from_mig(data_in, pack_factor, 8) when pack_rearranged else
              data_in;


end architecture;
