--==============================================================================
--
--     File Name: adb3_ocp_dm_adapter_dcfifo.vhd
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: adb3_ocp_dm_adapter
--
--   Description: Submodule of adb3_ocp_dm_adapter responsible for buffering
--                commands to the Direct Master OCP channel.
--
--   Limitations: 
--
--         Notes: 
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, TW, 26 Nov 2013: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_dm_int.all;

entity adb3_ocp_dm_adapter_dcfifo is
generic(
  ORDER           : in    natural);
port(
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  -- Commands from chopper
  chop_cmd        : in    ocp_CmdT;
  chop_addr       : in    adb3_ocp_addr_s;
  chop_blen       : in    adb3_ocp_burst_s;
  chop_dm_tag     : in    adb3_ocp_tag_s;
  chop_cmd_valid  : in    std_logic;
  -- To DM OCP port
  dm_cmd          : out   ocp_CmdT;
  dm_addr         : out   adb3_ocp_addr_s;
  dm_blen         : out   adb3_ocp_burst_s;
  dm_tag          : out   adb3_ocp_tag_s;
  dm_cmd_acc      : in    std_logic;
  -- Flow control
  nafull          : out   std_logic;
  -- Debug
  debug_info      : out   std_logic_vector(11 downto 0));
end;


architecture syn of adb3_ocp_dm_adapter_dcfifo is

  constant EMPTY_LO : natural := ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH + ADB3_OCP_TAG_WIDTH;
  constant EMPTY_HI : natural := EMPTY_LO + ADB3_OCP_CMD_WIDTH;
  constant WIDTH : natural := EMPTY_HI;
  
  signal wd : std_logic_vector(WIDTH - 1 downto 0);
  signal rq : std_logic_vector(WIDTH - 1 downto 0);
  signal radv : std_logic;
  signal rvalid : std_logic;
  
  signal logic0 : std_logic;

  
begin
  
  logic0 <= '0';
  
  wd <= chop_cmd & chop_dm_tag & chop_blen & chop_addr;
         
  U0 : adb3_ocp_dm_adapter_sfifo
  generic map(
    ORDER => ORDER,
    WIDTH => WIDTH,
    EMPTY_LO => EMPTY_LO,
    EMPTY_HI => EMPTY_HI)
  port map(
    rst => ocp_rst,
    sr => logic0,
    clk => ocp_clk,
    wen => chop_cmd_valid,
    wd => wd,
    wafull => open,
    wnafull => nafull,
    radv => radv,
    rvalid => rvalid,
    rq => rq,
    debug_info => debug_info);
  
  radv <= dm_cmd_acc AND rvalid;
  
  dm_addr <= rq(ADB3_OCP_ADDR_WIDTH - 1 downto 0);
  dm_blen <= rq(ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH - 1 downto ADB3_OCP_ADDR_WIDTH);
  dm_tag <= rq(EMPTY_LO - 1 downto ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH);
  dm_cmd <= rq(WIDTH - 1 downto EMPTY_LO);
  
end;
