--
-- Behavioural simulation model of ADM-PCIE-KU3-BCHI, Target FPGA side
--

library ieee;
use ieee.std_logic_1164.all;

package adb3_admxrcku1_bm is

  component adb3_admxrcku1_bm_x8_axi4 is
    generic(
      number_of_dma_engines    : natural range 0 to 4 := 1;
      number_of_pcie_dm_ports  : natural range 0 to 1 := 0;
      number_of_bar01_dm_ports : natural range 0 to 1 := 0;
      dma_engine0_config       : natural range 0 to 4 := 0;
      dma_engine1_config       : natural range 0 to 4 := 0;
      dma_engine2_config       : natural range 0 to 4 := 0;
      dma_engine3_config       : natural range 0 to 4 := 0;
      disable_model_io         : boolean := false;
      use_axi4_pcie_legalizer  : boolean := false;
      bar2_size                : natural range 15 to 21 := 15;
      enable_dma_abort         : boolean := false;
      enable_interrupt         : boolean := false;
      use_icap                 : boolean := true;
      use_diff_ref_clock       : boolean := true;
      pcie_dm_port_id_width    : natural range 1 to 4 := 1
    );
    port(
      -- PCIE
      perst_n                  : in    std_logic;
      pcie100_p                : in    std_logic;
      pcie100_n                : in    std_logic;
      refclk200_p              : in    std_logic := '0';
      refclk200_n              : in    std_logic := '1';
      refclk200_in             : in    std_logic := '0';
      refclk                   : out   std_logic;
      pci_exp_txn              : out   std_logic_vector(7 downto 0);
      pci_exp_txp              : out   std_logic_vector(7 downto 0);
      pci_exp_rxn              : in    std_logic_vector(7 downto 0);
      pci_exp_rxp              : in    std_logic_vector(7 downto 0);
      -- Model-specific I/O pins
      model_inout_i            : in    std_logic_vector(55 downto 0);
      model_inout_o            : out   std_logic_vector(55 downto 0);
      model_inout_t            : out   std_logic_vector(55 downto 0);
      -- VPD transport interface to 2nd core
      vpd_out                  : out   std_logic_vector(47 downto 0);
      
      -- Signals that are replicated between Target and Host via pcie_exp_{txn,txp,rxn,rxp}
      aclk : out std_logic;
      aresetn : out std_logic;
      ds_axi_araddr : out std_logic_vector(63 downto 0);
      ds_axi_arburst : out std_logic_vector(1 downto 0);
      ds_axi_arcache : out std_logic_vector(3 downto 0);
      ds_axi_arlen : out std_logic_vector(7 downto 0);
      ds_axi_arlock : out std_logic_vector(0 downto 0);
      ds_axi_arprot : out std_logic_vector(2 downto 0);
      ds_axi_arqos : out std_logic_vector(3 downto 0);
      ds_axi_arready : in std_logic := '0';
      ds_axi_arregion : out std_logic_vector(3 downto 0);
      ds_axi_arsize : out std_logic_vector(2 downto 0);
      ds_axi_arvalid : out std_logic;
      ds_axi_awaddr : out std_logic_vector(63 downto 0);
      ds_axi_awburst : out std_logic_vector(1 downto 0);
      ds_axi_awcache : out std_logic_vector(3 downto 0);
      ds_axi_awlen : out std_logic_vector(7 downto 0);
      ds_axi_awlock : out std_logic_vector(0 downto 0);
      ds_axi_awprot : out std_logic_vector(2 downto 0);
      ds_axi_awqos : out std_logic_vector(3 downto 0);
      ds_axi_awready : in std_logic := '0';
      ds_axi_awregion : out std_logic_vector(3 downto 0);
      ds_axi_awsize : out std_logic_vector(2 downto 0);
      ds_axi_awvalid : out std_logic;
      ds_axi_rdata : in std_logic_vector(255 downto 0) := (others => '0');
      ds_axi_rlast : in std_logic := '0';
      ds_axi_rready : out std_logic;
      ds_axi_rresp : in std_logic_vector(1 downto 0) := (others => '0');
      ds_axi_rvalid : in std_logic := '0';
      ds_axi_wdata : out std_logic_vector(255 downto 0);
      ds_axi_wlast : out std_logic;
      ds_axi_wready : in std_logic := '0';
      ds_axi_wstrb : out std_logic_vector(31 downto 0);
      ds_axi_wvalid : out std_logic;
      ds_axi_bready : out std_logic;
      ds_axi_bresp : in std_logic_vector(1 downto 0) := (others => '0');
      ds_axi_bvalid : in std_logic := '0';
      dma0_axi_araddr : out std_logic_vector(38 downto 0);
      dma0_axi_arburst : out std_logic_vector(1 downto 0);
      dma0_axi_arcache : out std_logic_vector(3 downto 0);
      dma0_axi_arlen : out std_logic_vector(7 downto 0);
      dma0_axi_arlock : out std_logic_vector(0 downto 0);
      dma0_axi_arprot : out std_logic_vector(2 downto 0);
      dma0_axi_arqos : out std_logic_vector(3 downto 0);
      dma0_axi_arready : in std_logic := '0';
      dma0_axi_arregion : out std_logic_vector(3 downto 0);
      dma0_axi_arsize : out std_logic_vector(2 downto 0);
      dma0_axi_arvalid : out std_logic;
      dma0_axi_awaddr : out std_logic_vector(38 downto 0);
      dma0_axi_awburst : out std_logic_vector(1 downto 0);
      dma0_axi_awcache : out std_logic_vector(3 downto 0);
      dma0_axi_awlen : out std_logic_vector(7 downto 0);
      dma0_axi_awlock : out std_logic_vector(0 downto 0);
      dma0_axi_awprot : out std_logic_vector(2 downto 0);
      dma0_axi_awqos : out std_logic_vector(3 downto 0);
      dma0_axi_awready : in std_logic := '0';
      dma0_axi_awregion : out std_logic_vector(3 downto 0);
      dma0_axi_awsize : out std_logic_vector(2 downto 0);
      dma0_axi_awvalid : out std_logic;
      dma0_axi_rdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma0_axi_rlast : in std_logic := '0';
      dma0_axi_rready : out std_logic;
      dma0_axi_rresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma0_axi_rvalid : in std_logic := '0';
      dma0_axi_wdata : out std_logic_vector(255 downto 0);
      dma0_axi_wstrb : out std_logic_vector(31 downto 0);
      dma0_axi_wlast : out std_logic;
      dma0_axi_wready : in std_logic := '0';
      dma0_axi_wvalid : out std_logic;
      dma0_axi_bready : out std_logic;
      dma0_axi_bresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma0_axi_bvalid : in std_logic := '0';
      dma1_axi_araddr : out std_logic_vector(38 downto 0);
      dma1_axi_arburst : out std_logic_vector(1 downto 0);
      dma1_axi_arcache : out std_logic_vector(3 downto 0);
      dma1_axi_arlen : out std_logic_vector(7 downto 0);
      dma1_axi_arlock : out std_logic_vector(0 downto 0);
      dma1_axi_arprot : out std_logic_vector(2 downto 0);
      dma1_axi_arqos : out std_logic_vector(3 downto 0);
      dma1_axi_arready : in std_logic := '0';
      dma1_axi_arregion : out std_logic_vector(3 downto 0);
      dma1_axi_arsize : out std_logic_vector(2 downto 0);
      dma1_axi_arvalid : out std_logic;
      dma1_axi_awaddr : out std_logic_vector(38 downto 0);
      dma1_axi_awburst : out std_logic_vector(1 downto 0);
      dma1_axi_awcache : out std_logic_vector(3 downto 0);
      dma1_axi_awlen : out std_logic_vector(7 downto 0);
      dma1_axi_awlock : out std_logic_vector(0 downto 0);
      dma1_axi_awprot : out std_logic_vector(2 downto 0);
      dma1_axi_awqos : out std_logic_vector(3 downto 0);
      dma1_axi_awready : in std_logic := '0';
      dma1_axi_awregion : out std_logic_vector(3 downto 0);
      dma1_axi_awsize : out std_logic_vector(2 downto 0);
      dma1_axi_awvalid : out std_logic;
      dma1_axi_rdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma1_axi_rlast : in std_logic := '0';
      dma1_axi_rready : out std_logic;
      dma1_axi_rresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma1_axi_rvalid : in std_logic := '0';
      dma1_axi_wdata : out std_logic_vector(255 downto 0);
      dma1_axi_wstrb : out std_logic_vector(31 downto 0);
      dma1_axi_wlast : out std_logic;
      dma1_axi_wready : in std_logic := '0';
      dma1_axi_wvalid : out std_logic;
      dma1_axi_bready : out std_logic;
      dma1_axi_bresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma1_axi_bvalid : in std_logic := '0';
      dma2_axi_araddr : out std_logic_vector(38 downto 0);
      dma2_axi_arburst : out std_logic_vector(1 downto 0);
      dma2_axi_arcache : out std_logic_vector(3 downto 0);
      dma2_axi_arlen : out std_logic_vector(7 downto 0);
      dma2_axi_arlock : out std_logic_vector(0 downto 0);
      dma2_axi_arprot : out std_logic_vector(2 downto 0);
      dma2_axi_arqos : out std_logic_vector(3 downto 0);
      dma2_axi_arready : in std_logic := '0';
      dma2_axi_arregion : out std_logic_vector(3 downto 0);
      dma2_axi_arsize : out std_logic_vector(2 downto 0);
      dma2_axi_arvalid : out std_logic;
      dma2_axi_awaddr : out std_logic_vector(38 downto 0);
      dma2_axi_awburst : out std_logic_vector(1 downto 0);
      dma2_axi_awcache : out std_logic_vector(3 downto 0);
      dma2_axi_awlen : out std_logic_vector(7 downto 0);
      dma2_axi_awlock : out std_logic_vector(0 downto 0);
      dma2_axi_awprot : out std_logic_vector(2 downto 0);
      dma2_axi_awqos : out std_logic_vector(3 downto 0);
      dma2_axi_awready : in std_logic := '0';
      dma2_axi_awregion : out std_logic_vector(3 downto 0);
      dma2_axi_awsize : out std_logic_vector(2 downto 0);
      dma2_axi_awvalid : out std_logic;
      dma2_axi_rdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma2_axi_rlast : in std_logic := '0';
      dma2_axi_rready : out std_logic;
      dma2_axi_rresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma2_axi_rvalid : in std_logic := '0';
      dma2_axi_wdata : out std_logic_vector(255 downto 0);
      dma2_axi_wstrb : out std_logic_vector(31 downto 0);
      dma2_axi_wlast : out std_logic;
      dma2_axi_wready : in std_logic := '0';
      dma2_axi_wvalid : out std_logic;
      dma2_axi_bready : out std_logic;
      dma2_axi_bresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma2_axi_bvalid : in std_logic := '0';
      dma3_axi_araddr : out std_logic_vector(38 downto 0);
      dma3_axi_arburst : out std_logic_vector(1 downto 0);
      dma3_axi_arcache : out std_logic_vector(3 downto 0);
      dma3_axi_arlen : out std_logic_vector(7 downto 0);
      dma3_axi_arlock : out std_logic_vector(0 downto 0);
      dma3_axi_arprot : out std_logic_vector(2 downto 0);
      dma3_axi_arqos : out std_logic_vector(3 downto 0);
      dma3_axi_arready : in std_logic := '0';
      dma3_axi_arregion : out std_logic_vector(3 downto 0);
      dma3_axi_arsize : out std_logic_vector(2 downto 0);
      dma3_axi_arvalid : out std_logic;
      dma3_axi_awaddr : out std_logic_vector(38 downto 0);
      dma3_axi_awburst : out std_logic_vector(1 downto 0);
      dma3_axi_awcache : out std_logic_vector(3 downto 0);
      dma3_axi_awlen : out std_logic_vector(7 downto 0);
      dma3_axi_awlock : out std_logic_vector(0 downto 0);
      dma3_axi_awprot : out std_logic_vector(2 downto 0);
      dma3_axi_awqos : out std_logic_vector(3 downto 0);
      dma3_axi_awready : in std_logic := '0';
      dma3_axi_awregion : out std_logic_vector(3 downto 0);
      dma3_axi_awsize : out std_logic_vector(2 downto 0);
      dma3_axi_awvalid : out std_logic;
      dma3_axi_rdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma3_axi_rlast : in std_logic := '0';
      dma3_axi_rready : out std_logic;
      dma3_axi_rresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma3_axi_rvalid : in std_logic := '0';
      dma3_axi_wdata : out std_logic_vector(255 downto 0);
      dma3_axi_wstrb : out std_logic_vector(31 downto 0);
      dma3_axi_wlast : out std_logic;
      dma3_axi_wready : in std_logic := '0';
      dma3_axi_wvalid : out std_logic;
      dma3_axi_bready : out std_logic;
      dma3_axi_bresp : in std_logic_vector(1 downto 0) := (others => '0');
      dma3_axi_bvalid : in std_logic := '0';
      dm_axi_araddr : in std_logic_vector(63 downto 0) := (others => '0');
      dm_axi_arburst : in std_logic_vector(1 downto 0) := (others => '0');
      dm_axi_arcache : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_arid : in std_logic_vector(pcie_dm_port_id_width-1 downto 0) := (others => '0');
      dm_axi_arlen : in std_logic_vector(7 downto 0) := (others => '0');
      dm_axi_arlock : in std_logic_vector(0 downto 0) := (others => '0');
      dm_axi_arprot : in std_logic_vector(2 downto 0) := (others => '0');
      dm_axi_arqos : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_arready : out std_logic;
      dm_axi_arregion : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_arsize : in std_logic_vector(2 downto 0) := (others => '0');
      dm_axi_arvalid : in std_logic := '0';
      dm_axi_awaddr : in std_logic_vector(63 downto 0) := (others => '0');
      dm_axi_awburst : in std_logic_vector(1 downto 0) := (others => '0');
      dm_axi_awcache : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_awid : in std_logic_vector(pcie_dm_port_id_width-1 downto 0) := (others => '0');
      dm_axi_awlen : in std_logic_vector(7 downto 0) := (others => '0');
      dm_axi_awlock : in std_logic_vector(0 downto 0) := (others => '0');
      dm_axi_awprot : in std_logic_vector(2 downto 0) := (others => '0');
      dm_axi_awqos : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_awready : out std_logic;
      dm_axi_awregion : in std_logic_vector(3 downto 0) := (others => '0');
      dm_axi_awsize : in std_logic_vector(2 downto 0) := (others => '0');
      dm_axi_awvalid : in std_logic := '0';
      dm_axi_rdata : out std_logic_vector(255 downto 0);
      dm_axi_rid : out std_logic_vector(pcie_dm_port_id_width-1 downto 0);
      dm_axi_rlast : out std_logic;
      dm_axi_rready : in std_logic := '0';
      dm_axi_rresp : out std_logic_vector(1 downto 0);
      dm_axi_rvalid : out std_logic;
      dm_axi_wdata : in std_logic_vector(255 downto 0) := (others => '0');
      dm_axi_wlast : in std_logic := '0';
      dm_axi_wready : out std_logic;
      dm_axi_wstrb : in std_logic_vector(31 downto 0) := (others => '0');
      dm_axi_wvalid : in std_logic := '0';
      dm_axi_bid : out std_logic_vector(pcie_dm_port_id_width-1 downto 0);
      dm_axi_bready : in std_logic := '0';
      dm_axi_bresp : out std_logic_vector(1 downto 0);
      dm_axi_bvalid : out std_logic;
      dmb_axi_araddr : in std_logic_vector(12 downto 0) := (others => '0');
      dmb_axi_arready : out std_logic;
      dmb_axi_arvalid : in std_logic := '0';
      dmb_axi_awaddr : in std_logic_vector(12 downto 0) := (others => '0');
      dmb_axi_awready : out std_logic;
      dmb_axi_awvalid : in std_logic := '0';
      dmb_axi_rdata : out std_logic_vector(31 downto 0);
      dmb_axi_rready : in std_logic := '0';
      dmb_axi_rresp : out std_logic_vector(1 downto 0);
      dmb_axi_rvalid : out std_logic;
      dmb_axi_wdata : in std_logic_vector(31 downto 0) := (others => '0');
      dmb_axi_wready : out std_logic;
      dmb_axi_wstrb : in std_logic_vector(3 downto 0) := (others => '0');
      dmb_axi_wvalid : in std_logic := '0';
      dmb_axi_bready : in std_logic := '0';
      dmb_axi_bresp : out std_logic_vector(1 downto 0);
      dmb_axi_bvalid : out std_logic;
      dma0_m_axis_tdata : out std_logic_vector(255 downto 0);
      dma0_m_axis_tready : in std_logic := '0';
      dma0_m_axis_tvalid : out std_logic;
      dma0_s_axis_tdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma0_s_axis_tready : out std_logic;
      dma0_s_axis_tvalid : in std_logic := '0';
      dma1_m_axis_tdata : out std_logic_vector(255 downto 0);
      dma1_m_axis_tready : in std_logic := '0';
      dma1_m_axis_tvalid : out std_logic;
      dma1_s_axis_tdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma1_s_axis_tready : out std_logic;
      dma1_s_axis_tvalid : in std_logic := '0';
      dma2_m_axis_tdata : out std_logic_vector(255 downto 0);
      dma2_m_axis_tready : in std_logic := '0';
      dma2_m_axis_tvalid : out std_logic;
      dma2_s_axis_tdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma2_s_axis_tready : out std_logic;
      dma2_s_axis_tvalid : in std_logic := '0';
      dma3_m_axis_tdata : out std_logic_vector(255 downto 0);
      dma3_m_axis_tready : in std_logic := '0';
      dma3_m_axis_tvalid : out std_logic;
      dma3_s_axis_tdata : in std_logic_vector(255 downto 0) := (others => '0');
      dma3_s_axis_tready : out std_logic;
      dma3_s_axis_tvalid : in std_logic := '0';
      interrupt : in std_logic := '0';
      dma0_abort : in std_logic := '0';
      dma1_abort : in std_logic := '0';
      dma2_abort : in std_logic := '0';
      dma3_abort : in std_logic := '0';
      core_status : out std_logic_vector(63 downto 0)
    );
  end component;

end package;

package body adb3_admxrcku1_bm is


end package body;
