--------------------------------------------------------------------------------
--
-- Module Name      : adb3_dma_engine
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : 
--
--
-- Dependencies     : adb3_ocp, adb3_dma
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_dma.all;

library unisim;
use unisim.vcomponents.all;

entity adb3_dma_engine256 is

  generic (
    registering_for_speed : boolean := true;
    fpga_descriptors      : boolean := true;
    fpga_to_pci_only      : boolean := false;
    pci_to_fpga_only      : boolean := false;
    read_worker_chipscope : boolean := false);  
  port (
    clk          : in  std_logic;
    irq          : out std_logic;
    pcie_ocp_m2s : out adb3w_ocp_m2sT;
    pcie_ocp_s2m : in  adb3w_ocp_s2mT;
    mptl_ocp_m2s : out adb3w_ocp_m2sT;
    mptl_ocp_s2m : in  adb3w_ocp_s2mT;
    control      : in  adb3_dma_control_register_type;
    status       : out adb3_dma_status_register_type);

end adb3_dma_engine256;

architecture synth of adb3_dma_engine256 is

  signal abort    : std_logic := '0';
  signal cleanup  : std_logic := '0';
  signal abort0   : std_logic := '0';
  signal cleanup0 : std_logic := '0';
  signal abort1   : std_logic := '0';
  signal cleanup1 : std_logic := '0';
  signal abort2   : std_logic := '0';
  signal cleanup2 : std_logic := '0';
  signal abort3   : std_logic := '0';
  signal cleanup3 : std_logic := '0';

  signal abort_df1   : std_logic := '0';
  signal cleanup_df1 : std_logic := '0';
  signal abort_df2   : std_logic := '0';
  signal cleanup_df2 : std_logic := '0';
  signal abort_ww1   : std_logic := '0';
  signal cleanup_ww1 : std_logic := '0';
  signal abort_ww2   : std_logic := '0';
  signal cleanup_ww2 : std_logic := '0';
  signal abort_rw1   : std_logic := '0';
  signal cleanup_rw1 : std_logic := '0';
  signal abort_rw2   : std_logic := '0';
  signal cleanup_rw2 : std_logic := '0';

  signal cleanup_rf : std_logic := '0';
  signal cleanup_wf : std_logic := '0';

  signal df1_fsm_idle          : std_logic;
  signal df1_fsm_aborting      : std_logic;
  signal df1_fsm_fail          : std_logic;
  signal df1_new_desc_addr     : std_logic_vector(63 downto 0);
  signal df1_new_desc_addr_we  : std_logic     := '0';
  signal df1_new_desc_addr_latch     : std_logic_vector(63 downto 0);
  signal df1_new_desc_latched  : std_logic     := '0';
  signal df1_ocp_m2s           : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal df1_ocp_s2m           : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal df1_pworker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal df1_pworker_cmd_valid : std_logic     := '0';
  signal df1_pworker_ready     : std_logic := '0';
  signal df1_mworker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal df1_mworker_cmd_valid : std_logic     := '0';
  signal df1_mworker_ready     : std_logic := '0';
  signal df1_irq               : std_logic;
  signal df1_irq_ack           : std_logic     := '0';
  signal df1_waiting           : std_logic;
  signal df1_waiting_ack       : std_logic     := '0';


  signal df2_fsm_idle          : std_logic;
  signal df2_fsm_aborting      : std_logic;
  signal df2_fsm_fail          : std_logic;
  signal df2_new_desc_addr     : std_logic_vector(63 downto 0);
  signal df2_new_desc_addr_we  : std_logic     := '0';
  signal df2_ocp_m2s           : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal df2_ocp_s2m           : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal df2_pworker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal df2_pworker_cmd_valid : std_logic     := '0';
  signal df2_pworker_ready     : std_logic := '0';
  signal df2_mworker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal df2_mworker_cmd_valid : std_logic     := '0';
  signal df2_mworker_ready     : std_logic := '0';
  signal df2_irq               : std_logic;
  signal df2_irq_ack           : std_logic     := '0';
  signal df2_waiting           : std_logic;
  signal df2_waiting_ack       : std_logic     := '0';


  signal ww1_fsm_idle         : std_logic_vector(1 downto 0);
  signal ww1_fsm_aborting     : std_logic_vector(1 downto 0);
  signal ww1_ocp_m2s          : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal ww1_ocp_s2m          : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal ww1_worker_cmd       : adb3_dma_worker_command_type;
  signal ww1_worker_cmd_valid : std_logic     := '0';
  signal ww1_worker_ready     : std_logic;
  signal ww1_max_burst        : std_logic_vector(8 downto 0);
  signal ww1_irq              : std_logic;
  signal ww1_irq_ack          : std_logic     := '0';
  signal ww1_fifo_data        : std_logic_vector(255 downto 0);
  signal ww1_fifo_empty       : std_logic := '1';
  signal ww1_fifo_level       : std_logic_vector(8 downto 0) := (others => '0');
  signal ww1_fifo_radv        : std_logic;


  signal ww2_fsm_idle         : std_logic_vector(1 downto 0);
  signal ww2_fsm_aborting     : std_logic_vector(1 downto 0);
  signal ww2_ocp_m2s          : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal ww2_ocp_s2m          : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal ww2_worker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal ww2_worker_cmd_valid : std_logic     := '0';
  signal ww2_worker_ready     : std_logic;
  signal ww2_max_burst        : std_logic_vector(8 downto 0);
  signal ww2_irq              : std_logic;
  signal ww2_irq_ack          : std_logic     := '0';
  signal ww2_fifo_data        : std_logic_vector(255 downto 0);
  signal ww2_fifo_empty       : std_logic := '1';
  signal ww2_fifo_level       : std_logic_vector(8 downto 0) := (others => '0');
  signal ww2_fifo_radv        : std_logic;


  signal rw1_fsm_idle         : std_logic_vector(2 downto 0);
  signal rw1_fsm_aborting     : std_logic_vector(2 downto 0);
  signal rw1_fsm_error        : std_logic;
  signal rw1_ocp_m2s          : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal rw1_ocp_s2m          : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal rw1_worker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal rw1_worker_cmd_valid : std_logic     := '0';
  signal rw1_worker_ready     : std_logic;
  signal rw1_max_burst        : std_logic_vector(8 downto 0);
  signal rw1_irq              : std_logic;
  signal rw1_irq_ack          : std_logic     := '0';
  signal rw1_fifo_data        : std_logic_vector(255 downto 0);
  signal rw1_fifo_nfull       : std_logic;
  signal rw1_fifo_wadv        : std_logic;

  signal rw1_fifo_data_r1        : std_logic_vector(255 downto 0);
  signal rw1_fifo_wadv_r1        : std_logic := '0';

  signal rw2_fsm_idle         : std_logic_vector(2 downto 0);
  signal rw2_fsm_aborting     : std_logic_vector(2 downto 0);
  signal rw2_fsm_error        : std_logic;
  signal rw2_ocp_m2s          : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal rw2_ocp_s2m          : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal rw2_worker_cmd       : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal rw2_worker_cmd_valid : std_logic     := '0';
  signal rw2_worker_ready     : std_logic;
  signal rw2_max_burst        : std_logic_vector(8 downto 0);
  signal rw2_irq              : std_logic;
  signal rw2_irq_ack          : std_logic     := '0';
  signal rw2_fifo_data        : std_logic_vector(255 downto 0);
  signal rw2_fifo_nfull       : std_logic;
  signal rw2_fifo_wadv        : std_logic;

  signal rw2_fifo_data_r1        : std_logic_vector(255 downto 0);
  signal rw2_fifo_wadv_r1        : std_logic := '0';
  
  signal awf1_fail   : std_logic := '0';
  signal awf1_rotate : std_logic_vector(4 downto 0);
  signal awf1_rotate_we  : std_logic := '0';

  signal arf1_fail       : std_logic := '0';
  signal arf1_rotate     : std_logic_vector(4 downto 0);
  signal arf1_rotate_we  : std_logic := '0';
  
  signal arf1_fifo_data  : std_logic_vector(255 downto 0);
  signal arf1_fifo_empty : std_logic;
  signal arf1_fifo_radv  : std_logic;
  signal arf1_fifo_level : std_logic_vector(8 downto 0);

  signal ignore_first_word : std_logic := '0';

  signal illegal_we : std_logic:= '0';

  signal power_on_counter : std_logic_vector(5 downto 0) := (others => '1');
  signal power_on_idle : std_logic := '0';
  signal clean_up_counter : std_logic_vector(5 downto 0) := (others => '1');
  signal clean_up_idle : std_logic := '0';
  
  
  
  --signal tw_worker_cmd       : adb3_dma_worker_command_type;
  signal tw_worker_cmd_valid : std_logic := '0';
  --signal tr_worker_cmd       : adb3_dma_worker_command_type;
  signal tr_worker_cmd_valid : std_logic := '0';

  signal hw_worker_cmd_valid_r1 : std_logic := '0';
  signal hr_worker_cmd_valid_r1 : std_logic := '0';

--  signal tw_worker_cmd_valid_r1 : std_logic := '0';
--  signal tr_worker_cmd_valid_r1 : std_logic := '0';

  signal irq_ack : std_logic := '0';
  signal irq_ack_r1 : std_logic := '0';
  signal irq_ack_r2 : std_logic := '0';

  signal need_extra_write : std_logic := '0';

  signal abort_irq     : std_logic := '0';
  signal abort_irq_ack : std_logic := '0';

  signal irq_enable  : std_logic_vector(7 downto 0) := (others => '0');
  signal dma_enable  : std_logic                    := '0';
  signal dma_enable1 : std_logic                    := '0';
  signal dma_enable2 : std_logic                    := '0';
  signal dma_enable3 : std_logic                    := '0';
  signal dma_enable4 : std_logic                    := '0';

  signal fsms_idle_reg : std_logic := '1';

  signal df1_alignment : std_logic_vector(4 downto 0);
  signal df1_alignment_ifw : std_logic := '0';
  signal df1_alignment_set : std_logic := '0';

  signal df2_alignment : std_logic_vector(4 downto 0);
  signal df2_alignment_ifw : std_logic := '0';
  signal df2_alignment_set : std_logic := '0';
  
  signal ww1_completed : std_logic := '0';
  signal ww2_completed : std_logic := '0';
  signal ww_completed : std_logic := '0';

  signal df1_debug : std_logic_vector(31 downto 0);

  attribute shreg_extract : string;
  attribute shreg_extract of df1_waiting_ack : signal is "no";
  attribute shreg_extract of cleanup : signal is "no";
  attribute shreg_extract of cleanup0 : signal is "no";
  attribute shreg_extract of cleanup1 : signal is "no";
  attribute shreg_extract of cleanup2 : signal is "no";
  attribute shreg_extract of abort : signal is "no";
  attribute shreg_extract of abort0 : signal is "no";
  attribute shreg_extract of abort1 : signal is "no";
  attribute shreg_extract of abort2 : signal is "no";

  attribute shreg_extract of rw1_fifo_wadv_r1 : signal is "no";
  attribute shreg_extract of rw2_fifo_wadv_r1 : signal is "no";
  attribute shreg_extract of rw1_fifo_data_r1 : signal is "no";
  attribute shreg_extract of rw2_fifo_data_r1 : signal is "no";
  attribute shreg_extract of awf1_rotate_we : signal is "no";
  attribute shreg_extract of status : signal is "no";

  signal enable_prefetch : std_logic := '0';

begin  -- synth


  -- Descriptor Fetch from PCIe side
  descriptor_fetch_1 : entity work.descriptor_fetch256_pre
    generic map (
      registering_for_speed => registering_for_speed)
    port map (
      clk                   => clk,
      abort                 => abort,
      cleanup               => cleanup,
      fsm_idle              => df1_fsm_idle,
      fsm_aborting          => df1_fsm_aborting,
      fsm_fail              => df1_fsm_fail,
      new_desc_addr         => df1_new_desc_addr,
      new_desc_addr_we      => df1_new_desc_addr_we,
      ocp_m2s               => df1_ocp_m2s,
      ocp_s2m               => df1_ocp_s2m,
      pcie_worker_cmd       => df1_pworker_cmd,
      pcie_worker_cmd_valid => df1_pworker_cmd_valid,
      pcie_worker_ready     => df1_pworker_ready,
      mptl_worker_cmd       => df1_mworker_cmd,
      mptl_worker_cmd_valid => df1_mworker_cmd_valid,
      mptl_worker_ready     => df1_mworker_ready,
      irq                   => df1_irq,
      irq_ack               => df1_irq_ack,
      waiting               => df1_waiting,
      waiting_ack           => df1_waiting_ack,
      write_finished        => ww_completed,
      debug_count           => df1_debug,
      alignment             => df1_alignment,
      alignment_ifw         => df1_alignment_ifw,
      alignment_set         => df1_alignment_set,
      enable_prefetch       => enable_prefetch);

  gen_fpga_desc : if fpga_descriptors generate
    -- Descriptor Fetch from FPGA
    descriptor_fetch_2 : entity work.descriptor_fetch256_pre
      generic map (
        registering_for_speed => registering_for_speed)
      port map (
        clk                   => clk,
        abort                 => abort,
        cleanup               => cleanup,
        fsm_idle              => df2_fsm_idle,
        fsm_aborting          => df2_fsm_aborting,
        fsm_fail              => df2_fsm_fail,
        new_desc_addr         => df2_new_desc_addr,
        new_desc_addr_we      => df2_new_desc_addr_we,
        ocp_m2s               => df2_ocp_m2s,
        ocp_s2m               => df2_ocp_s2m,
        pcie_worker_cmd       => df2_pworker_cmd,
        pcie_worker_cmd_valid => df2_pworker_cmd_valid,
        pcie_worker_ready     => df2_pworker_ready,
        mptl_worker_cmd       => df2_mworker_cmd,
        mptl_worker_cmd_valid => df2_mworker_cmd_valid,
        mptl_worker_ready     => df2_mworker_ready,
        irq                   => df2_irq,
        irq_ack               => df2_irq_ack,
        waiting               => df2_waiting,
        waiting_ack           => df2_waiting_ack,
        write_finished        => ww_completed,
        debug_count           => open,
        alignment             => df2_alignment,
        alignment_ifw         => df2_alignment_ifw,
        alignment_set         => df2_alignment_set,
        enable_prefetch       => '0');

  end generate gen_fpga_desc;

  gen_no_fpga_desc : if not fpga_descriptors generate
    df2_fsm_idle     <= '1';
    df2_fsm_aborting <= '0';
    df2_fsm_fail     <= '0';
    df2_irq          <= '0';
    df2_waiting      <= '0';
    df2_pworker_cmd_valid <= '0';
    df2_mworker_cmd_valid <= '0';
    df2_ocp_m2s <= ADB3W_OCP_DEFAULT_M2S;
  end generate gen_no_fpga_desc;

  gen_write_only0 : if not pci_to_fpga_only generate
    -- Write worker from FIFO to PCIe (to host)
    write_worker_1 : entity work.write_worker256
      generic map (
        left_bits             => 26,
        registering_for_speed => registering_for_speed)
      port map (
        clk              => clk,
        abort            => abort,
        cleanup          => cleanup,
        fsm_idle         => ww1_fsm_idle,
        fsm_aborting     => ww1_fsm_aborting,
        ocp_m2s          => ww1_ocp_m2s,
        ocp_s2m          => ww1_ocp_s2m,
        worker_cmd       => ww1_worker_cmd,
        worker_cmd_valid => ww1_worker_cmd_valid,
        worker_ready     => ww1_worker_ready,
        worker_completed => ww1_completed,
        max_burst        => ww1_max_burst,
        irq              => ww1_irq,
        irq_ack          => ww1_irq_ack,
        fifo_data        => ww1_fifo_data,
        fifo_empty       => ww1_fifo_empty,
        fifo_level       => ww1_fifo_level,
        fifo_radv        => ww1_fifo_radv);


    -- DMA from FPGA to Host FIFO
    alignment_write_fifo_1 : entity work.alignment_write_fifo256
      port map (
        clk               => clk,
        data_in           => rw2_fifo_data_r1,
        push              => rw2_fifo_wadv_r1,
        nfull             => rw2_fifo_nfull,
        data_out          => ww1_fifo_data,
        radv              => ww1_fifo_radv,
        empty             => ww1_fifo_empty,
        level             => ww1_fifo_level,
        fail              => awf1_fail,
        cleanup           => cleanup,
        rotate            => awf1_rotate,
        rotate_we         => awf1_rotate_we,
        ignore_first_word => ignore_first_word);
  end generate gen_write_only0;

  gen_not_write_only0 : if pci_to_fpga_only generate
    ww1_fsm_idle <= (others => '1');
    ww1_fsm_aborting <= (others => '0');
    ww1_worker_ready <= '1';
    ww1_irq <= '0';
    rw2_fsm_idle <= (others => '1');
    rw2_fsm_error <= '0';
    rw2_fsm_aborting <= (others => '0');
    rw2_worker_ready <= '1';
    rw2_irq <= '0';
  end generate gen_not_write_only0;

  reg_fifo_wriotes: process (clk) is
  begin  -- process reg_fifo_wriotes
    if rising_edge(clk) then            -- rising clock edge
      rw1_fifo_data_r1 <= rw1_fifo_data;
      rw1_fifo_wadv_r1 <= rw1_fifo_wadv;
      rw2_fifo_data_r1 <= rw2_fifo_data;
      rw2_fifo_wadv_r1 <= rw2_fifo_wadv;
    end if;
  end process reg_fifo_wriotes;
  
  gen_read_only0 : if not fpga_to_pci_only generate

    -- Read Worker from PCIe (host) to FIFO
    read_worker_1 : entity work.read_worker256
      generic map (
        check_tags            => true,
        left_bits             => 26,
        registering_for_speed => registering_for_speed,
        chipscope => read_worker_chipscope )
      port map (
        clk              => clk,
        abort            => abort,
        cleanup          => cleanup,
        fsm_idle         => rw1_fsm_idle,
        fsm_aborting     => rw1_fsm_aborting,
        fsm_error        => rw1_fsm_error,
        ocp_m2s          => rw1_ocp_m2s,
        ocp_s2m          => rw1_ocp_s2m,
        worker_cmd       => rw1_worker_cmd,
        worker_cmd_valid => rw1_worker_cmd_valid,
        worker_ready     => rw1_worker_ready,
        max_burst        => rw1_max_burst,
        need_extra_write => need_extra_write,
        irq              => rw1_irq,
        irq_ack          => rw1_irq_ack,
        fifo_data        => rw1_fifo_data,
        fifo_nfull       => rw1_fifo_nfull,
        fifo_wadv        => rw1_fifo_wadv);

    -- DMA from Host to FPGA FIFO
    alignment_read_fifo_1 : entity work.alignment_read_fifo256
      port map (
        clk               => clk,
        data_in           => rw1_fifo_data_r1,
        push              => rw1_fifo_wadv_r1,
        nfull             => rw1_fifo_nfull,
        data_out          => arf1_fifo_data,
        radv              => arf1_fifo_radv,
        empty             => arf1_fifo_empty,
        fail              => arf1_fail,
        cleanup           => cleanup,
        level             => arf1_fifo_level,
        rotate            => arf1_rotate,
        rotate_we         => arf1_rotate_we,
        ignore_first_word => ignore_first_word);

    -- FIFO read by FPGA writer
    ww2_fifo_data  <= arf1_fifo_data;
    ww2_fifo_empty <= arf1_fifo_empty;
    ww2_fifo_level <= arf1_fifo_level;
    arf1_fifo_radv <= ww2_fifo_radv;

    -- Write worker from FIFO to MPTL (to target FPGA)
    write_worker_2 : entity work.write_worker256
      generic map (
        addr_bits             => 39,
        left_bits             => 26,
        buffer_descriptors    => true,
        registering_for_speed => registering_for_speed)
      port map (
        clk              => clk,
        abort            => abort,
        cleanup          => cleanup,
        fsm_idle         => ww2_fsm_idle,
        fsm_aborting     => ww2_fsm_aborting,
        ocp_m2s          => ww2_ocp_m2s,
        ocp_s2m          => ww2_ocp_s2m,
        worker_cmd       => ww2_worker_cmd,
        worker_cmd_valid => ww2_worker_cmd_valid,
        worker_ready     => ww2_worker_ready,
        worker_completed => ww2_completed,
        max_burst        => ww2_max_burst,
        irq              => ww2_irq,
        irq_ack          => ww2_irq_ack,
        fifo_data        => ww2_fifo_data,
        fifo_empty       => ww2_fifo_empty,
        fifo_level       => ww2_fifo_level,
        fifo_radv        => ww2_fifo_radv);


  end generate gen_read_only0;

  gen_not_read_only0 : if fpga_to_pci_only generate
    ww2_fsm_idle <= (others => '1');
    ww2_fsm_aborting <= (others => '0');
    ww2_worker_ready <= '1';
    ww2_irq <= '0';
    rw1_fsm_idle <= (others => '1');
    rw1_fsm_error <= '0';
    rw1_fsm_aborting <= (others => '0');
    rw1_worker_ready <= '1';
    rw1_irq <= '0';
  end generate gen_not_read_only0;

  gen_write_only1 : if not pci_to_fpga_only generate
    -- Read worker from MPTL (target FPGA) to FIFO 
    read_worker_2 : entity work.read_worker256
      generic map (
        check_tags            => false,
        addr_bits             => 39,
        left_bits             => 26,
        buffer_descriptors    => true,
        registering_for_speed => registering_for_speed)
      port map (
        clk              => clk,
        abort            => abort,
        cleanup          => cleanup,
        fsm_idle         => rw2_fsm_idle,
        fsm_aborting     => rw2_fsm_aborting,
        fsm_error        => rw2_fsm_error,
        ocp_m2s          => rw2_ocp_m2s,
        ocp_s2m          => rw2_ocp_s2m,
        worker_cmd       => rw2_worker_cmd,
        worker_cmd_valid => rw2_worker_cmd_valid,
        worker_ready     => rw2_worker_ready,
        max_burst        => rw2_max_burst,
        need_extra_write => need_extra_write,
        irq              => rw2_irq,
        irq_ack          => rw2_irq_ack,
        fifo_data        => rw2_fifo_data,
        fifo_nfull       => rw2_fifo_nfull,
        fifo_wadv        => rw2_fifo_wadv);
  end generate gen_write_only1;

  -- OCP MUX for PCIe port
  dma_ocp_mux_1 : entity work.dma_ocp_mux256
    port map (
      clk            => clk,
      enable         => dma_enable3,
      external_m2s   => pcie_ocp_m2s,
      external_s2m   => pcie_ocp_s2m,
      desc_fetch_m2s => df1_ocp_m2s,
      desc_fetch_s2m => df1_ocp_s2m,
      reader_m2s     => rw1_ocp_m2s,
      reader_s2m     => rw1_ocp_s2m,
      writer_m2s     => ww1_ocp_m2s,
      writer_s2m     => ww1_ocp_s2m);

  -- OCP MUX for MPTL port
  dma_ocp_mux_2 : entity work.dma_ocp_mux256
    port map (
      clk            => clk,
      enable         => dma_enable4,
      external_m2s   => mptl_ocp_m2s,
      external_s2m   => mptl_ocp_s2m,
      desc_fetch_m2s => df2_ocp_m2s,
      desc_fetch_s2m => df2_ocp_s2m,
      reader_m2s     => rw2_ocp_m2s,
      reader_s2m     => rw2_ocp_s2m,
      writer_m2s     => ww2_ocp_m2s,
      writer_s2m     => ww2_ocp_s2m);


  reg_worker_cmds: process (clk) is
  begin  -- process reg_worker_cmds
    if rising_edge(clk) then            -- rising clock edge
      if df1_pworker_cmd_valid = '1' then
        ww1_worker_cmd <= df1_pworker_cmd;
      elsif df2_pworker_cmd_valid = '1' then
        ww1_worker_cmd <= df2_pworker_cmd;
      end if;
      
      ww1_worker_cmd_valid <= dma_enable1 and (
                                           (not df1_pworker_cmd.read_data and df1_pworker_cmd_valid) or
                                           (not df2_pworker_cmd.read_data and df2_pworker_cmd_valid));
      if df1_pworker_cmd_valid = '1' then
        rw1_worker_cmd <= df1_pworker_cmd;
      elsif df2_pworker_cmd_valid = '1' then
        rw1_worker_cmd <= df2_pworker_cmd;
      end if;
 
      rw1_worker_cmd_valid <= dma_enable1 and (
                                           (df1_pworker_cmd.read_data and df1_pworker_cmd_valid) or
                                           (df2_pworker_cmd.read_data and df2_pworker_cmd_valid));

      df1_pworker_ready <= ww1_worker_ready and rw1_worker_ready;
      df2_pworker_ready <= ww1_worker_ready and rw1_worker_ready;


      if df1_mworker_cmd_valid = '1' then
        ww2_worker_cmd <= df1_mworker_cmd;
      elsif df2_mworker_cmd_valid = '1' then
        ww2_worker_cmd <= df2_mworker_cmd;
      end if;   

      ww2_worker_cmd_valid <= dma_enable2 and (
                                           (not df1_mworker_cmd.read_data and df1_mworker_cmd_valid) or
                                           (not df2_mworker_cmd.read_data and df2_mworker_cmd_valid));
      if df1_mworker_cmd_valid = '1' then
        rw2_worker_cmd <= df1_mworker_cmd;
      elsif df2_mworker_cmd_valid = '1' then
        rw2_worker_cmd <= df2_mworker_cmd;
      end if;

      rw2_worker_cmd_valid <= dma_enable2 and (
                                           (df1_mworker_cmd.read_data and df1_mworker_cmd_valid) or
                                           (df2_mworker_cmd.read_data and df2_mworker_cmd_valid));

      df1_mworker_ready <= ww2_worker_ready and rw2_worker_ready;
      df2_mworker_ready <= ww2_worker_ready and rw2_worker_ready;
    end if;
  end process reg_worker_cmds;



  registers0 : process (clk)
  begin  -- process registers0
    if clk'event and clk = '1' then     -- rising clock edge
      if OR_reduce(power_on_counter) = '1' then
        power_on_counter <= power_on_counter  -1;
        power_on_idle <= AND_reduce(rw2_fsm_idle & rw1_fsm_idle & ww2_fsm_idle & ww1_fsm_idle & df2_fsm_idle & df1_fsm_idle);
      end if;

      if OR_reduce(clean_up_counter) = '1' then
        clean_up_counter <= clean_up_counter  -1;
        clean_up_idle <= AND_reduce(rw2_fsm_idle & rw1_fsm_idle & ww2_fsm_idle & ww1_fsm_idle & df2_fsm_idle & df1_fsm_idle);
      elsif cleanup = '1' then
        clean_up_counter <= (others => '1');
      end if;
      
      status.fsm_idle     <= clean_up_idle & power_on_idle & rw2_fsm_idle & rw1_fsm_idle & ww2_fsm_idle & ww1_fsm_idle & df2_fsm_idle & df1_fsm_idle;
      status.fsm_aborting <= "00" & rw2_fsm_aborting & rw1_fsm_aborting & ww2_fsm_aborting & ww1_fsm_aborting & df2_fsm_aborting & df1_fsm_aborting;
      status.fsm_fail     <= rw2_fsm_error & rw1_fsm_error & df2_fsm_fail & df1_fsm_fail;
      status.illegal_we   <= illegal_we;
      status.irq_status   <= abort_irq & '0' & rw2_irq & rw1_irq & ww2_irq & ww1_irq & df2_irq & df1_irq;
      status.fifo_info    <= EXT(arf1_fail & ww2_fifo_empty & ww2_fifo_level, 16) &
                          EXT(awf1_fail & ww1_fifo_empty & ww1_fifo_level, 16);
      status.df_waiting <= df2_waiting & df1_waiting;
      status.debug(63 downto 32) <= (others => '-');
      status.debug(31 downto 0) <= df1_debug;
      -- Force DMA enables to only fan out to a small number of modules 
      dma_enable  <= control.enable;
      dma_enable1 <= dma_enable;
      dma_enable2 <= dma_enable1;
      dma_enable3 <= dma_enable2 and not cleanup;
      dma_enable4 <= dma_enable3;
      
      abort  <= abort0;
      abort0 <= abort1;
      abort1 <= abort2 or abort3;
      abort2 <= abort3;
      if control.abort = '1' then
        if control.cleanup = '0' then
          abort3     <= '1';
          illegal_we <= '0';
        else
          abort3     <= '0';
          illegal_we <= '1';
        end if;
      else
        abort3 <= '0';
      end if;

      if abort3 = '1' then
        abort_irq <= '1';
      elsif abort_irq_ack = '1' then
        abort_irq <= '0';
      end if;

      cleanup <= cleanup0;
      cleanup0 <= cleanup1;-- or cleanup2 or cleanup3;
      cleanup1 <= cleanup2 or cleanup3;
      cleanup2 <= cleanup3;
      if control.cleanup = '1' then
        if control.abort = '0' then
          cleanup3   <= '1';
          illegal_we <= '0';
        else
          cleanup3   <= '0';
          illegal_we <= '1';
        end if;
      else
        cleanup3 <= '0';
      end if;

      if control.irq_ack_we = '1' then
        illegal_we    <= '0';
        df1_irq_ack   <= control.irq_ack_register(0);
        df2_irq_ack   <= control.irq_ack_register(1);
        ww1_irq_ack   <= control.irq_ack_register(2);
        ww2_irq_ack   <= control.irq_ack_register(3);
        rw1_irq_ack   <= control.irq_ack_register(4);
        rw2_irq_ack   <= control.irq_ack_register(5);
        abort_irq_ack <= control.irq_ack_register(7);
        irq_ack       <= '1';
      else
        df1_irq_ack   <= '0';
        df2_irq_ack   <= '0';
        ww1_irq_ack   <= '0';
        ww2_irq_ack   <= '0';
        rw1_irq_ack   <= '0';
        rw2_irq_ack   <= '0';
        abort_irq_ack <= '0';
        irq_ack       <= '0';
      end if;
      irq_ack_r1 <= irq_ack;
      irq_ack_r2 <= irq_ack_r1;
      irq <= not (irq_ack or irq_ack_r1 or irq_ack_r2) and OR_reduce((abort_irq & '0' &  rw2_irq & rw1_irq & ww2_irq & ww1_irq & df2_irq & df1_irq) and irq_enable);

      df1_waiting_ack <= control.descriptor_host_wait_ack;
      df2_waiting_ack <= control.descriptor_fpga_wait_ack;


      if control.irq_enable_we = '1' then
        irq_enable <= control.irq_enable;
      end if;


      fsms_idle_reg <= AND_reduce(rw2_fsm_idle & rw1_fsm_idle & ww2_fsm_idle & ww1_fsm_idle & df2_fsm_idle & df1_fsm_idle);

      if df1_alignment_set = '1' then
        arf1_rotate  <= df1_alignment;
        need_extra_write  <= OR_reduce(df1_alignment);
        ignore_first_word <= df1_alignment_ifw;
        arf1_rotate_we <= '1';
      elsif df2_alignment_set = '1' then
        arf1_rotate  <= df2_alignment;
        need_extra_write  <= OR_reduce(df2_alignment);
        ignore_first_word <= df2_alignment_ifw;
        arf1_rotate_we <= '1';
      else
        arf1_rotate_we <= '0';
      end if;
      
      awf1_rotate_we <=arf1_rotate_we;
      awf1_rotate       <= arf1_rotate;

      if control.descriptor_host_we = '1' then
        if fsms_idle_reg = '1' then
          illegal_we           <= '0';
          df1_new_desc_addr_we <= control.enable;
          df1_new_desc_addr    <= control.descriptor_host_addr;
        else
          if df1_new_desc_latched = '0' then
            df1_new_desc_latched <= '1';
            df1_new_desc_addr_latch    <= control.descriptor_host_addr;
          else
            -- Can't start descriptor fetch unless all FSM IDLE 
            illegal_we           <= '1';
            df1_new_desc_addr_we <= '0';
          end if; 
        end if;
      else
        if df1_new_desc_latched = '1' and fsms_idle_reg = '1' then
          df1_new_desc_addr <= df1_new_desc_addr_latch;
          df1_new_desc_latched <= '0';
          df1_new_desc_addr_we <= '1';
        else
          df1_new_desc_addr_we <= '0';
        end if;
      end if;
      if abort = '1' then
        df1_new_desc_latched <= '0';
      end if;
      

      if control.descriptor_fpga_we = '1' then
        if fsms_idle_reg = '1' then
          illegal_we           <= '0';
          df2_new_desc_addr_we <= control.enable;
          df2_new_desc_addr    <= control.descriptor_fpga_addr;
        else
          -- Can't start descriptor fetch unless all FSM IDLE 
          illegal_we           <= '1';
          df2_new_desc_addr_we <= '0';
        end if;
      else
        df2_new_desc_addr_we <= '0';
      end if;


      ww1_max_burst <= control.host_write_worker_max_burst(8 downto 0);
      rw1_max_burst <= control.host_read_worker_max_burst(8 downto 0);
      ww2_max_burst <= control.fpga_write_worker_max_burst(8 downto 0);
      rw2_max_burst <= control.fpga_read_worker_max_burst(8 downto 0);

      ww_completed <= ww1_completed or ww2_completed;

      if rw1_max_burst < "000001000" then
        enable_prefetch <= '0';
      else
        enable_prefetch <= '1';
      end if;
       
      
    end if;
  end process registers0;

  
end synth;
