library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.pcie.ALL;
use work.adb3_ocp.ALL;

entity pcie_pkt_counter is
  port(
    clk                 : in  std_logic;
    sr                  : in  std_logic;
    eop_in              : in  std_logic;
    eop_out             : in  std_logic;

    push                : in  std_logic;
    pop                 : in  std_logic;

    pkt_available       : out std_logic := '0');


end pcie_pkt_counter;

architecture synth of pcie_pkt_counter is
  
signal packet_count : std_logic_vector(6 downto 0);
signal pushdel1     : std_logic := '0';
signal pushdel2     : std_logic := '0';
signal eop_in_del1  : std_logic := '0';
signal eop_in_del2  : std_logic := '0';

begin
  process(clk)
  variable pushpop : std_logic_vector(1 downto 0);
  begin
  if(rising_edge(clk))then
    if(sr = '1')then
      packet_count <= (others => '0');
      pushdel1 <= '0';
      pushdel2 <= '0';
      eop_in_del1 <= '0';
      eop_in_del2 <= '0';
      pkt_available <= '0';      
      
    else
      pushdel1 <= push;
      pushdel2 <= pushdel1;
      eop_in_del1 <= eop_in;
      eop_in_del2 <= eop_in_del1;
      
      pushpop := pushdel2 & pop;
      case(pushpop)is
        when "10" =>  if(eop_in_del2 = '1')then --(1)
                        packet_count <= packet_count + '1';
                        pkt_available <= '1';      
                      end if;
        when "01" =>  if(eop_out = '1')then
                        if(packet_count > "0000000")then
                          packet_count <= packet_count - '1';
                        end if;
                        if(packet_count = "0000001")then
                          pkt_available <= '0';      
                        end if;
                      end if;
        when "11" =>  
                      if((eop_in_del2 = '1')and(eop_out='1'))then -- (1)
                        packet_count <= packet_count;
                      elsif(eop_in_del2 = '1')then --(1)
                        packet_count <= packet_count + '1';
                        pkt_available <= '1';      
                      elsif(eop_out = '1')then
                        if(packet_count > "0000000")then
                          packet_count <= packet_count - '1';
                          pkt_available <= '1';      
                        end if;
                        if(packet_count = "0000001")then
                          pkt_available <= '0';      
                        end if;
                      else
                        packet_count <= packet_count;
                      end if;
        when others => packet_count <= packet_count;
      end case;
      end if;
    end if;
  end process;
  
--   process(clk)
--   begin
--     if(rising_edge(clk))then
--       if(packet_count > "0000000")then
--         pkt_available <= '1';
--       else
--         pkt_available <= '0';
--       end if;
--     end if;
--   end process;
  
end synth;