
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

--********************************************************************************************

package uart_pkg is



--********************************************************************************************
--	Constant Definitions
--********************************************************************************************
	-- 8 bits, 0 to 7
--	constant	RXALL_BITS_CNT	:positive := 7;
	-- 8 bits, 0 to 7
--	constant	TXEND_BIT_CNT	:positive := 7;

--RX constants
	constant	RXSTOP_ENABLED	:std_logic := '1';

--TX constants


--Common values
	constant	RS232_DEFAULT	:std_logic := '1';	-- channel idle
	constant	RS232_START	:std_logic := '0';	-- channel active
	constant	RS232_STOP	:std_logic := '1';	-- channel idle


	---- Address defs relative to start of decode area ( 16 max)
	--constant	SER_CNTRL_REG   :std_logic_vector :=X"0";
	--constant	DIVIDER_REG    :std_logic_vector :=X"1";
	--constant	TXFIFO_REG  :std_logic_vector :=X"2";
	--constant	RX_FIFO_REG :std_logic_vector :=X"3";
	--constant	TXSTATUS_REG	:std_logic_vector :=X"4";
	--constant	RXSTATUS_REG	:std_logic_vector :=X"5";


	constant	CLOCK_RATIO 	:positive := 16;

    
  component ser_tx
  generic
  (
    clock_ratio	  : positive  := 16;
    data_size     : natural   := 8;
    idig_wait		  : natural   := 1;
    msb_first     : boolean   := false
  );
  port
  (
    reset		    : in		std_logic;
    tx_ck		    : in		std_logic;
    tx_cke		    : in		std_logic;
    tx_data		  : in		std_logic_vector( data_size - 1 downto 0 );
    tx_enab		  : in		std_logic;
    tx_rdy		  : out	  std_logic;
    tx_ser_out	: out		std_logic
  );
  end component;  

  component ser_rx
  generic
  (
    clock_ratio	  : positive  := 16;
    data_size     : natural   := 8;
    msb_first     : boolean   := false
  );
  port (

      reset		    : in		std_logic;
      rx_ck		    : in		std_logic;		
      rx_cke		    : in		std_logic;	
      rx_enab		  : in		std_logic;
      ser_data_ip	: in		std_logic;
      rx_err		  : out	  std_logic;
      rx_drdy		  : out	  std_logic;
      rx_data		  : out	  std_logic_vector( data_size - 1 downto 0 )
    );
  end component;  

  component common_uart
    generic
    (
      data_size                 : natural   := 8
    );
    port
    (
      rx_ser_data               : in  std_logic;
      tx_ser_data               : out std_logic;
      rst                       : in  std_logic;
      clk                       : in  std_logic;
      reg_divide                : in  std_logic_vector( 15 downto 0 );
      reg_tx_fifo               : in  std_logic_vector( data_size - 1 downto 0 );
      write_tx_fifo             : in  std_logic;
      reg_rx_fifo               : out std_logic_vector( data_size - 1 downto 0 );
      read_rx_fifo              : in  std_logic;
      rx_full                   : out std_logic;
      rx_empty                  : out std_logic;
      tx_full                   : out std_logic;
      tx_empty                  : out std_logic
    );
  end component;

end package;
