--==============================================================================
--
--     File Name: adb3_ocp_dm_adapter_ocfifo.vhd
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: adb3_ocp_dm_adapter
--
--   Description: Submodule of adb3_ocp_dm_adapter responsible for buffering
--                commands from the general OCP port.
--
--   Limitations: 
--
--         Notes: 
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, TW, 26 Nov 2013: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_dm_int.all;

entity adb3_ocp_dm_adapter_ocfifo is
generic(
  ORDER           : in    natural);
port(
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  -- Commands in from general OCP port
  ocp_cmd         : in    ocp_CmdT;
  ocp_addr        : in    adb3_ocp_addr_s;
  ocp_blen        : in    adb3_ocp_burst_s;
  ocp_tag         : in    adb3_ocp_tag_s;
  ocp_cmd_acc     : out   std_logic;
  -- Commands out to Chopper
  ocf_cmd_adv     : in    std_logic;
  ocf_cmd         : out   ocp_CmdT;
  ocf_addr        : out   adb3_ocp_addr_s;
  ocf_blen        : out   adb3_ocp_burst_s;
  ocf_tag         : out   adb3_ocp_tag_s;
  ocf_cmd_valid   : out   std_logic;
  -- Debug
  debug_info      : out   std_logic_vector(11 downto 0));
end;


architecture syn of adb3_ocp_dm_adapter_ocfifo is

  constant WIDTH : natural := ADB3_OCP_CMD_WIDTH + ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH + ADB3_OCP_TAG_WIDTH;
  
  signal wen : std_logic;
  signal wd : std_logic_vector(WIDTH - 1 downto 0);
  signal rq : std_logic_vector(WIDTH - 1 downto 0);
  signal nafull : std_logic;
  
  signal logic0 : std_logic;

  
begin
  
  logic0 <= '0';
  
  wen <= '1' when nafull = '1' AND ocp_cmd /= OCP_CMD_IDLE else '0';
  wd <= ocp_cmd & ocp_tag & ocp_blen & ocp_addr;
  ocp_cmd_acc <= nafull;
         
  U0 : adb3_ocp_dm_adapter_sfifo
  generic map(
    ORDER => ORDER,
    WIDTH => WIDTH)
  port map(
    rst => ocp_rst,
    sr => logic0,
    clk => ocp_clk,
    wen => wen,
    wd => wd,
    wafull => open,
    wnafull => nafull,
    radv => ocf_cmd_adv,
    rvalid => ocf_cmd_valid,
    rq => rq,
    debug_info => debug_info);
  
  ocf_addr <= rq(ADB3_OCP_ADDR_WIDTH - 1 downto 0);
  ocf_blen <= rq(ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH - 1 downto ADB3_OCP_ADDR_WIDTH);
  ocf_tag <= rq(ADB3_OCP_TAG_WIDTH + ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH - 1 downto ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH);
  ocf_cmd <= rq(ADB3_OCP_CMD_WIDTH + ADB3_OCP_TAG_WIDTH + ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH - 1 downto ADB3_OCP_TAG_WIDTH + ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH);
  
end;
