--
-- pcie_pkg.vhd - definitions file for pcie bridge signals in adb3
--
--
-- (c) copyright alpha data ltd. 2009
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.ALL;

package pcie_config is

    
constant cfg_no_masters     : natural := 3;
constant cfg_no_slaves      : natural := 5;
constant cfg_source_size    : natural := 4;
constant cfg_no_32bit_bars  : natural := 2;

constant cfg_max_data_width  : natural := 128;
constant cfg_no_control_bits : natural := 4;

-- -----------------------------------------------------------------------------
-- pcie parameter constants
-- data width of 128 bits + 3 control (SOP,EOP,EMOD)
-- -----------------------------------------------------------------------------
constant cfg_pcie_core_width    : integer := 64;
constant cfg_pcie_addr_width    : integer := 64;
constant pcie_no_lanes          : integer := 8;


-- -----------------------------------------------------------------------------
-- Xilinx PCIe Endpoint constants
-- -----------------------------------------------------------------------------
constant REVISION_ID                   : bit_vector := X"03";
                                       
constant DEVICE_ID                     : bit_vector := X"ADB3";
constant VENDOR_ID                     : bit_vector := X"4144";
constant SUBSYSTEM_ID                  : bit_vector := X"0801";
constant SUBSYSTEM_VENDOR_ID           : bit_vector := X"4144";
                                       
constant DISABLE_LANE_REVERSAL         : boolean    := FALSE;
constant LINK_CAP_MAX_LINK_SPEED       : bit_vector := X"3";
constant LINK_CAP_MAX_LINK_WIDTH       : bit_vector := X"08";
constant LINK_CAP_MAX_LINK_WIDTH_int   : integer    := 4;
constant LTSSM_MAX_LINK_WIDTH          : bit_vector := X"08";
constant USER_CLK_FREQ                 : integer    := 3;  -- 250MHz
constant LINK_STATUS_SLOT_CLOCK_CONFIG : boolean    := TRUE;
constant REF_CLK_FREQ                  : integer    := 0;  -- 100MHz

-------------------------------------------------------------------------------
-- DNA FPGA setting
-------------------------------------------------------------------------------

constant FPGA_DNA_VALUE : std_Logic_vector(0 to 56) := "000000000000000000000000000000000000000000000000000000001";


end package pcie_config;


package body pcie_config is
end package body pcie_config;

