--------------------------------------------------------------------------------
-- Company:         Alpha-Data Parallel Systems
-- Engineer:        Alan Crawford
--
-- Create Date:
-- Design Name:
-- Module Name:     pcie_ocp_slave64
-- Project Name:    adb3
-- Target Device:   V5FX
-- Tool versions:
-- Description:     PCIE to OCP slave port - converts OCP READ/WRITES into PCIe
--                  and translates CPLDs back into OCP
--
-- Dependencies:    PCIe/OCP Packages for signal records
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.pcie.ALL;
use work.adb3_ocp.ALL;
use work.pciegen3_pkg.all;

entity dma_if is
  generic(
    dmChannel           : integer := -1;
    channelCount        : integer := 5;
    dataWidth           : integer := 256
  );
  port(
    clk                 : in  std_logic;
    sr                  : in  std_logic;

    m2s_i               : in  adb3w_ocp_m2sT_array;
    s2m_o               : out adb3w_ocp_s2mT_array;

    rq_b2c              : out rq_b2c_t;
    rq_c2b              : in  rq_c2b_t;

    maxTotalPIF         : in std_logic_vector(7 downto 0);
    maxPIF              : in std_logic_vector(7 downto 0);
    dmaClr              : in std_Logic_vector(7 downto 0);
     status              : out std_Logic_vector(31 downto 0);
    rc_b2c              : out rc_b2c_t;
    rc_c2b              : in  rc_c2b_t
   
  );
end dma_if;

architecture synth of dma_if is

signal  rc_b2c_int          : rc_b2c_t;

signal srcPush              : std_logic_vector(channelCount-1 downto 0);
signal srcReq               : std_logic_vector(channelCount-1 downto 0);
signal srcData              : channelArray := (others => (others => '0'));
signal srcDataToFIFO        : channelArray := (others => (others => '0'));
signal respReq              : std_logic_vector(channelCount-1 downto 0);
signal space_avail              : std_logic_vector(channelCount-1 downto 0);
signal maskpop              : std_logic_vector(channelCount-1 downto 0);
signal srcEmpty              : std_logic_vector(channelCount-1 downto 0);
                            
signal respData             : std_logic_vector(dataWidth+3 downto 0) := (others => '0');
signal dest_tag             : std_logic_vector(3 downto 0) := "0000";

type   readCounts           is array (0 to channelCount-1) of std_logic_vector(7 downto 0);
signal actualPIF            : readCounts;

signal respPush             : std_logic_vector(channelCount-1 downto 0);
signal srcNotFull           : std_logic_vector(channelCount-1 downto 0);
 signal respNotFull          : std_logic_vector(channelCount-1 downto 0);
signal respNotFull_sticky          : std_logic_vector(channelCount-1 downto 0) := (others => '0');
signal cpldCompleteStrobes  : std_logic_vector(15 downto 0);
signal cpldCompleteFlag     : std_logic := '0';
signal delay                : std_logic_vector(15 downto 0):= X"0000";
signal srcPop               : std_logic_vector(channelCount-1 downto 0);
signal srcLast              : std_logic_vector(channelCount-1 downto 0);
signal opData               : std_logic_vector(dataWidth+dataWidth/32+3 downto 0);
signal opData_r1            : std_logic_vector(dataWidth+dataWidth/32+3 downto 0);
signal temp_tdata           : std_logic_vector(dataWidth+dataWidth/32+3 downto 0);
signal tmpOp                : std_logic_vector(channelCount-1 downto 0);
signal totalPIF             : std_logic_vector(7 downto 0) := (others => '0');
signal pausePIF             : std_logic := '0';
signal opPush               : std_logic;
signal opPush_r1            : std_logic;
signal opFull               : std_logic;
signal tmp_tvalid           : std_logic;
signal pip                  : std_logic;
signal totalCycles          : std_logic_vector(15 downto 0) := (others => '0');
signal activeCycles         : std_logic_vector(15 downto 0) := (others => '0');
signal push_select          : std_logic_vector(15 downto 0) := (others => '0');
signal dest_channel         : std_logic_vector(3 downto 0) := X"0";
signal cpldComplete         : std_logic := '0';
signal temp1                : std_logic_vector(7 downto 0);
signal temp2                : std_logic_vector(7 downto 0);
signal temp3                : std_logic_vector(7 downto 0);
signal temp4                : std_logic_vector(7 downto 0);
signal int_seq_no           : std_logic_vector(3 downto 0) := "0001";
signal tempOpFull           : std_logic;
-- signal pre_pargen_data      : std_logic_vector(255 downto 0);
constant chIndexVector      : std_logic_vector(79 downto 0) := "00000000010001000011001000010100110001110100001001010100101101100011010110101111";
signal pipmon               : std_logic := '0';
signal piperror             : std_logic := '0';
signal pop_request          : std_logic;
signal full_shift           : std_logic_vector(5 downto 0);

signal local_rst : std_logic := '1';
signal local_sr1 : std_logic := '1';
signal local_sr : std_logic := '1';

signal dma_rst : std_logic_vector(7 downto 0);
signal local_dma_rst : std_logic_vector(7 downto 0);

begin

  local_rst <= sr;
  process (clk,local_rst) is
  begin  -- process
    if local_rst = '1' then
      local_sr1 <= '1';
    elsif rising_edge(clk) then            -- rising clock edge
      local_sr1 <= '0';
    end if;
  end process;
  
  process (clk) is
  begin  -- process
    if rising_edge(clk) then            -- rising clock edge
      local_sr <= local_sr1;
    end if;
  end process;
  

monitor: process(Clk)

  begin

    if(rising_edge(Clk))then

      if(tmp_tvalid = '0')and(rq_c2b.s_axis_rq_tready='1')then

        if(temp_tdata(dataWidth+dataWidth/32+3) = '1')and(pipmon = '1')then

          piperror <= '1';

        end if;

        if(temp_tdata(dataWidth+dataWidth/32+3) = '0')and(temp_tdata(dataWidth+dataWidth/32+2) = '1')and(pipmon = '0')then

          piperror <= '1';

        end if;
          
        if(temp_tdata(dataWidth+dataWidth/32+3) = '1')and(temp_tdata(dataWidth+dataWidth/32+2) = '0')and(pipmon = '0')then

          pipmon <= '1';

        end if;

        if(temp_tdata(dataWidth+dataWidth/32+2) = '1')and(pipmon = '1')then

          pipmon <= '0';

        end if;

      end if;

    end if;

  end process monitor;


  merger :  dma_merger
  generic map(
    channelCount         => channelCount,
    dataWidth            => dataWidth
    )
   port map(
      Ck                 => Clk,
      srcReq             => srcReq,
      srcData            => srcData,
      srcPop             => srcPop,
      opData             => opData,
      opPush             => opPush,
      opFull             => opFull
   );

  loop_channels : for chIndex in 0 to channelCount-1 generate
    dma_channels: if(chIndex <(channelCount)) and (chIndex /= dmChannel) generate
      dma_rst(chIndex) <= local_rst or dmaClr(chIndex);
      local_dma_rst(chIndex) <= local_sr or dmaClr(chIndex);
      chOCPIF : dma_ocp_if 
      generic map (
              chIndexVector           => chIndexVector(79-chIndex*5 downto 75-chIndex*5),
              dataWidth               => dataWidth
      )
      port map(
        clk                     => Clk,
        sr                      => dma_rst(chIndex),
        us_counter              => "0000000000",
        m2s_i                   => m2s_i(chIndex),
        s2m_o                   => s2m_o(chIndex),
    
        slave_o2p_push          => srcPush(chIndex),
        slave_o2p_data          => srcDataToFIFO(chIndex),
        slave_o2p_notfull       => srcNotFull(chIndex),
        slave_p2o_push          => push_select(chIndex),
--         slave_p2o_tag           => dest_tag,
        slave_p2o_data          => respData,
         slave_p2o_notfull       => respNotFull(chIndex),
        disableReads            => pausePIF,
        cpldComplete            => cpldCompleteStrobes(chIndex),
        actualPIF               => actualPIF(chIndex),
        maxPIF                  => maxPIF
        );
      end generate;
      dm_channel: if(chIndex = dmChannel) generate
      direct_master_chOCPIF : dma_ocp_if 
      generic map (
              chIndexVector           => chIndexVector(79-(channelCount-1)*5 downto 75-(channelCount-1)*5),
              dataWidth               => dataWidth
      )
      port map(
        clk                     => Clk,
        sr                      => local_rst,
        us_counter              => "0000000000",
        m2s_i                   => m2s_i(chIndex),
        s2m_o                   => s2m_o(chIndex),
    
        slave_o2p_push          => srcPush(chIndex),
        slave_o2p_data          => srcDataToFIFO(chIndex),
        slave_o2p_notfull       => srcNotFull(chIndex),
        slave_p2o_push          => push_select(chIndex),
--         slave_p2o_tag           => dest_tag,
        slave_p2o_data          => respData,
         slave_p2o_notfull       => respNotFull(chIndex),
        disableReads            => pausePIF,
        cpldComplete            => cpldCompleteStrobes(chIndex),
        actualPIF               => actualPIF(chIndex),
        maxPIF                  => x"08"
        );
      end generate;
    srcNotFull(chIndex) <= not space_avail(chIndex);  
    maskpop(chIndex) <= srcPop(chIndex) and not srcEmpty(chIndex);
    chReqFIFO : packet_sfifo
    generic map(
          RAM_TYPE    => "AUTO",
          ORDER       => 7,
          WIDTH       => dataWidth+dataWidth/32+4,
          AFULL_LEVEL => 100
    )
    port map(
      rst         => dma_rst(chIndex),
      sr          => '0', --sr,
      clk         => clk,
      wen         => srcPush(chIndex),
      wd          => srcDataToFIFO(chIndex),
      wfull       => open,
      wafull      =>  space_avail(chIndex),
      radv        => maskpop(chIndex),
      rq          => srcData(chIndex), 
      rlevel      => open,
      rempty      => srcEmpty(chIndex),
      debug_info  => open);
  
      
    srcReq(chIndex) <= tmpOp(chIndex)    ;

    test : pcie_pkt_counter 
      port map(
        clk            => clk, 
        sr             => local_dma_rst(chIndex),
        eop_in         => srcDataToFIFO(chIndex)(dataWidth+dataWidth/32+2),
        eop_out        => srcData(chIndex)(dataWidth+dataWidth/32+3),
        push           => srcPush(chIndex),
        pop            => srcPop(chIndex),
        pkt_available  => tmpOp(chIndex)
      );
  end generate;

  pop_request <= rq_c2b.s_axis_rq_tready and not tmp_tvalid;
  
  coreFIFO : packet_sfifo
    generic map(
          RAM_TYPE    => "AUTO",
          ORDER       => 7,
          WIDTH       => dataWidth+dataWidth/32+4,
          AFULL_LEVEL => 120
    )
    port map(
      rst         => local_rst,
      sr          => '0', --sr,
      clk         => clk,
      wen         => opPush_r1,
      wd          => opData_r1,
      wfull       => open,
      wafull      => tempOpFull,
      radv        => pop_request,
      rq          => temp_tdata, 
      rlevel      => open,
      rempty      => tmp_tvalid,
      debug_info  => open);
  
   process(clk)
   begin
    if(rising_edge(clk))then
      full_shift <= full_shift(4 downto 0) & tempOpFull;
    end if;
  end process;   
  process(full_shift)
  begin      
    if(full_shift /= "000000")then
      opFull <= '1';
    else
      opFull <= '0';
    end if;
  end process; 
      
--    opFull <= tempOpFull;
   
  process(Clk)
  begin
    if(rising_edge(Clk))then
      opPush_r1 <= opPush;
      opData_r1 <= oPData;
      
      totalCycles <= totalCycles+'1';
      if(opPush_r1 ='1')then
        activeCycles <= activeCycles+'1';
        if(opData_r1(dataWidth+dataWidth/32+3) = '0')or(opData_r1(dataWidth+dataWidth/32+2) = '0')then
          if(opData_r1(dataWidth+dataWidth/32+3) = '1')then
            if(pip = '0')then
              pip <= '1';
            end if;
          end if;
          if(opData_r1(dataWidth+dataWidth/32+2) = '1')then
            if(pip = '1')then
              pip <= '0';
            end if;
          end if;
        end if;
      end if;
    end if;
  end  process;      
        
      
  process(Clk)
  begin
    if(rising_edge(Clk))then
      if(temp_tdata(dataWidth+dataWidth/32+3) = '1')and(tmp_tvalid = '0')then
        int_seq_no <= int_seq_no + "0001";
      end if;
    end if;
  end process;
  
          
  rq_b2c.s_axis_rq_tvalid <= not tmp_tvalid;
  process(temp_tdata,tmp_tvalid,int_seq_no)
  begin
    if(temp_tdata(dataWidth+dataWidth/32+3) = '1')then
      rq_b2c.s_axis_rq_tdata <= temp_tdata(dataWidth-1 downto 0);
      
      rq_b2c.s_axis_rq_tuser(7 downto 0) <= temp_tdata(111 downto 104); -- lbe/fbe
      rq_b2c.s_axis_rq_tuser(10 downto 8) <= temp_tdata(114 downto 112); -- address offset
      rq_b2c.s_axis_rq_tuser(23 downto 11) <= (others => '0');
      rq_b2c.s_axis_rq_tuser(27 downto 24) <= int_seq_no;
      rq_b2c.s_axis_rq_tuser(axi4_data_widthRX/8+27 downto 28) <= (others => '0');
      rq_b2c.s_axis_rq_tlast <= temp_tdata(dataWidth+dataWidth/32+2) and not tmp_tvalid;
      if(tmp_tvalid = '0')then
        if(temp_tdata(75) = '1')then        -- writes
          rq_b2c.s_axis_rq_tkeep <= X"FF";
        else                              -- reads
          rq_b2c.s_axis_rq_tkeep <= X"0F";
        end if;
      else
        rq_b2c.s_axis_rq_tkeep <= (others => '0');
      end if;
      
      rq_b2c.s_axis_rq_tdata(dataWidth-1 downto 115) <= temp_tdata(dataWidth-1 downto 115);
      rq_b2c.s_axis_rq_tdata(114 downto 104) <= (others => '0');
      rq_b2c.s_axis_rq_tdata(103 downto 0) <= temp_tdata(103 downto 0);
      
--       pre_pargen_data(dataWidth-1 downto 115) <= temp_tdata(dataWidth-1 downto 115);
--       pre_pargen_data(114 downto 104)         <= (others => '0');
--       pre_pargen_data(103 downto 0)           <= temp_tdata(103 downto 0);
      
      
    else
      rq_b2c.s_axis_rq_tuser(axi4_data_widthRX/8+27 downto 28) <= (others => '0');
      rq_b2c.s_axis_rq_tuser(27 downto 0) <= (others => '0');
      rq_b2c.s_axis_rq_tdata <= temp_tdata(dataWidth-1 downto 0);
--       pre_pargen_data        <= temp_tdata(dataWidth-1 downto 0);
      rq_b2c.s_axis_rq_tlast <= temp_tdata(dataWidth+dataWidth/32+2) and not tmp_tvalid;
      if(temp_tdata(dataWidth+dataWidth/32+2) = '0')then
        if(tmp_tvalid = '0')then
          rq_b2c.s_axis_rq_tdata <= temp_tdata(dataWidth-1 downto 0);
          rq_b2c.s_axis_rq_tkeep <= (others => '1');
        else
          rq_b2c.s_axis_rq_tkeep <= (others => '0');
          rq_b2c.s_axis_rq_tdata <= (others => '0');
        end if;
      else
        if(tmp_tvalid = '0')then
          rq_b2c.s_axis_rq_tkeep <= temp_tdata(dataWidth+dataWidth/32-1 downto dataWidth);
          rq_b2c.s_axis_rq_tdata <= temp_tdata(dataWidth-1 downto 0);
        else
          rq_b2c.s_axis_rq_tkeep <= (others => '0');
          rq_b2c.s_axis_rq_tdata <= (others => '0');
        end if;
      end if;
    end if;
  end process;
  
--   parity_blocks: for index in 1 to 32 generate
--   begin
--     pg_blk : pargen
--       port map(
--         data_in => pre_pargen_data(index*8-1 downto (index-1)*8),
--         parity_odd => rq_b2c.s_axis_rq_tuser(27+index)
--       );
--   end generate parity_blocks;
  
  process(clk)
  begin
    if(rising_edge(clk))then
      cpldComplete <= '0';
      if((rc_c2b.m_axis_rc_tvalid = '1')and
         (rc_b2c_int.m_axis_rc_tready = '1')and
         (rc_c2b.m_axis_rc_tuser(32) = '1'))then
        dest_channel <= rc_c2b.m_axis_rc_tdata(70 downto 67);
        dest_tag     <= '0'&rc_c2b.m_axis_rc_tdata(66 downto 64);
        cpldCompleteFlag <= rc_c2b.m_axis_rc_tdata(30);
      end if;
      if((rc_c2b.m_axis_rc_tvalid = '1')and
         (rc_b2c_int.m_axis_rc_tready = '1')and
         (cpldCompleteFlag = '1')and
         (rc_c2b.m_axis_rc_tlast = '1'))then
        cpldCompleteFlag <= '0';
        cpldComplete <= '1';
      end if;
      respData <= dest_tag & rc_c2b.m_axis_rc_tdata;
    end if;
  end process;
  
  process(clk)
  begin
    if(rising_edge(clk))then
      if((rc_c2b.m_axis_rc_tvalid = '1')and(rc_b2c_int.m_axis_rc_tready = '1')and(rc_c2b.m_axis_rc_tuser(32) = '0'))then
        case(dest_channel) is
          when "0000" => push_select <= X"0001";
          when "0001" => push_select <= X"0002";
          when "0010" => push_select <= X"0004";
          when "0011" => push_select <= X"0008";
          when "0100" => push_select <= X"0010";
          when "0101" => push_select <= X"0020";
          when "0110" => push_select <= X"0040";
          when "0111" => push_select <= X"0080";
          when "1000" => push_select <= X"0100";
          when "1001" => push_select <= X"0200";
          when "1010" => push_select <= X"0400";
          when "1011" => push_select <= X"0800";
          when "1100" => push_select <= X"1000";
          when "1101" => push_select <= X"2000";
          when "1110" => push_select <= X"4000";
          when others => push_select <= X"8000";
        end case;
      else
        push_select <= X"0000";
      end if;
    end if;
  end process;
  
  process(dest_channel,cpldComplete)
  begin
    if(cpldComplete = '1')then
      case(dest_channel) is
        when "0000" => cpldCompleteStrobes <= X"0001";
        when "0001" => cpldCompleteStrobes <= X"0002";
        when "0010" => cpldCompleteStrobes <= X"0004";
        when "0011" => cpldCompleteStrobes <= X"0008";
        when "0100" => cpldCompleteStrobes <= X"0010";
        when "0101" => cpldCompleteStrobes <= X"0020";
        when "0110" => cpldCompleteStrobes <= X"0040";
        when "0111" => cpldCompleteStrobes <= X"0080";
        when "1000" => cpldCompleteStrobes <= X"0100";
        when "1001" => cpldCompleteStrobes <= X"0200";
        when "1010" => cpldCompleteStrobes <= X"0400";
        when "1011" => cpldCompleteStrobes <= X"0800";
        when "1100" => cpldCompleteStrobes <= X"1000";
        when "1101" => cpldCompleteStrobes <= X"2000";
        when "1110" => cpldCompleteStrobes <= X"4000";
        when others => cpldCompleteStrobes <= X"8000";
      end case;
    else
      cpldCompleteStrobes <= X"0000";
    end if;
  end process;
     
  process(clk)
    variable temp4 : std_logic_vector(7 downto 0);
  begin
    if(rising_edge(clk))then
      
      temp4 := (others => '0');
      for i in 0 to channelCount-1 loop
        temp4:= temp4 + actualPIF(i);
      end loop;  -- i
      totalPIF <= temp4;
      if(pausePIF = '0')then
        if (totalPIF >= maxTotalPIF) or piperror = '1' then
          pausePIF <= '1';
        else
          pausePIF <= '0';
        end if;
      else
        if(totalPIF<maxTotalPIF)then
          pausePIF <= '0';
        end if;        
      end if;

      status <= (others => '0');
      status(7 downto 0) <= totalPIF;
      for i in 0 to channelCount-1 loop
        status(11+4*i downto 8+4*i) <= actualPIF(i)(3 downto 0);
        if respNotFull(i) = '0' then
          respNotFull_sticky(i) <= '1';
        end if;
        if i<4 then
          status(28+i) <= respNotFull(i);
          status(24+i) <= respNotFull_sticky(i);
        end if;
      end loop;  -- i
 
      
    end if;
  end process;  
  
  rc_b2c_int.m_axis_rc_tready <= '1';
  rc_b2c <= rc_b2c_int;
  rq_b2c.s_axis_rq_tparity <= (others => '0');

  

  
end synth;

