--------------------------------------------------------------------------------
-- Company:         Alpha-Data Parallel Systems
-- Engineer:        Alan Crawford
--
-- Create Date:
-- Design Name:
-- Module Name:     pcie_ocp_slave64
-- Project Name:    adb3
-- Target Device:   V5FX
-- Tool versions:
-- Description:     PCIE to OCP slave port - converts OCP READ/WRITES into PCIe
--                  and translates CPLDs back into OCP
--
-- Dependencies:    PCIe/OCP Packages for signal records
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.pcie.ALL;
use work.adb3_ocp.ALL;
use work.pciegen3_pkg.all;

entity dma_ocp_if is
  generic(chIndexVector       : std_logic_vector(4 downto 0);
          dataWidth           : integer := 256
          );
  port(
    clk                     : in  std_logic;
    sr                      : in  std_logic;
    us_counter              : in  std_logic_vector(9 downto 0);
    m2s_i                   : in  adb3w_ocp_m2sT;
    s2m_o                   : out adb3w_ocp_s2mT; -- := ADB3_OCP_DEFAULT_S2M;

    slave_o2p_push          : out std_logic := '0';
    slave_o2p_data          : out std_logic_vector(dataWidth+dataWidth/32+3 downto 0);
    slave_o2p_notfull       : in  std_logic;

    slave_p2o_push          : in  std_logic;
--     slave_p2o_tag           : in  std_logic_vector(3 downto 0);
    slave_p2o_data          : in  std_logic_vector(dataWidth+3 downto 0);
    slave_p2o_notfull       : out std_logic;
    cpldComplete            : in  std_logic;
    disableReads            : in  std_logic;
    actualPIF               : out std_logic_vector(7 downto 0);
    maxPIF                  : in  std_logic_vector(7 downto 0) := "00000011"
    );

end dma_ocp_if;

architecture synth of dma_ocp_if is

signal dw_count_writes                : std_logic_vector(9 downto 0) := (others => '0');
signal dw_count_reads                 : std_logic_vector(9 downto 0) := (others => '0');
--signal dw_valid                       : std_logic_vector(dataWidth/32-1 downto 0) := (OTHERS => '0');
signal dw_valid                       : std_logic_vector(7 downto 0) := (OTHERS => '0');
signal dw_keep                        : std_logic_vector(dataWidth/32-1 downto 0) := (OTHERS => '0');
signal reg_cmd                        : adb3w_ocp_m2sT; -- := idle_ocp_master;
signal reg_data_saved                 : std_logic_vector(dataWidth-1 downto 0) := (others => '0');
signal accept_cmd                     : std_logic := '0';
signal accept_cmd_e                   : std_logic := '0';
signal accept_cmd_reg                 : std_logic := '0';
signal accept_data                    : std_logic := '0';
signal accept_data_q                  : std_logic := '0';
signal fbe                            : std_logic_vector(3 downto 0) := (others => '0');
signal lbe                            : std_logic_vector(3 downto 0) := (others => '0');
signal pcie_header                    : std_logic_vector(dataWidth+dataWidth/32-1 downto 0) := (others => '0');
signal rev_ocp_data                   : std_logic_vector(dataWidth-1 downto 0) := (others => '0');
signal addr_offset                    : std_logic_vector(2 downto 0) := (others => '0');
-- signal slave_p2o_tag_out              : std_logic_vector(3 downto 0) := (others => '0');
signal tx_pkt_empty                   : std_logic := '1';
signal tx_pkt_pop                     : std_logic := '0';
signal pcie_header_in                 : std_logic_vector(21 downto 0) := (others => '0');
signal tx_pkt_sop                     : std_logic := '0';
signal tx_push                        : std_logic := '0';
type   o2p_slave_fsm_g3_t             is( SLAVE_IDLE,SLAVE_CMD,SLAVE_SEND_READ,SLAVE_SEND_READ_HOLD,SLAVE_SEND_READ_END,SLAVE_SEND_WRITE1,SLAVE_SEND_WRITE2, SLAVE_SEND_WRITE3);

signal slave_fsm                      : o2p_slave_fsm_g3_t := SLAVE_IDLE;
signal next_slave_fsm                 : o2p_slave_fsm_g3_t := SLAVE_IDLE;
signal tag_request                    : std_logic := '0';
signal readComplete                   : std_logic := '0';
signal cpld_s_fsm                     : CPLD_S_FSM_T := S_FSM_IDLE;
signal next_cpld_s_fsm                : CPLD_S_FSM_T := S_FSM_IDLE;
signal data_hold                      : std_logic_vector(31 downto 0) := (others => '0');
signal data_out                       : std_logic_vector(127 downto 0) := (others => '0');
signal cpld_empty                     : std_logic := '1';
signal cpld_not_full                  : std_logic;
signal temp_cpld_ocp                  : std_logic_vector(dataWidth+7 downto 0) := (others => '0');
signal burstlen                       : std_logic_vector(7 downto 0) := (others => '0');
signal bcount                         : std_logic_vector(11 downto 0) := (others => '0');
                                      
signal tag_cplt                       : std_logic := '0';
signal sop_delay1                     : std_logic := '0';
signal sop_delay2                     : std_logic := '0';
signal reads_full                     : std_logic := '0';
signal tags_in_flight                 : std_logic_vector(3 downto 0) := "0000";
signal slave_p2o_push_r1              : std_logic := '0';
signal slave_p2o_data_r1              : std_logic_vector(dataWidth+7 downto 0) := (others => '0');

-- signal cpldcompletedel                : std_logic := '0';
signal tx_pkt_available               : std_logic;
Signal one_dw		                      : std_logic := '0';
signal processed_reads                : integer := 0;
signal next_tag_ready                 : std_logic := '1';
subtype  tag_valueT                   is std_logic_vector(7 downto 0);
type     tag_managerT                 is array(0 to 7) of tag_valueT;

signal tag_value_in                   : tag_valueT;
signal tag_value_out                  : tag_valueT;
signal tag_store                      : tag_managerT;
signal tx_tag                         : std_logic_vector(2 downto 0) := "000";
signal rx_tag                         : std_logic_vector(2 downto 0) := "000";
signal tag_active                     : std_logic_vector(7 downto 0) := X"00";
signal temp_wafull                    : std_logic;
signal tmp_radv                       : std_logic;

signal accept_cmd_based_on_tags : std_logic := '1';


signal local_rst : std_logic := '1';
signal abort_detect : std_logic := '0';

begin

  s2m_o.CmdAccept <= accept_cmd;
  s2m_o.dataAccept <= accept_data;

-- -----------------------------------------------------------------------------
-- Reverse the bytes of  the OCP Data
-- -----------------------------------------------------------------------------
      rev_ocp_data <= m2s_i.data;

-- -----------------------------------------------------------------------------
-- Store OCP Tags against PCIE Tags so returned completion data can be
-- correctly tagged
-- -----------------------------------------------------------------------------

  tag_value_in <= reg_cmd.tag(7 downto 0);
  
  gen_tags : process(clk)
   begin
    if(rising_edge(clk))then
      if((tag_request = '1') and (cpldComplete = '0') and (reads_full = '0')and (tag_active(conv_integer(tx_tag)) = '0'))then
        tags_in_flight <= tags_in_flight + '1';
      elsif((tag_request = '0') and(cpldComplete = '1') and (tags_in_flight /="0000"))then
        tags_in_flight <= tags_in_flight - '1';
      end if;
      if((tag_request = '1') and (reads_full = '0')and (tag_active(conv_integer(tx_tag)) = '0'))then
        tag_store(conv_integer(tx_tag)) <= tag_value_in;
        tx_tag <= tx_tag+'1';
      end if;
      if local_rst = '1' then
        tx_tag <= (others => '0');
        tags_in_flight <= (others => '0');
      end if;
    end if;
  end process gen_tags;
  
  actualPIF <= "0000"&tags_in_flight;

  mon_tags : process(clk)
   begin
    if(rising_edge(clk))then
--       cpldcompletedel <= cpldcomplete;
      if((tag_request = '1') and (reads_full = '0')and(tag_active(conv_integer(tx_tag)) = '0'))then
        tag_active(conv_integer(tx_tag)) <= '1';
      end if;
      if(cpldcomplete = '1') then
        tag_active(conv_integer(slave_p2o_data(dataWidth+2 downto dataWidth))) <= '0';
      end if;
      if local_rst = '1' then
        tag_active <= (others => '0');
      end if;
    end if;
  end process;  
  
  
  maxPIFtest : process(clk)
   begin
    if(rising_edge(clk))then
      if(tags_in_flight = maxPIF) then
        reads_full <= '1';
        next_tag_ready <= '0';
      else
        reads_full <= '0';
        next_tag_ready <= (not tag_active(conv_integer(tx_tag)));
      end if;
      --reads_full <= '0';
      accept_cmd_based_on_tags <=  ( not reads_full) and ( not tag_active(conv_integer(tx_tag)));
    end if;
  end process maxPIFtest;

-- -----------------------------------------------------------------------------
-- Wait for an OCP command and then capture the OCP Command
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
-- control capture of OCP data based on valid data being available and the
-- FSM being in the appropriate FSM states
-- -----------------------------------------------------------------------------


  accept_cmd_e <= '1' when ((((m2s_i.cmd = OCP_CMD_READ)and(next_tag_ready='1'))or(m2s_i.cmd = OCP_CMD_WRITE))and (accept_cmd = '0') and(slave_o2p_notfull = '1') and (next_slave_fsm =SLAVE_IDLE)) and (disableReads = '0') else '0';

    
  capture_cmd : process(clk)
  begin
    if(rising_edge(clk))then
      if accept_cmd_e = '1' then
        accept_cmd <= '1';
        reg_cmd <= m2s_i;
        bcount <= m2s_i.burstlength;
      else
        accept_cmd <= '0';
      end if;
      if((m2s_i.datavalid = '1') and (((next_slave_fsm = SLAVE_CMD)and(reg_cmd.cmd=OCP_CMD_WRITE))or(((next_slave_fsm = SLAVE_SEND_WRITE2)and(bcount /= "000000000000")))))then 
        accept_data_q <= '1';
        bcount <= bcount - '1';
       else
        accept_data_q <= '0';
      end if;
    end if;
  end process;

  accept_data <= accept_data_q; 

-- -----------------------------------------------------------------------------
-- check the byte enables to see how many of the DWs are used
-- -----------------------------------------------------------------------------
  check_bes : process(m2s_i.databyteen)
  begin
    if(m2s_i.databyteen(31 downto 28) ="0000")then
      dw_valid(7) <= '0';
    else
      dw_valid(7) <= '1';
    end if;
    if(m2s_i.databyteen(27 downto 24) ="0000")then
      dw_valid(6) <= '0';
    else
      dw_valid(6) <= '1';
    end if;
    if(m2s_i.databyteen(23 downto 20) ="0000")then
      dw_valid(5) <= '0';
    else
      dw_valid(5) <= '1';
    end if;
    if(m2s_i.databyteen(19 downto 16) ="0000")then
      dw_valid(4) <= '0';
    else
      dw_valid(4) <= '1';
    end if;
    if(m2s_i.databyteen(15 downto 12) ="0000")then
      dw_valid(3) <= '0';
    else
      dw_valid(3) <= '1';
    end if;
    if(m2s_i.databyteen(11 downto 8) ="0000")then
      dw_valid(2) <= '0';
    else
      dw_valid(2) <= '1';
    end if;
    if(m2s_i.databyteen(7 downto 4) ="0000")then
      dw_valid(1) <= '0';
    else
      dw_valid(1) <= '1';
    end if;
    if(m2s_i.databyteen(3 downto 0) ="0000")then
      dw_valid(0) <= '0';
    else
      dw_valid(0) <= '1';
    end if;
  end process check_bes;

-- -----------------------------------------------------------------------------
-- calculate the length of the PCIE request (in DWs) based on the burstlength
-- and the byte enables
-- REMEMEBER: When burstlength > 1 ALL byte enables are active
--            Only when burstlength = 1 can the byte enables be '0'
--            but must be continuous
-- -----------------------------------------------------------------------------
  length_cnt : process(reg_cmd.burstlength, dw_valid,reg_cmd.cmd,reg_cmd.addr,m2s_i)
  variable tmp_length : std_logic_vector(9 downto 0) := (others => '0');
  variable tmp_len_end : std_logic_vector(9 downto 0) := (others => '0');
  variable tmp_len_start : std_logic_vector(9 downto 0) := (others => '0');
  variable tmp_len_sande : std_logic_vector(9 downto 0) := (others => '0');
  begin
    if(reg_cmd.burstlength /= "000000000001")then
      if(reg_cmd.tag(7 downto 6) = "10")then
        tmp_length := reg_cmd.burstlength(6 downto 0) & "000" - "0000010000";
        tmp_len_start := "0000001000" - ("0000000" & reg_cmd.addr(4 downto 2));
--         if(reg_cmd.tag(1 downto 0) /= "00")then
          tmp_len_end := "0000000" & reg_cmd.tag(4 downto 2)+'1';
--         else
--           tmp_len_end := "0000000" & reg_cmd.tag(4 downto 2);
--         end if;
        tmp_len_sande := tmp_len_start+tmp_len_end;
        dw_count_writes <= tmp_length + tmp_len_sande;
        dw_count_reads <= reg_cmd.burstlength(6 downto 0) & "000";
      else
        dw_count_writes <= reg_cmd.burstlength(6 downto 0) & "000";
        dw_count_reads <= reg_cmd.burstlength(6 downto 0) & "000";
      end if;
    else
      case(reg_cmd.addr(4 downto 2))is
        when "000"      => dw_count_reads  <= "0000001000";
        when "001"      => dw_count_reads  <= "0000000111";
        when "010"      => dw_count_reads  <= "0000000110";
        when "011"      => dw_count_reads  <= "0000000101";
        when "100"      => dw_count_reads  <= "0000000100";
        when "101"      => dw_count_reads  <= "0000000011";
        when "110"      => dw_count_reads  <= "0000000010";
        when others     => dw_count_reads  <= "0000000001";
      end case;
      case(dw_valid)is
        when "00000001" => dw_count_writes <= "0000000001";
        when "00000010" => dw_count_writes <= "0000000001";
        when "00000100" => dw_count_writes <= "0000000001";
        when "00001000" => dw_count_writes <= "0000000001";
        when "00010000" => dw_count_writes <= "0000000001";
        when "00100000" => dw_count_writes <= "0000000001";
        when "01000000" => dw_count_writes <= "0000000001";
        when "10000000" => dw_count_writes <= "0000000001";
        when "00000011" => dw_count_writes <= "0000000010";
        when "00000110" => dw_count_writes <= "0000000010";
        when "00001100" => dw_count_writes <= "0000000010";
        when "00011000" => dw_count_writes <= "0000000010";
        when "00110000" => dw_count_writes <= "0000000010";
        when "01100000" => dw_count_writes <= "0000000010";
        when "11000000" => dw_count_writes <= "0000000010";
        when "00000111" => dw_count_writes <= "0000000011";
        when "00001110" => dw_count_writes <= "0000000011";
        when "00011100" => dw_count_writes <= "0000000011";
        when "00111000" => dw_count_writes <= "0000000011";
        when "01110000" => dw_count_writes <= "0000000011";
        when "11100000" => dw_count_writes <= "0000000011";
        when "00001111" => dw_count_writes <= "0000000100";
        when "00011110" => dw_count_writes <= "0000000100";
        when "00111100" => dw_count_writes <= "0000000100";
        when "01111000" => dw_count_writes <= "0000000100";
        when "11110000" => dw_count_writes <= "0000000100";
        when "00011111" => dw_count_writes <= "0000000101";
        when "00111110" => dw_count_writes <= "0000000101";
        when "01111100" => dw_count_writes <= "0000000101";
        when "11111000" => dw_count_writes <= "0000000101";
        when "00111111" => dw_count_writes <= "0000000110";
        when "01111110" => dw_count_writes <= "0000000110";
        when "11111100" => dw_count_writes <= "0000000110";
        when "01111111" => dw_count_writes <= "0000000111";
        when "11111110" => dw_count_writes <= "0000000111";
        when "11111111" => dw_count_writes <= "0000001000";
        when others =>     dw_count_writes <= "0000000000";
-- -----------------------------------------------------------------------------
-- synthesis translate_off
-- -----------------------------------------------------------------------------
--             assert false report "ERROR - Illegal Byte Enable declaration in OCP Data" severity error;
-- -----------------------------------------------------------------------------
-- synthesis translate_on
-- -----------------------------------------------------------------------------
      end case;
    end if;
    case(dw_valid)is
      when "00000000" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= "0000";
                          addr_offset <= "000";
      when "00000001" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= "0000";
                          addr_offset <= "000";
      when "00000010" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= "0000";
                          addr_offset <= "001";
      when "00000100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= "0000";
                          addr_offset <= "010";
      when "00001000" =>  fbe         <= m2s_i.databyteen(15 downto 12);
                          lbe         <= "0000";
                          addr_offset <= "011";
      when "00010000" =>  fbe         <= m2s_i.databyteen(19 downto 16);
                          lbe         <= "0000";
                          addr_offset <= "100";
      when "00100000" =>  fbe         <= m2s_i.databyteen(23 downto 20);
                          lbe         <= "0000";
                          addr_offset <= "101";
      when "01000000" =>  fbe         <= m2s_i.databyteen(27 downto 24);
                          lbe         <= "0000";
                          addr_offset <= "110";
      when "10000000" =>  fbe         <= m2s_i.databyteen(31 downto 28);
                          lbe         <= "0000";
                          addr_offset <= "111";
      when "00000011" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(7 downto 4);
                          addr_offset <= "000";
      when "00000110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(11 downto 8);
                          addr_offset <= "001";
      when "00001100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= m2s_i.databyteen(15 downto 12);
                          addr_offset <= "010";
      when "00011000" =>  fbe         <= m2s_i.databyteen(15 downto 12);
                          lbe         <= m2s_i.databyteen(19 downto 16);
                          addr_offset <= "011";
      when "00110000" =>  fbe         <= m2s_i.databyteen(19 downto 16);
                          lbe         <= m2s_i.databyteen(23 downto 20);
                          addr_offset <= "100";
      when "01100000" =>  fbe         <= m2s_i.databyteen(23 downto 20);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "101";
      when "11000000" =>  fbe         <= m2s_i.databyteen(27 downto 24);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "110";
      when "00000111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(11 downto 8);
                          addr_offset <= "000";
      when "00001110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(15 downto 12);
                          addr_offset <= "001";
      when "00011100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= m2s_i.databyteen(19 downto 16);
                          addr_offset <= "010";
      when "00111000" =>  fbe         <= m2s_i.databyteen(15 downto 12);
                          lbe         <= m2s_i.databyteen(23 downto 20);
                          addr_offset <= "011";
      when "01110000" =>  fbe         <= m2s_i.databyteen(19 downto 16);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "100";
      when "11100000" =>  fbe         <= m2s_i.databyteen(23 downto 20);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "101";
      when "00001111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(15 downto 12);
                          addr_offset <= "000";
      when "00011110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(19 downto 16);
                          addr_offset <= "001";
      when "00111100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= m2s_i.databyteen(23 downto 20);
                          addr_offset <= "010";
      when "01111000" =>  fbe         <= m2s_i.databyteen(15 downto 12);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "011";
      when "11110000" =>  fbe         <= m2s_i.databyteen(19 downto 16);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "100";
      when "00011111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(19 downto 16);
                          addr_offset <= "000";
      when "00111110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(23 downto 20);
                          addr_offset <= "001";
      when "01111100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "010";
      when "11111000" =>  fbe         <= m2s_i.databyteen(15 downto 12);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "011";
      when "00111111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(23 downto 20);
                          addr_offset <= "000";
      when "01111110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "001";
      when "11111100" =>  fbe         <= m2s_i.databyteen(11 downto 8);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "010";
      when "01111111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(27 downto 24);
                          addr_offset <= "000";
      when "11111110" =>  fbe         <= m2s_i.databyteen(7 downto 4);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "001";
      when "11111111" =>  fbe         <= m2s_i.databyteen(3 downto 0);
                          lbe         <= m2s_i.databyteen(31 downto 28);
                          addr_offset <= "000";
      when others =>      lbe         <= "0000";
                          fbe         <= "0000";
                          addr_offset <= "000";
-- -----------------------------------------------------------------------------
-- synthesis translate_off
-- -----------------------------------------------------------------------------
        assert false report "ERROR - Illegal Byte Enable declaration in OCP Data" severity error;
-- -----------------------------------------------------------------------------
-- synthesis translate_on
-- -----------------------------------------------------------------------------
    end case;
    if(reg_cmd.burstlength /= "000000000001")then
      lbe <= "1111";
      if(reg_cmd.tag(7 downto 6) = "10")then
        case(reg_cmd.addr(1 downto 0)) is
          when "00"   => fbe <= "1111";
          when "01"   => fbe <= "1110";
          when "10"   => fbe <= "1100";
          when others => fbe <= "1000";
        end case;
        case(reg_cmd.tag(1 downto 0)) is
          when "00"   => lbe <= "0001";
          when "01"   => lbe <= "0011";
          when "10"   => lbe <= "0111";
          when others => lbe <= "1111";
        end case;
      end if;
    end if;
    if(reg_cmd.cmd = OCP_CMD_READ)then
      lbe <= "1111";
      fbe <= "1111";
    end if;
  end process length_cnt;


-- -----------------------------------------------------------------------------
-- create the header for the PCIE packets Read and Write only
-- uses 64 bit addressing only if top 32 bits of OCP address is not 0
-- -----------------------------------------------------------------------------
  gen_header : process(fbe,lbe,addr_offset,reg_cmd,dw_count_writes,dw_count_reads,tx_tag)
  begin
    pcie_header(dataWidth+dataWidth/32-1 downto 120) <= (others => '0');
    pcie_header(119 downto 115) <= (others => '0');
    if(reg_cmd.cmd = OCP_CMD_WRITE)then
      pcie_header(111 downto 108) <= lbe;
      pcie_header(107 downto 104) <= fbe;
      pcie_header(114 downto 112) <= addr_offset;
    else  
      if(dw_count_reads = "0000000001")then
        pcie_header(111 downto 108) <= "0000";
      else
        pcie_header(111 downto 108) <= "1111";
      end if;
      pcie_header(107 downto 104) <= "1111";
      pcie_header(114 downto 112) <= "000";
    end if;      
    if(reg_cmd.cmd = OCP_CMD_WRITE)then
      pcie_header(103 downto 101) <= (others => '0');
      pcie_header(100 downto 96)  <= (others => '0');
    else
      pcie_header(103 downto 99) <= chIndexVector;
      pcie_header(98 downto 96)  <= tx_tag;
    end if;
    pcie_header(95 downto 79)   <= (others => '0');
    if(reg_cmd.cmd = OCP_CMD_WRITE)then
      pcie_header(78 downto 75) <= "0001";
    else
      pcie_header(78 downto 75) <= "0000";
    end if;
    if(reg_cmd.cmd = OCP_CMD_WRITE)then
      pcie_header(74 downto 64) <= '0' & dw_count_writes;
      pcie_header(63 downto 5)  <= reg_cmd.addr(63 downto 5);
      pcie_header(4 downto 2)   <= addr_offset(2 downto 0);
    else
      pcie_header(74 downto 64) <= '0' & dw_count_reads;
      pcie_header(63 downto 2)  <= reg_cmd.addr(63 downto 2);
    end if;
    pcie_header(1 downto 0)   <= "00";            
  end process gen_header;

-- -----------------------------------------------------------------------------
-- sequence FSM for creating the read and write PCIE packets from OCP commands
-- -----------------------------------------------------------------------------
  fsm_gen : process(slave_fsm, accept_cmd, reg_cmd,bcount, dw_count_writes,m2s_i,cpld_not_full,slave_o2p_notfull,next_tag_ready)
  begin
    case(slave_fsm)is
      when SLAVE_IDLE           =>  if(accept_cmd = '1')then
                                      next_slave_fsm <= SLAVE_CMD;
                                    else
                                      next_slave_fsm <= SLAVE_IDLE;
                                    end if;
      when SLAVE_CMD            =>  --if(slave_o2p_notfull = '1')then  
                                      if((reg_cmd.cmd = OCP_CMD_READ))then
                                        if((next_tag_ready = '1'))then
                                          next_slave_fsm <= SLAVE_SEND_READ;
                                        else
                                          next_slave_fsm <= SLAVE_CMD;
                                        end if;
                                      else
                                        if(m2s_i.dataValid = '1')then
                                          next_slave_fsm <= SLAVE_SEND_WRITE2;
                                        else
                                          next_slave_fsm <= SLAVE_CMD;
                                        end if;
                                      end if;
--                                     else
--                                       next_slave_fsm <= SLAVE_CMD;
--                                     end if;
      when SLAVE_SEND_READ      =>  if(m2s_i.cmd /= OCP_CMD_WRITE)then
                                      next_slave_fsm <= SLAVE_SEND_READ_HOLD;
                                    else
                                      next_slave_fsm <= SLAVE_IDLE;
                                    end if;
      when SLAVE_SEND_READ_HOLD =>  next_slave_fsm <= SLAVE_SEND_READ_END;
      when SLAVE_SEND_READ_END  =>  next_slave_fsm <= SLAVE_IDLE;
      when SLAVE_SEND_WRITE1    =>  next_slave_fsm <= SLAVE_SEND_WRITE2;
      when SLAVE_SEND_WRITE2    =>  if(bcount = "000000000000")then 
                                      if(accept_cmd = '1')then
                                        next_slave_fsm <= SLAVE_CMD;
                                      else
                                        next_slave_fsm <= SLAVE_IDLE;
                                      end if;
                                    else
                                      next_slave_fsm <= SLAVE_SEND_WRITE2;
                                    end if;
      when SLAVE_SEND_WRITE3    =>  if(accept_cmd = '1')then
                                      next_slave_fsm <= SLAVE_CMD;
                                    else
                                      next_slave_fsm <= SLAVE_IDLE;
                                    end if;
    end case;
  end process fsm_gen;

-- -----------------------------------------------------------------------------
-- Sequence the control signals and restructure data for transfer to OCP FIFO
-- -----------------------------------------------------------------------------
  fsm_update : process(clk)
  begin
    if(rising_edge(clk))then
      slave_fsm <= next_slave_fsm;
      if(next_slave_fsm = SLAVE_SEND_READ)then
        processed_reads <= processed_reads + 1;
        tag_request <= '1';
      else
        tag_request <= '0';
      end if;
      if local_rst = '1' then
        slave_fsm <= SLAVE_IDLE;
      end if;
    end if;
  end process fsm_update;
 
  push_data : process(clk) --next_slave_fsm,slave_fsm,accept_data)
  begin
     if(rising_edge(clk))then
      if ((accept_cmd = '1') or (accept_data = '1')) then
          slave_o2p_push <= '1';
      else
        slave_o2p_push <= '0';--accept_data;
      end if;
    end if;
  end process push_data;

  process(dw_valid)
  begin
    if(dw_valid(7) = '1')then
      dw_keep <= X"FF";
    elsif(dw_valid(6) = '1')then
      dw_keep <= X"7F";
    elsif(dw_valid(5) = '1')then
      dw_keep <= X"3F";
    elsif(dw_valid(4) = '1')then
      dw_keep <= X"1F";
    elsif(dw_valid(3) = '1')then
      dw_keep <= X"0F";
    elsif(dw_valid(2) = '1')then
      dw_keep <= X"07";
    elsif(dw_valid(1) = '1')then
      dw_keep <= X"03";
    else
      dw_keep <= X"01";
    end if;
  end process;
  
  op_data : process(clk)
  begin
    if(rising_edge(clk))then
      if(next_slave_fsm = SLAVE_CMD)then
        slave_o2p_data(dataWidth+dataWidth/32-1 downto 0) <= pcie_header;
        slave_o2p_data(dataWidth+dataWidth/32-1 downto dataWidth) <= dw_valid ;
        if(m2s_i.cmd = OCP_CMD_READ)then
          slave_o2p_data(dataWidth+dataWidth/32+3 downto dataWidth+dataWidth/32) <= "1111";
          slave_o2p_data(dataWidth+dataWidth/32-1 downto dataWidth) <= X"FF" ;
        else
          slave_o2p_data(dataWidth+dataWidth/32-1 downto dataWidth) <= X"FF" ;
          slave_o2p_data(dataWidth+dataWidth/32+3 downto dataWidth+dataWidth/32) <= "1000";
        end if;
      else
        slave_o2p_data(dataWidth-1 downto 0) <= rev_ocp_data ;
--         if((next_slave_fsm = SLAVE_SEND_WRITE2))then
          if(bcount = "000000000000")then
            slave_o2p_data(dataWidth+dataWidth/32+3 downto dataWidth+dataWidth/32) <= "0111";
          else
            slave_o2p_data(dataWidth+dataWidth/32+3 downto dataWidth+dataWidth/32) <= "0011";
          end if;
          slave_o2p_data(dataWidth+dataWidth/32-1 downto dataWidth) <= dw_keep ;
--         end if;
      end if;
      slave_p2o_data_r1 <= tag_store(conv_integer(slave_p2o_data(dataWidth+2 downto dataWidth))) & slave_p2o_data(dataWidth-1 downto 0);
      slave_p2o_push_r1 <= slave_p2o_push;
    end if;
  end process op_data;

-- -----------------------------------------------------------------------------
-- OCP Return Data FIFO

    i_data_out_fifo : packet_sfifo
    generic map(
          RAM_TYPE    => "AUTO",
          ORDER       => 7,
          WIDTH       => axi4_data_widthRX+8,
          AFULL_LEVEL => 110
    )
    port map(
      rst         => sr,
      sr          => '0',
      clk         => clk,
      wen         => slave_p2o_push_r1,
      wd          => slave_p2o_data_r1,
      wfull       => open,
      wafull      => temp_wafull,
      radv        => tmp_radv,
      rq          => temp_cpld_ocp, 
      rlevel      => open,
      rempty      => cpld_empty,
      debug_info  => open);
  
    cpld_not_full <= not temp_wafull;
    slave_p2o_notfull <= cpld_not_full;
    tmp_radv <= m2s_i.respaccept and not cpld_empty;
-- -----------------------------------------------------------------------------
-- create OCP signals from FIFO outputs
-- -----------------------------------------------------------------------------
  s2m_o.tag   <= temp_cpld_ocp(dataWidth+7 downto dataWidth); --tag_store(conv_integer(temp_cpld_ocp(dataWidth+2 downto dataWidth)));
  s2m_o.data  <= temp_cpld_ocp(dataWidth-1 downto 0);

-- -----------------------------------------------------------------------------
-- create OCP Return data VALID status
-- -----------------------------------------------------------------------------
  gen_ocp_resp : process(cpld_empty)
  begin
    if(cpld_empty = '1')then
      s2m_o.resp <= OCP_RESP_NONE;
    else
      s2m_o.resp <= OCP_RESP_VALID;
    end if;
  end process gen_ocp_resp;



  -- Generate a local synchoronous FSM reset in response Gloal Async RST or
  -- DMA abort detection
  gen_local_rst: process (clk,sr) is
  begin  -- process gen_local_rst
    if sr = '1' then
      local_rst <= '1';
    elsif rising_edge(clk) then              -- rising clock edge
      local_rst <= '0';
    end if;
  end process gen_local_rst;

  --  -- Detect OCP Cmd being removed before accepted 
  --detect_DMA_abort: process (clk) is
  --begin  -- process detect_DMA_abort
  --  if rising_edge(clk) then          -- rising clock edge
  --    if m2s_i.Cmd = OCP_CMD_IDLE and accept_cmd = '1' then
  --      abort_detect <= '1';
  --    else
  --      abort_detect <= '0';
  --    end if;
  --  end if;
  --end process detect_DMA_abort;
    
end synth;



         
