--------------------------------------------------------------------------------
--
-- Module Name      : async_pulse
--
--                    Copyright 2007 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      :
-- Takes a single pulse across clock domains
--
-- For any rising edge on I, generates a 1 clock cycle pulse on O,
-- Using CLK.
-- Delayed by at least 1 CLK cycle
--
--
-- Dependencies     : 
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on


entity async_pulse is
  
  port (
    clk : in std_logic;
    I   : in std_Logic;
    O   : out std_Logic);

end async_pulse;

architecture rtl of async_pulse is

  signal latch,latch_reg1,latch_reg2,latch_reg3,rst_latch : std_logic := '0';

  attribute shreg_extract : string;
  attribute shreg_extract of latch_reg1 : signal is "no";
  attribute shreg_extract of latch_reg2 : signal is "no";
  attribute shreg_extract of latch_reg3 : signal is "no";
  
begin  -- rtl

  -- Latch rising edge of I
  process (I, rst_latch)
  begin  -- process
    if rst_latch = '1' then             -- asynchronous reset (active low)
      latch <= '0';
    elsif I'event and I = '1' then  -- rising clock edge
      latch <= '1';
    end if;
  end process;


  process (clk)
  begin  -- process
    if clk'event and clk = '1' then  -- rising clock edge
      latch_reg1 <= latch;
      latch_reg2 <= latch_reg1;
      latch_reg3 <= latch_reg2;
    end if;
  end process;

  O <= latch_reg2 and not latch_reg3;
  
  rst_latch <= latch_reg2;
  
end rtl;
