--------------------------------------------------------------------------------
--
-- Module Name      : adb3l_to_adb3
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : Conversion module connecting ADB3L Master and ADB3 Slave
--
--
-- Dependencies     : adb3_ocp
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library Work;
use work.adb3_ocp.all;

entity adb3l_to_adb3 is
  port (
    pri_m2s : in  adb3l_ocp_m2sT;
    pri_s2m : out adb3l_ocp_s2mT;
    sec_m2s : out adb3_ocp_m2sT;
    sec_s2m : in  adb3_ocp_s2mT);
end entity adb3l_to_adb3;

architecture rtl of adb3l_to_adb3 is

begin  -- architecture rtl

  sec_m2s.Cmd <= pri_m2s.Cmd;

  sec_m2s.Addr(ADB3_OCP_ADDR_WIDTH-1 downto ADB3L_OCP_ADDR_WIDTH) <= (others => '0');
  sec_m2s.Addr(ADB3L_OCP_ADDR_WIDTH-1 downto 4) <= pri_m2s.Addr(ADB3L_OCP_ADDR_WIDTH-1 downto 4);
  sec_m2s.Addr(3 downto 0) <= (others => '0');

  sec_m2s.BurstLength <= X"001";

  sec_m2s.Data <= pri_m2s.Data & pri_m2s.Data & pri_m2s.Data & pri_m2s.Data;
  sec_m2s.DataByteEn <= X"000" & pri_m2s.DataByteEn when pri_m2s.Addr(3 downto 2) = "00" else
                         X"00" & pri_m2s.DataByteEn & X"0" when pri_m2s.Addr(3 downto 2) = "01" else
                         X"0" & pri_m2s.DataByteEn & X"00" when pri_m2s.Addr(3 downto 2) = "10" else
                         pri_m2s.DataByteEn & x"000" when pri_m2s.Addr(3 downto 2) = "11" else
                        X"0000";

  sec_m2s.DataValid <= pri_m2s.DataValid;

  sec_m2s.RespAccept <= pri_m2s.RespAccept;

  sec_m2s.Tag <= "000000" & pri_m2s.Addr(3 downto 2);

  pri_s2m.CmdAccept <= sec_s2m.CmdAccept;
  pri_s2m.DataAccept <= sec_s2m.DataAccept;
  pri_s2m.Data <= sec_s2m.Data(31 downto 0) when sec_s2m.Tag(1 downto 0) = "00" else
                  sec_s2m.Data(63 downto 32) when sec_s2m.Tag(1 downto 0) = "01" else
                  sec_s2m.Data(95 downto 64) when sec_s2m.Tag(1 downto 0) = "10" else
                  sec_s2m.Data(127 downto 96);
                        
  
  pri_s2m.Resp <= sec_s2m.Resp;

end architecture rtl;
