--------------------------------------------------------------------------------
--
-- Module Name      : mptl128_controller
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      :  
--
--
-- Dependencies     : adb3_ocp, mptl
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.mptl.all;

entity mptl128_controller is

  generic (
    channel : natural := 1;
    update_period : natural := 256);
  port (
    -- MPTL_Interfaces
    clk                    : in  std_logic;
    mptl128_data_out        : out  std_logic_vector(127 downto 0) := (others => '0');
    mptl128_valid_out       : out  std_logic := '0';
    mptl128_accept_out      : in  std_logic;
    mptl128_data_in         : in  std_logic_vector(127 downto 0);
    mptl128_valid_in        : in  std_logic;
    -- Last Packet Levels
    local_last_packets     : in  std_Logic_vector(127 downto 0);
    remote_last_packets    : out std_logic_vector(127 downto 0) := (others => '0');
    -- GPIO (for interrupts?)
    gpio_in                : in  std_Logic_vector(31 downto 0);
    gpio_out               : out std_Logic_vector(31 downto 0) := (others => '0');
    gpio_status            : in  std_Logic_vector(31 downto 0);
    remote_status          : out std_Logic_vector(31 downto 0);
    packet_sent            : out std_Logic := '0';
    packet_received        : out std_Logic := '0';
    online                 : in  std_logic);

end mptl128_controller;

architecture rtl of mptl128_controller is

  signal update_counter : std_logic_vector(10 downto 0) := (others => '0');
  signal send_message : std_logic := '0';
  signal send_message_count : std_logic_vector(2 downto 0) := "000";

  signal received_message_count : std_logic_vector(1 downto 0) := "00";

  signal diff : std_logic_vector(8 downto 0) := (others => '0');
  signal diff1 : std_logic;
  signal local_last_packets_reg : std_Logic_vector(127 downto 0) := (others => '0');
  signal local_last_packets_reg1 : std_Logic_vector(127 downto 0) := (others => '0');
  signal remote_last_packets_reg : std_Logic_vector(127 downto 0) := (others => '0');
  signal gpio_in_reg : std_logic_vector(31 downto 0);

  signal online_reg1 : std_logic := '0';
  signal online_reg : std_logic := '0';
  
 attribute shreg_extract : string;
 attribute shreg_extract of online_reg1 : signal is "no";
 attribute shreg_extract of online_reg : signal is "no";

  signal packet_counter : std_logic_vector(7 downto 0) := (others => '0');
  
begin  -- rtl

  send_messages0: process (clk)
  begin  -- process send_messages0
    if clk'event and clk = '1' then  -- rising clock edge
      packet_sent <= '0';
      online_reg1 <= online;
      online_reg <= online_reg1;
      
      if update_counter = CONV_STD_LOGIC_VECTOR(update_period,11) or diff1 = '1' then
        send_message <= online_reg;
        update_counter <= (others => '0');
      else
        update_counter <= update_counter+1;
        send_message <= '0';
      end if;

      --mptl128_data_out(47 downto 32) <= online_reg & send_message_count & send_message & update_counter;
      
      
      if send_message_count = "000" then
        if send_message = '1' then
          send_message_count <= "011";
          mptl128_valid_out <= '1';
          mptl128_data_out <= (others => '0');  -- Set rest of header to 0
          mptl128_data_out(3 downto 0) <=  MPTL_STATUS;
          mptl128_data_out(7 downto 4) <= CONV_STD_LOGIC_VECTOR(channel,4);
          mptl128_data_out(12 downto 8) <= CONV_STD_LOGIC_VECTOR(MPTL128_CONTROLLER_PACKET_SIZE,5);
          mptl128_data_out(23 downto 16) <= packet_counter;
           -- Set Checksum
          mptl128_data_out(39 downto 36) <= MPTL_STATUS xor CONV_STD_LOGIC_VECTOR(channel,4) xor CONV_STD_LOGIC_VECTOR(MPTL128_CONTROLLER_PACKET_SIZE,4) xor packet_counter(3 downto 0) xor packet_counter(7 downto 4);
          
          mptl128_data_out(95 downto 64) <= gpio_in_reg;
          mptl128_data_out(127 downto 96) <= gpio_status;
        end if;
      elsif send_message_count = "011" then
        if mptl128_accept_out = '1' then    
          send_message_count <= "001";
          mptl128_data_out <= local_last_packets_reg1(127 downto 0);
        end if;
      else
        send_message_count <= "000";
        mptl128_valid_out <= '0';
        packet_sent <= '1';
        packet_counter <= packet_counter+1;
      end if;


      local_last_packets_reg <= local_last_packets;
      local_last_packets_reg1 <= local_last_packets_reg;
      gpio_in_reg <= gpio_in;
      for i in 0 to 7 loop
        if local_last_packets_reg(16*i+15 downto 16*i) = local_last_packets(16*i+15 downto 16*i) then
          diff(i) <= '0';
        else
          diff(i) <= '1';
        end if;
      end loop;  -- i
      if gpio_in_reg = gpio_in then
        diff(8) <= '0';
      else
        diff(8) <= '1';
      end if;
      diff1 <= OR_reduce(diff);
    end if;
  end process send_messages0;


  receive_message0: process (clk)
  begin  -- process receive_message0
    if clk'event and clk = '1' then  -- rising clock edge
      packet_received <= '0';
      if received_message_count = "00" then
        if mptl128_valid_in = '1' and mptl128_data_in(3 downto 0) = MPTL_STATUS then
          received_message_count <= "10";
          gpio_out <=  mptl128_data_in(95 downto 64);
          remote_status <= mptl128_data_in(127 downto 96);
        end if;
      else --received_message_count = "10" then
        if mptl128_valid_in = '1' then
          remote_last_packets_reg <=  mptl128_data_in;
          received_message_count <= "00";
          packet_received <= '1';
        end if;
      end if;
      remote_last_packets <= remote_last_packets_reg;
    end if;
  end process receive_message0;
  
  
end rtl;
