--------------------------------------------------------------------------------
--
-- Module Name      : mptl128_gth7_interface
--
--                    Copyright 2012 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : KU GTH Instantiation for 4x32 bit GTP MPTL-HSS link
--
--
-- Dependencies     : 
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity mptl128_gthu_interface is
  port (
    clkin           : in  std_logic;
    pllrst          : in  std_logic;
    txrst           : in  std_logic;
    ref_clk         : out std_logic;
    rref_clk        : out std_logic;
    gtp_clk         : in  std_logic;
    drpclk          : in  std_logic;
    -- Connection to MPTL64HSS Buffers
    gtp_data_in     : in  std_logic_vector(127 downto 0);
    gtp_charisk_in  : in  std_logic_vector(15 downto 0);
    gtp_data_out    : out std_logic_vector(127 downto 0);
    gtp_charisk_out : out std_logic_vector(15 downto 0);
    -- Connection to Serial Ports
    tx_p            : out std_logic_vector(3 downto 0);
    tx_n            : out std_logic_vector(3 downto 0);
    rx_p            : in  std_logic_vector(3 downto 0);
    rx_n            : in  std_logic_vector(3 downto 0);
    -- Control
    loopback        : in  std_logic_vector(2 downto 0);
    config          : in  std_logic_vector(15 downto 0);
    rx_reset        : in  std_logic_vector(7 downto 0);
    en_comma_align  : in  std_logic;
    powerdown       : in  std_logic;
    pll_locked      : out std_logic;
    rx_status       : out std_logic_vector(23 downto 0);
    rx_loss_of_sync : out std_logic_vector(15 downto 0));

end mptl128_gthu_interface;

architecture ku_x4 of mptl128_gthu_interface is

  signal txoutclk : std_logic_vector(3 downto 0);
  signal usrclk_in : std_logic_vector(3 downto 0);
  signal rx_reset_done : std_logic_vector(0 downto 0);
  signal tx_reset_done : std_logic_vector(0 downto 0);
  signal align         : std_logic_vector(3 downto 0);

  signal usrrdy             : std_logic := '0';

 signal  txctrl0_in                         :   STD_LOGIC_VECTOR (63 downto 0) := (others => '0');
 signal   txctrl1_in                         :   STD_LOGIC_VECTOR (63 downto 0) := (others => '0');
 signal   txctrl2_in                         :   STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
 signal   rxctrl0_out                        :  STD_LOGIC_VECTOR (63 downto 0);
 signal   rxctrl1_out                        :  STD_LOGIC_VECTOR (63 downto 0);
 signal   rxctrl2_out                        :  STD_LOGIC_VECTOR (31 downto 0);
 signal   rxctrl3_out                        :  STD_LOGIC_VECTOR (31 downto 0);
 
 signal   drpclk_in                          : STD_LOGIC_VECTOR (3 downto 0);
signal   refclk_in                          : STD_LOGIC_VECTOR (3 downto 0);

component gtwizard_ultrascale_0_example_wrapper is
  port (
    gtwiz_userclk_tx_reset_in          : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_tx_srcclk_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_tx_usrclk_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_tx_usrclk2_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_tx_active_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_clk_freerun_in         : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_all_in                 : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_tx_pll_and_datapath_in : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_tx_datapath_in         : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_rx_pll_and_datapath_in : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_rx_datapath_in         : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_rx_cdr_stable_out      : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_tx_done_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_reset_rx_done_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_rx_reset_in          : in  STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_rx_srcclk_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_rx_usrclk_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_rx_usrclk2_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userclk_rx_active_out            : out STD_LOGIC_VECTOR (0 to 0);
    gtwiz_userdata_tx_in               : in  STD_LOGIC_VECTOR (127 downto 0);
    gtwiz_userdata_rx_out              : out STD_LOGIC_VECTOR (127 downto 0);
    drpclk_in                          : in  STD_LOGIC_VECTOR (3 downto 0);
    gthrxn_in                          : in  STD_LOGIC_VECTOR (3 downto 0);
    gthrxp_in                          : in  STD_LOGIC_VECTOR (3 downto 0);
    gtrefclk0_in                       : in  STD_LOGIC_VECTOR (3 downto 0);
    rx8b10ben_in                       : in  STD_LOGIC_VECTOR (3 downto 0);
    rxcommadeten_in                    : in  STD_LOGIC_VECTOR (3 downto 0);
    rxmcommaalignen_in                 : in  STD_LOGIC_VECTOR (3 downto 0);
    rxpcommaalignen_in                 : in  STD_LOGIC_VECTOR (3 downto 0);
    tx8b10ben_in                       : in  STD_LOGIC_VECTOR (3 downto 0);
    txctrl0_in                         : in  STD_LOGIC_VECTOR (63 downto 0);
    txctrl1_in                         : in  STD_LOGIC_VECTOR (63 downto 0);
    txctrl2_in                         : in  STD_LOGIC_VECTOR (31 downto 0);
    gthtxn_out                         : out STD_LOGIC_VECTOR (3 downto 0);
    gthtxp_out                         : out STD_LOGIC_VECTOR (3 downto 0);
    rxbyteisaligned_out                : out STD_LOGIC_VECTOR (3 downto 0);
    rxbyterealign_out                  : out STD_LOGIC_VECTOR (3 downto 0);
    rxcommadet_out                     : out STD_LOGIC_VECTOR (3 downto 0);
    rxctrl0_out                        : out STD_LOGIC_VECTOR (63 downto 0);
    rxctrl1_out                        : out STD_LOGIC_VECTOR (63 downto 0);
    rxctrl2_out                        : out STD_LOGIC_VECTOR (31 downto 0);
    rxctrl3_out                        : out STD_LOGIC_VECTOR (31 downto 0);
    rxpmaresetdone_out                 : out STD_LOGIC_VECTOR (3 downto 0);
    txpmaresetdone_out                 : out STD_LOGIC_VECTOR (3 downto 0));
end component;
signal drpclk_sim : std_logic := '0';

signal gtwiz_userclk_tx_reset_in          : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_userclk_tx_srcclk_out        : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_tx_usrclk_out        : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_tx_usrclk2_out       : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_tx_active_out        : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_reset_clk_freerun_in         : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_reset_all_in                 : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_reset_tx_pll_and_datapath_in : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_reset_tx_datapath_in         : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_reset_rx_pll_and_datapath_in : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_reset_rx_datapath_in         : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_reset_rx_cdr_stable_out      : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_reset_tx_done_out            : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_reset_rx_done_out            : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_rx_reset_in          : STD_LOGIC_VECTOR (0 to 0) := "0";
signal gtwiz_userclk_rx_srcclk_out        : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_rx_usrclk_out        : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_rx_usrclk2_out       : STD_LOGIC_VECTOR (0 to 0);
signal gtwiz_userclk_rx_active_out        : STD_LOGIC_VECTOR (0 to 0);

 signal rxpmaresetdone_out                 : STD_LOGIC_VECTOR (3 downto 0);
 signal txpmaresetdone_out                 : STD_LOGIC_VECTOR (3 downto 0);

begin  -- rtl

  drpclk_sim <= not drpclk_sim after 12.5 ns;

--  bufg_refclk : BUFG_GT
--    port map(
--      I => gtwiz_userclk_tx_srcclk_out(0),
--    DIV => "000",
--CLRMASK => '0',
--    CLR => gtwiz_userclk_tx_reset_in(0),
-- CEMASK => '0',
--     CE => '1',
--      O => ref_clk);
  ref_clk <= gtwiz_userclk_tx_usrclk2_out(0);
  rref_clk <= gtwiz_userclk_rx_usrclk2_out(0);
 
  pll_locked     <= '1' when rxpmaresetdone_out = "1111" and txpmaresetdone_out = "1111" else '0';
    
  usrrdy <= gtwiz_reset_rx_cdr_stable_out(0); 

  usrclk_in <= gtp_clk & gtp_clk & gtp_clk & gtp_clk;
  --drpclk_in <= drpclk & drpclk & drpclk & drpclk;
  drpclk_in <= drpclk & drpclk & drpclk & drpclk;
  refclk_in <= clkin & clkin & clkin & clkin;

  gtwiz_userclk_tx_reset_in(0) <= '1' when txpmaresetdone_out /= "1111" and rxpmaresetdone_out /= "1111" else '0';
  gtwiz_reset_all_in(0) <= pllrst;
  gtwiz_userclk_rx_reset_in(0) <= '1' when rxpmaresetdone_out /= "1111" and txpmaresetdone_out /= "1111"  else '0';
  
  gtwizard_ultrascale_0_example_wrapper_1: gtwizard_ultrascale_0_example_wrapper
    port map (
      gtwiz_userclk_tx_reset_in          => gtwiz_userclk_tx_reset_in,
      gtwiz_userclk_tx_srcclk_out        => gtwiz_userclk_tx_srcclk_out,
      gtwiz_userclk_tx_usrclk_out        => gtwiz_userclk_tx_usrclk_out,
      gtwiz_userclk_tx_usrclk2_out       => gtwiz_userclk_tx_usrclk2_out,
      gtwiz_userclk_tx_active_out        => gtwiz_userclk_tx_active_out,
      gtwiz_reset_clk_freerun_in(0)         => drpclk,
      gtwiz_reset_all_in                 => gtwiz_reset_all_in,
      gtwiz_reset_tx_pll_and_datapath_in => gtwiz_reset_tx_pll_and_datapath_in,
      gtwiz_reset_tx_datapath_in         => gtwiz_reset_tx_datapath_in,
      gtwiz_reset_rx_pll_and_datapath_in => gtwiz_reset_rx_pll_and_datapath_in,
      gtwiz_reset_rx_datapath_in         => gtwiz_reset_rx_datapath_in,
      gtwiz_reset_rx_cdr_stable_out      => gtwiz_reset_rx_cdr_stable_out,
      gtwiz_reset_tx_done_out            => gtwiz_reset_tx_done_out,
      gtwiz_reset_rx_done_out            => gtwiz_reset_rx_done_out,
      gtwiz_userclk_rx_reset_in          => gtwiz_userclk_rx_reset_in,
      gtwiz_userclk_rx_srcclk_out        => gtwiz_userclk_rx_srcclk_out,
      gtwiz_userclk_rx_usrclk_out        => gtwiz_userclk_rx_usrclk_out,
      gtwiz_userclk_rx_usrclk2_out       => gtwiz_userclk_rx_usrclk2_out,
      gtwiz_userclk_rx_active_out        => gtwiz_userclk_rx_active_out,
 
    gtwiz_userdata_tx_in               => gtp_data_in,
    gtwiz_userdata_rx_out              => gtp_data_out,
    drpclk_in                          => drpclk_in,
    gthrxn_in                          => rx_n,
    gthrxp_in                          => rx_p,
    gtrefclk0_in                       => refclk_in,
    rx8b10ben_in                       => "1111",
    rxcommadeten_in                    => "1111",
    rxmcommaalignen_in                 => align,
    rxpcommaalignen_in                 => align,
    tx8b10ben_in                       => "1111",
    txctrl0_in                         => txctrl0_in,
    txctrl1_in                         => txctrl1_in,
    txctrl2_in                         => txctrl2_in,
    gthtxn_out                         => tx_n,
    gthtxp_out                         => tx_p,
    rxbyteisaligned_out                => open,
    rxbyterealign_out                  => open,
    rxcommadet_out                     => open,
    rxctrl0_out                        => rxctrl0_out,
    rxctrl1_out                        => rxctrl1_out,
    rxctrl2_out                        => rxctrl2_out,
    rxctrl3_out                        => rxctrl3_out,
    rxpmaresetdone_out                 => rxpmaresetdone_out,
    txpmaresetdone_out                 => txpmaresetdone_out);

  txctrl2_in(3 downto 0) <= gtp_charisk_in(3 downto 0);
  txctrl2_in(11 downto 8) <= gtp_charisk_in(7 downto 4);
  txctrl2_in(19 downto 16) <= gtp_charisk_in(11 downto 8);
  txctrl2_in(27 downto 24) <= gtp_charisk_in(15 downto 12);
  
  gtp_charisk_out(3 downto 0) <= rxctrl2_out(3 downto 0);
  gtp_charisk_out(7 downto 4) <= rxctrl2_out(11 downto 8);
  gtp_charisk_out(11 downto 8) <= rxctrl2_out(19 downto 16);
  gtp_charisk_out(15 downto 12) <= rxctrl2_out(27 downto 24);
  

    align       <= "1111" when gtwiz_reset_rx_done_out(0) = '1' and en_comma_align = '1' else "0000";
    rx_status(0) <= gtwiz_reset_rx_done_out(0);

    
  
    rx_loss_of_sync(1) <= OR_reduce(rxctrl3_out(3 downto 0));
    rx_loss_of_sync(0) <= '0';
    rx_loss_of_sync(3) <= OR_reduce(rxctrl3_out(11 downto 8));
    rx_loss_of_sync(2) <= '0';
    rx_loss_of_sync(5) <= OR_reduce(rxctrl3_out(19 downto 16));
    rx_loss_of_sync(4) <= '0';
    rx_loss_of_sync(7) <= OR_reduce(rxctrl3_out(27 downto 24));
    rx_loss_of_sync(6) <= '0';
   

  rx_loss_of_sync(15 downto 8) <= (others => '0');
  
end ku_x4;
