
library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.mptl.all;
use work.mptl_pins.all;

entity mptl128_interface_target_ku_slv is
  generic (
    enable_bridge_master  : boolean := true;
    enable_target_master  : boolean := false;
    enable_dma     : boolean := true;
    wrapper_reset_signals : boolean := true;
    break_out_ocp : boolean := false;
    behav_sim_ocp_over_mptl : boolean := false
    );
  port (
    -- OCP Interface, packed into single SLV
    ocp_clk            : in  std_logic;
    ds_m2s   : out std_logic_vector(232 downto 0);
    ds_s2m   : in  std_logic_vector(139 downto 0);
    dma_m2s   : out std_logic_vector(465 downto 0);
    dma_s2m   : in  std_logic_vector(279 downto 0);
    dma_abort          : in  std_logic_vector(1 downto 0);
    abmi_m2s           : in  std_logic_vector(232 downto 0);
    abmi_s2m           : out std_logic_vector(139 downto 0);
    -- MPTL Interface
    mptl_tx_p          : out std_logic_vector(3 downto 0);
    mptl_tx_n          : out std_logic_vector(3 downto 0);
    mptl_rx_p          : in  std_logic_vector(3 downto 0);
    mptl_rx_n          : in  std_logic_vector(3 downto 0);
    mptl_clk_p         : in  std_logic;
    mptl_clk_n         : in  std_logic;
    mptl_clk_out       : out std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n            : out std_logic;
    mptl_target_gtp_online_n            : out std_logic;
    mptl_bridge_gtp_online_n            : in  std_logic;
    -- MPTL128 Sideband Interface
    gpio_t2b           : in  std_logic_vector(15 downto 0);
    gpio_b2t           : out std_logic_vector(15 downto 0);
    -- Extra Wrapper Signals
    ocp_ready          : in  std_logic;
    mptl_ready         : out std_logic;
    -- Broken Out OCP Signals
    ds_m2s_cmd   : out std_logic_vector(2 downto 0);
    ds_m2s_addr   : out std_logic_vector(63 downto 0);
    ds_m2s_burstlength : out std_logic_vector(11 downto 0);
    ds_m2s_data   : out std_logic_vector(127 downto 0);
    ds_m2s_databyteen   : out std_logic_vector(15 downto 0);
    ds_m2s_datavalid   : out std_logic;
    ds_m2s_respaccept  : out std_logic;
    ds_m2s_tag   : out std_logic_vector(7 downto 0);
    ds_s2m_cmdaccept   : in  std_logic;
    ds_s2m_dataaccept   : in  std_logic;
    ds_s2m_data   : in  std_logic_vector(127 downto 0);
    ds_s2m_resp   : in  std_logic_vector(1 downto 0);
    ds_s2m_tag   : in  std_logic_vector(7 downto 0);

    
    dma0_m2s_cmd   : out std_logic_vector(2 downto 0);
    dma0_m2s_addr   : out std_logic_vector(63 downto 0);
    dma0_m2s_burstlength : out std_logic_vector(11 downto 0);
    dma0_m2s_data   : out std_logic_vector(127 downto 0);
    dma0_m2s_databyteen   : out std_logic_vector(15 downto 0);
    dma0_m2s_datavalid   : out std_logic;
    dma0_m2s_respaccept  : out std_logic;
    dma0_m2s_tag   : out std_logic_vector(7 downto 0);
    dma0_s2m_cmdaccept   : in  std_logic;
    dma0_s2m_dataaccept   : in  std_logic;
    dma0_s2m_data   : in  std_logic_vector(127 downto 0);
    dma0_s2m_resp   : in  std_logic_vector(1 downto 0);
    dma0_s2m_tag   : in  std_logic_vector(7 downto 0);

    dma1_m2s_cmd   : out std_logic_vector(2 downto 0);
    dma1_m2s_addr   : out std_logic_vector(63 downto 0);
    dma1_m2s_burstlength : out std_logic_vector(11 downto 0);
    dma1_m2s_data   : out std_logic_vector(127 downto 0);
    dma1_m2s_databyteen   : out std_logic_vector(15 downto 0);
    dma1_m2s_datavalid   : out std_logic;
    dma1_m2s_respaccept  : out std_logic;
    dma1_m2s_tag   : out std_logic_vector(7 downto 0);
    dma1_s2m_cmdaccept   : in  std_logic;
    dma1_s2m_dataaccept   : in  std_logic;
    dma1_s2m_data   : in  std_logic_vector(127 downto 0);
    dma1_s2m_resp   : in  std_logic_vector(1 downto 0);
    dma1_s2m_tag   : in  std_logic_vector(7 downto 0);

    abmi_m2s_cmd   : in std_logic_vector(2 downto 0);
    abmi_m2s_addr   : in std_logic_vector(63 downto 0);
    abmi_m2s_burstlength : in std_logic_vector(11 downto 0);
    abmi_m2s_data   : in std_logic_vector(127 downto 0);
    abmi_m2s_databyteen   : in std_logic_vector(15 downto 0);
    abmi_m2s_datavalid   : in std_logic;
    abmi_m2s_respaccept  : in std_logic;
    abmi_m2s_tag   : in std_logic_vector(7 downto 0);
    abmi_s2m_cmdaccept   : out  std_logic;
    abmi_s2m_dataaccept   : out  std_logic;
    abmi_s2m_data   : out  std_logic_vector(127 downto 0);
    abmi_s2m_resp   : out  std_logic_vector(1 downto 0);
    abmi_s2m_tag   : out  std_logic_vector(7 downto 0)

    
    );

end mptl128_interface_target_ku_slv;

architecture rtl of mptl128_interface_target_ku_slv is

  signal ds_m2s_i   : adb3_ocp_m2sT;
  signal ds_s2m_i   : adb3_ocp_s2mT;
  signal dma_m2s_i   : adb3_ocp_m2sT_array(0 to 1);
  signal dma_s2m_i   : adb3_ocp_s2mT_array(0 to 1);
  signal abmi_m2s_i   : adb3_ocp_m2sT;
  signal abmi_s2m_i   : adb3_ocp_s2mT;


   -- MPTL sideband
  signal local_mptl_target_configured_n     : std_logic;
  signal local_mptl_target_gtp_online_n : std_logic;
  signal local_mptl_bridge_gtp_online_n : std_logic;

  -- MPTL ready logic
  signal mptl_bridge_gtp_online_sr_n : std_logic_vector(31 downto 0) := (others => '1');
  signal mptl_target_gtp_online_sr_n : std_logic_vector(31 downto 0) := (others => '1');
  signal i_mptl_ready : std_logic := '0';
  signal mptl_clk_out_i : std_logic;


  function behav_sim_decision_func(
    constant behav_sim : boolean := false)
    return boolean is
  begin
    return behav_sim;
  end;

  constant behav_sim : boolean := behav_sim_decision_func
  -- synthesis translate_off
  (behav_sim => behav_sim_ocp_over_mptl)
  -- synthesis translate_on
  ;

  component mptl_tgt_ku_slv_bm is
    generic (
      enable_bridge_master  : boolean;
      enable_target_master  : boolean;
      enable_dma            : boolean;
      wrapper_reset_signals : boolean;
      break_out_ocp         : boolean);
    port (
      mptl_tx_p                : out std_logic_vector(3 downto 0);
      mptl_tx_n                : out std_logic_vector(3 downto 0);
      mptl_rx_p                : in  std_logic_vector(3 downto 0);
      mptl_rx_n                : in  std_logic_vector(3 downto 0);
      mptl_clk_p               : in  std_logic;
      mptl_clk_n               : in  std_logic;
      mptl_clk_out             : out std_logic;
      mptl_target_configured_n : out std_logic;
      mptl_target_gtp_online_n : out std_logic;
      mptl_bridge_gtp_online_n : in  std_logic;
      ocp_ready                : in  std_logic;
      mptl_ready               : out std_logic;
      ocp_clk                  : in  std_logic;
      ds_m2s                   : out std_logic_vector(232 downto 0);
      ds_s2m                   : in  std_logic_vector(139 downto 0) := (others => '0');
      dma_m2s                  : out std_logic_vector(465 downto 0);
      dma_s2m                  : in  std_logic_vector(279 downto 0) := (others => '0');
      dma_abort                : in  std_logic_vector(1 downto 0)   := (others => '0');
      abmi_m2s                 : in  std_logic_vector(232 downto 0) := (others => '0');
      abmi_s2m                 : out std_logic_vector(139 downto 0);
      gpio_t2b                 : in  std_logic_vector(15 downto 0);
      gpio_b2t                 : out std_logic_vector(15 downto 0);
      ds_m2s_cmd               : out std_logic_vector(2 downto 0);
      ds_m2s_addr              : out std_logic_vector(63 downto 0);
      ds_m2s_burstlength       : out std_logic_vector(11 downto 0);
      ds_m2s_data              : out std_logic_vector(127 downto 0);
      ds_m2s_databyteen        : out std_logic_vector(15 downto 0);
      ds_m2s_datavalid         : out std_logic;
      ds_m2s_respaccept        : out std_logic;
      ds_m2s_tag               : out std_logic_vector(7 downto 0);
      ds_s2m_cmdaccept         : in  std_logic                      := '0';
      ds_s2m_dataaccept        : in  std_logic                      := '0';
      ds_s2m_data              : in  std_logic_vector(127 downto 0) := (others => '0');
      ds_s2m_resp              : in  std_logic_vector(1 downto 0)   := (others => '0');
      ds_s2m_tag               : in  std_logic_vector(7 downto 0)   := (others => '0');
      dma0_m2s_cmd             : out std_logic_vector(2 downto 0);
      dma0_m2s_addr            : out std_logic_vector(63 downto 0);
      dma0_m2s_burstlength     : out std_logic_vector(11 downto 0);
      dma0_m2s_data            : out std_logic_vector(127 downto 0);
      dma0_m2s_databyteen      : out std_logic_vector(15 downto 0);
      dma0_m2s_datavalid       : out std_logic;
      dma0_m2s_respaccept      : out std_logic;
      dma0_m2s_tag             : out std_logic_vector(7 downto 0);
      dma0_s2m_cmdaccept       : in  std_logic                      := '0';
      dma0_s2m_dataaccept      : in  std_logic                      := '0';
      dma0_s2m_data            : in  std_logic_vector(127 downto 0) := (others => '0');
      dma0_s2m_resp            : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma0_s2m_tag             : in  std_logic_vector(7 downto 0)   := (others => '0');
      dma1_m2s_cmd             : out std_logic_vector(2 downto 0);
      dma1_m2s_addr            : out std_logic_vector(63 downto 0);
      dma1_m2s_burstlength     : out std_logic_vector(11 downto 0);
      dma1_m2s_data            : out std_logic_vector(127 downto 0);
      dma1_m2s_databyteen      : out std_logic_vector(15 downto 0);
      dma1_m2s_datavalid       : out std_logic;
      dma1_m2s_respaccept      : out std_logic;
      dma1_m2s_tag             : out std_logic_vector(7 downto 0);
      dma1_s2m_cmdaccept       : in  std_logic                      := '0';
      dma1_s2m_dataaccept      : in  std_logic                      := '0';
      dma1_s2m_data            : in  std_logic_vector(127 downto 0) := (others => '0');
      dma1_s2m_resp            : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma1_s2m_tag             : in  std_logic_vector(7 downto 0)   := (others => '0');
      abmi_m2s_cmd             : in  std_logic_vector(2 downto 0)   := (others => '0');
      abmi_m2s_addr            : in  std_logic_vector(63 downto 0)  := (others => '0');
      abmi_m2s_burstlength     : in  std_logic_vector(11 downto 0)  := (others => '0');
      abmi_m2s_data            : in  std_logic_vector(127 downto 0) := (others => '0');
      abmi_m2s_databyteen      : in  std_logic_vector(15 downto 0)  := (others => '0');
      abmi_m2s_datavalid       : in  std_logic                      := '0';
      abmi_m2s_respaccept      : in  std_logic                      := '0';
      abmi_m2s_tag             : in  std_logic_vector(7 downto 0)   := (others => '0');
      abmi_s2m_cmdaccept       : out std_logic;
      abmi_s2m_dataaccept      : out std_logic;
      abmi_s2m_data            : out std_logic_vector(127 downto 0);
      abmi_s2m_resp            : out std_logic_vector(1 downto 0);
      abmi_s2m_tag             : out std_logic_vector(7 downto 0));
  end component mptl_tgt_ku_slv_bm;
  
begin  -- rtl

  gen_synth: if behav_sim = false generate

     
  gen_packed_signal_arrays: if not break_out_ocp generate
    ds_s2m_i <= to_adb3_ocp_s2mT(ds_s2m);
    ds_m2s <= to_std_logic_vector(ds_m2s_i);

    dma_s2m_i <= to_adb3_ocp_s2mT_array(dma_s2m);
    dma_m2s <= to_std_logic_vector(dma_m2s_i); 

    abmi_m2s_i <= to_adb3_ocp_m2sT(abmi_m2s);
    abmi_s2m <= to_std_logic_vector(abmi_s2m_i);

    ds_m2s_cmd <= (others => '0');
    ds_m2s_addr <= (others => '0');
    ds_m2s_burstlength <= (others => '0');
    ds_m2s_data <= (others => '0');
    ds_m2s_databyteen <= (others => '0');
    ds_m2s_datavalid <= '0';
    ds_m2s_respaccept <= '0';
    ds_m2s_tag <= (others => '0');
    

    dma0_m2s_cmd <= (others => '0');
    dma0_m2s_addr <= (others => '0');
    dma0_m2s_burstlength <= (others => '0');
    dma0_m2s_data <= (others => '0');
    dma0_m2s_databyteen <= (others => '0');
    dma0_m2s_datavalid <= '0';
    dma0_m2s_respaccept <= '0';
    dma0_m2s_tag <= (others => '0');

    dma1_m2s_cmd <= (others => '0');
    dma1_m2s_addr <= (others => '0');
    dma1_m2s_burstlength <= (others => '0');
    dma1_m2s_data <= (others => '0');
    dma1_m2s_databyteen <= (others => '0');
    dma1_m2s_datavalid <= '0';
    dma1_m2s_respaccept <= '0';
    dma1_m2s_tag <= (others => '0');

    abmi_s2m_cmdaccept <= '0';
    abmi_s2m_dataaccept <= '0';
    abmi_s2m_data <= (others => '0');
    abmi_s2m_resp <= (others => '0');
    abmi_s2m_tag <= (others => '0');
    
  end generate gen_packed_signal_arrays;
  
  
  gen_broken_out_signals: if break_out_ocp generate
    ds_s2m_i.CmdAccept <= ds_s2m_cmdaccept;
    ds_s2m_i.DataAccept <= ds_s2m_dataaccept;
    ds_s2m_i.Data <= ds_s2m_data;
    ds_s2m_i.Resp <= ds_s2m_resp;
    ds_s2m_i.Tag <= ds_s2m_tag;
    ds_m2s_cmd <= ds_m2s_i.Cmd;
    ds_m2s_addr <= ds_m2s_i.Addr;
    ds_m2s_burstlength <= ds_m2s_i.BurstLength;
    ds_m2s_data <= ds_m2s_i.Data;
    ds_m2s_databyteen <= ds_m2s_i.DataByteEn;
    ds_m2s_datavalid <= ds_m2s_i.DataValid;
    ds_m2s_respaccept <= ds_m2s_i.RespAccept;
    ds_m2s_tag <= ds_m2s_i.Tag;

    dma_s2m_i(0).CmdAccept <= dma0_s2m_cmdaccept;
    dma_s2m_i(0).DataAccept <= dma0_s2m_dataaccept;
    dma_s2m_i(0).Data <= dma0_s2m_data;
    dma_s2m_i(0).Resp <= dma0_s2m_resp;
    dma_s2m_i(0).Tag <= dma0_s2m_tag;
    dma0_m2s_cmd <= dma_m2s_i(0).Cmd;
    dma0_m2s_addr <= dma_m2s_i(0).Addr;
    dma0_m2s_burstlength <= dma_m2s_i(0).BurstLength;
    dma0_m2s_data <= dma_m2s_i(0).Data;
    dma0_m2s_databyteen <= dma_m2s_i(0).DataByteEn;
    dma0_m2s_datavalid <= dma_m2s_i(0).DataValid;
    dma0_m2s_respaccept <= dma_m2s_i(0).RespAccept;
    dma0_m2s_tag <= dma_m2s_i(0).Tag;

    dma_s2m_i(1).CmdAccept <= dma1_s2m_cmdaccept;
    dma_s2m_i(1).DataAccept <= dma1_s2m_dataaccept;
    dma_s2m_i(1).Data <= dma1_s2m_data;
    dma_s2m_i(1).Resp <= dma1_s2m_resp;
    dma_s2m_i(1).Tag <= dma1_s2m_tag;
    dma1_m2s_cmd <= dma_m2s_i(1).Cmd;
    dma1_m2s_addr <= dma_m2s_i(1).Addr;
    dma1_m2s_burstlength <= dma_m2s_i(1).BurstLength;
    dma1_m2s_data <= dma_m2s_i(1).Data;
    dma1_m2s_databyteen <= dma_m2s_i(1).DataByteEn;
    dma1_m2s_datavalid <= dma_m2s_i(1).DataValid;
    dma1_m2s_respaccept <= dma_m2s_i(1).RespAccept;
    dma1_m2s_tag <= dma_m2s_i(1).Tag;


    abmi_s2m_cmdaccept <= abmi_s2m_i.CmdAccept;
    abmi_s2m_dataaccept <= abmi_s2m_i.DataAccept;
    abmi_s2m_data <= abmi_s2m_i.Data;
    abmi_s2m_resp <= abmi_s2m_i.Resp;
    abmi_s2m_tag <= abmi_s2m_i.Tag;
    abmi_m2s_i.Cmd <= abmi_m2s_cmd;
    abmi_m2s_i.Addr <= abmi_m2s_addr;
    abmi_m2s_i.BurstLength <= abmi_m2s_burstlength;
    abmi_m2s_i.Data <= abmi_m2s_data;
    abmi_m2s_i.DataByteEn <= abmi_m2s_databyteen;
    abmi_m2s_i.DataValid <= abmi_m2s_datavalid;
    abmi_m2s_i.RespAccept <= abmi_m2s_respaccept;
    abmi_m2s_i.Tag <= abmi_m2s_tag;



    ds_m2s <= (others => '0');
    dma_m2s <= (others => '0');
    abmi_s2m <= (others => '0');

    
  end generate gen_broken_out_signals;





  mptl_clk_out <= mptl_clk_out_i;
  
  mptl128_interface_target_ku_1: entity work.mptl128_interface_target_ku
     generic map (
       enable_bridge_master => enable_bridge_master,
       enable_target_master => enable_target_master,
       enable_dma => enable_dma)
    port map (
      ocp_clk            => ocp_clk,
      direct_slave_m2s   => ds_m2s_i,
      direct_slave_s2m   => ds_s2m_i,
      dma_channels_m2s   => dma_m2s_i,
      dma_channels_s2m   => dma_s2m_i,
      dma_abort          => dma_abort,
      abmi_m2s   => abmi_m2s_i,
      abmi_s2m   => abmi_s2m_i,
      mptl_tx.p            => mptl_tx_p,
      mptl_tx.n            => mptl_tx_n,    
      mptl_rx.p            => mptl_rx_p,
      mptl_rx.n            => mptl_rx_n,
      mptl_clk.p           => mptl_clk_p,
      mptl_clk.n           => mptl_clk_n,
      mptl_clk_out         => mptl_clk_out_i,
      mptl_target_configured_n  =>   local_mptl_target_configured_n,    
      mptl_target_gtp_online_n    =>    local_mptl_target_gtp_online_n, 
      mptl_bridge_gtp_online_n    =>   local_mptl_bridge_gtp_online_n,  
      gpio_t2b           => gpio_t2b,
      gpio_b2t           => gpio_b2t);




  gen_no_reset_sigs: if not wrapper_reset_signals generate
    mptl_target_configured_n <= local_mptl_target_configured_n;
    mptl_target_gtp_online_n <= local_mptl_target_gtp_online_n;
    local_mptl_bridge_gtp_online_n <= mptl_bridge_gtp_online_n;
  end generate gen_no_reset_sigs;
  

  gen_reset_sigs: if wrapper_reset_signals generate
     
  -- Hold off MPTL target configured until all OCP clocks are stable (resets inactive)
  
  mptl_target_configured_n <= local_mptl_target_configured_n OR NOT ocp_ready;

  -- Connect MPTL initialisation signals
  mptl_target_gtp_online_n <= local_mptl_target_gtp_online_n;
  local_mptl_bridge_gtp_online_n <= mptl_bridge_gtp_online_n;

  -- 'mptl_ready' output can be used to reset application logic, but must ignore short retry pulses
  gen_mptl_ready : process (mptl_clk_out_i)
  begin  -- process
    if rising_edge(mptl_clk_out_i) then
       mptl_bridge_gtp_online_sr_n <= mptl_bridge_gtp_online_sr_n(30 downto 0) & local_mptl_bridge_gtp_online_n;
       mptl_target_gtp_online_sr_n <= mptl_target_gtp_online_sr_n(30 downto 0) & local_mptl_target_gtp_online_n;
       i_mptl_ready <= NOT (mptl_bridge_gtp_online_sr_n(0) AND mptl_bridge_gtp_online_sr_n(31)) AND
                       NOT (mptl_target_gtp_online_sr_n(0) AND mptl_target_gtp_online_sr_n(31));
    end if;
  end process;

  mptl_ready <= i_mptl_ready;

  end generate gen_reset_sigs;

   end generate gen_synth;


  --synthesis translate_off
  gen_fast_sim: if behav_sim = true generate

    mptl_tgt_ku_slv_bm_i : mptl_tgt_ku_slv_bm
    generic map(
      enable_bridge_master  => enable_bridge_master,  -- boolean := true
      enable_target_master  => enable_target_master,  -- boolean := false
      enable_dma            => enable_dma,            -- boolean := true
      wrapper_reset_signals => wrapper_reset_signals, -- boolean := true
      break_out_ocp         => break_out_ocp )        -- boolean := false
    port map(
      -- MPTL
      mptl_tx_p                => mptl_tx_p,                -- out   std_logic_vector(3 downto 0)
      mptl_tx_n                => mptl_tx_n,                -- out   std_logic_vector(3 downto 0)
      mptl_rx_p                => mptl_rx_p,                -- in    std_logic_vector(3 downto 0)
      mptl_rx_n                => mptl_rx_n,                -- in    std_logic_vector(3 donwto 0)
      mptl_clk_p               => mptl_clk_p,               -- in    std_logic
      mptl_clk_n               => mptl_clk_n,               -- in    std_logic
      mptl_clk_out             => mptl_clk_out,             -- out   std_logic
      -- MPTL initialisation
      mptl_target_configured_n => local_mptl_target_configured_n, -- out   std_logic
      mptl_target_gtp_online_n => local_mptl_target_gtp_online_n, -- out   std_logic
      mptl_bridge_gtp_online_n => local_mptl_bridge_gtp_online_n, -- in    std_logic

      -- Extra Wrapper Signals
      ocp_ready                => ocp_ready,  -- in    std_logic
      mptl_ready               => mptl_ready, -- out   std_logic

      -- Signals that are replicated between Target and Host via mptl_{tx,rx}
      -- AXI4                              
      ocp_clk   => ocp_clk,   -- in    std_logic
      ds_m2s    => ds_m2s,    -- out   std_logic_vector(232 downto 0)
      ds_s2m    => ds_s2m,    -- in    std_logic_vector(139 downto 0) := (others => '0')
      dma_m2s   => dma_m2s,   -- out   std_logic_vector(465 downto 0)
      dma_s2m   => dma_s2m,   -- in    std_logic_vector(279 downto 0) := (others => '0')
      dma_abort => dma_abort, -- in    std_logic_vector(1 downto 0) := (others => '0')
      abmi_m2s  => abmi_m2s,  -- in    std_logic_vector(232 downto 0) := (others => '0')
      abmi_s2m  => abmi_s2m,  -- out   std_logic_vector(139 downto 0)
      -- MPTL sideband                        
      gpio_t2b => gpio_t2b, -- in    std_logic_vector(15 downto 0)
      gpio_b2t => gpio_b2t, -- out   std_logic_vector(15 downto 0)

      -- Broken Out OCP Signals
      ds_m2s_cmd           => ds_m2s_cmd,         -- out   std_logic_vector(2 downto 0)
      ds_m2s_addr          => ds_m2s_addr,        -- out   std_logic_vector(63 downto 0)
      ds_m2s_burstlength   => ds_m2s_burstlength, -- out   std_logic_vector(11 downto 0)
      ds_m2s_data          => ds_m2s_data,        -- out   std_logic_vector(127 downto 0)
      ds_m2s_databyteen    => ds_m2s_databyteen,  -- out   std_logic_vector(15 downto 0)
      ds_m2s_datavalid     => ds_m2s_datavalid,   -- out   std_logic
      ds_m2s_respaccept    => ds_m2s_respaccept,  -- out   std_logic
      ds_m2s_tag           => ds_m2s_tag,         -- out   std_logic_vector(7 downto 0)
      ds_s2m_cmdaccept     => ds_s2m_cmdaccept,   -- in    std_logic := '0'
      ds_s2m_dataaccept    => ds_s2m_dataaccept,  -- in    std_logic := '0'
      ds_s2m_data          => ds_s2m_data,        -- in    std_logic_vector(127 downto 0) := (others => '0')
      ds_s2m_resp          => ds_s2m_resp,        -- in    std_logic_vector(1 downto 0) := (others => '0')
      ds_s2m_tag           => ds_s2m_tag,         -- in    std_logic_vector(7 downto 0) := (others => '0')
   
      dma0_m2s_cmd         => dma0_m2s_cmd,         -- out   std_logic_vector(2 downto 0)
      dma0_m2s_addr        => dma0_m2s_addr,        -- out   std_logic_vector(63 downto 0)
      dma0_m2s_burstlength => dma0_m2s_burstlength, -- out   std_logic_vector(11 downto 0)
      dma0_m2s_data        => dma0_m2s_data,        -- out   std_logic_vector(127 downto 0)
      dma0_m2s_databyteen  => dma0_m2s_databyteen,  -- out   std_logic_vector(15 downto 0)
      dma0_m2s_datavalid   => dma0_m2s_datavalid,   -- out   std_logic
      dma0_m2s_respaccept  => dma0_m2s_respaccept,  -- out   std_logic
      dma0_m2s_tag         => dma0_m2s_tag,         -- out   std_logic_vector(7 downto 0)
      dma0_s2m_cmdaccept   => dma0_s2m_cmdaccept,   -- in    std_logic := '0'
      dma0_s2m_dataaccept  => dma0_s2m_dataaccept,  -- in    std_logic := '0'
      dma0_s2m_data        => dma0_s2m_data,        -- in    std_logic_vector(127 downto 0) := (others => '0')
      dma0_s2m_resp        => dma0_s2m_resp,        -- in    std_logic_vector(1 downto 0) := (others => '0')
      dma0_s2m_tag         => dma0_s2m_tag,         -- in    std_logic_vector(7 downto 0) := (others => '0')
     
      dma1_m2s_cmd         => dma1_m2s_cmd,         -- out   std_logic_vector(2 downto 0)
      dma1_m2s_addr        => dma1_m2s_addr,        -- out   std_logic_vector(63 downto 0)
      dma1_m2s_burstlength => dma1_m2s_burstlength, -- out   std_logic_vector(11 downto 0)
      dma1_m2s_data        => dma1_m2s_data,        -- out   std_logic_vector(127 downto 0)
      dma1_m2s_databyteen  => dma1_m2s_databyteen,  -- out   std_logic_vector(15 downto 0)
      dma1_m2s_datavalid   => dma1_m2s_datavalid,   -- out   std_logic
      dma1_m2s_respaccept  => dma1_m2s_respaccept,  -- out   std_logic
      dma1_m2s_tag         => dma1_m2s_tag,         -- out   std_logic_vector(7 downto 0)
      dma1_s2m_cmdaccept   => dma1_s2m_cmdaccept,   -- in    std_logic := '0'
      dma1_s2m_dataaccept  => dma1_s2m_dataaccept,  -- in    std_logic := '0'
      dma1_s2m_data        => dma1_s2m_data,        -- in    std_logic_vector(127 downto 0) := (others => '0')
      dma1_s2m_resp        => dma1_s2m_resp,        -- in    std_logic_vector(1 downto 0) := (others => '0')
      dma1_s2m_tag         => dma1_s2m_tag,         -- in    std_logic_vector(7 downto 0) := (others => '0')

      abmi_m2s_cmd         => abmi_m2s_cmd,         -- in    std_logic_vector(2 downto 0) := (others => '0')
      abmi_m2s_addr        => abmi_m2s_addr,        -- in    std_logic_vector(63 downto 0) := (others => '0')
      abmi_m2s_burstlength => abmi_m2s_burstlength, -- in    std_logic_vector(11 downto 0) := (others => '0')
      abmi_m2s_data        => abmi_m2s_data,        -- in    std_logic_vector(127 downto 0) := (others => '0')
      abmi_m2s_databyteen  => abmi_m2s_databyteen,  -- in    std_logic_vector(15 downto 0) := (others => '0')
      abmi_m2s_datavalid   => abmi_m2s_datavalid,   -- in    std_logic := '0'
      abmi_m2s_respaccept  => abmi_m2s_respaccept,  -- in    std_logic := '0'
      abmi_m2s_tag         => abmi_m2s_tag,         -- in    std_logic_vector(7 downto 0) := (others => '0')
      abmi_s2m_cmdaccept   => abmi_s2m_cmdaccept,   -- out   std_logic
      abmi_s2m_dataaccept  => abmi_s2m_dataaccept,  -- out   std_logic
      abmi_s2m_data        => abmi_s2m_data,        -- out   std_logic_vector(127 downto 0)
      abmi_s2m_resp        => abmi_s2m_resp,        -- out   std_logic_vector(1 downto 0)
      abmi_s2m_tag       => abmi_s2m_tag);         -- out   std_logic_vector(7 downto 0)

  end generate gen_fast_sim;
  --synthesis translate_on

  
end rtl;
