#include "factory.h"
#include "graphwndtoolbar.h"

#include "icon/amp_sm.h"
#include "icon/bool_sm.h"
#include "icon/close_sm.h"
#include "icon/go_sm.h"
#include "icon/pause_sm.h"
#include "icon/plus_sm.h"
#include "icon/temp_sm.h"
#include "icon/volt_sm.h"

// -------------------------------------------------------------------------
// Start of IGraphWndToolbar interface
// -------------------------------------------------------------------------

void
CGraphWndToolbar::deviceChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  SensorInfo* pSensorInfo;
  bool_t bEnableTemp = FALSE;
  bool_t bEnableVolt = FALSE;
  bool_t bEnableAmp = FALSE;
  bool_t bEnableBool = FALSE;

  m_pParent->getScope()->removeAllTraces();
  m_pParent->getScope()->addTraceDone();

  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    pSensorInfo = &devInfo.m_pSensors[i];
    if (pSensorInfo->m_bValid && pSensorInfo->m_bKnownDataType) {
      switch (pSensorInfo->m_info.Unit) {
      case ADMXRC3_UNIT_C:
        bEnableTemp = TRUE;
        break;

      case ADMXRC3_UNIT_V:
        bEnableVolt = TRUE;
        break;

      case ADMXRC3_UNIT_A:
        bEnableAmp = TRUE;
        break;

      case ADMXRC3_DATA_BOOL:
        bEnableBool = TRUE;
        break;

      default:
        break;
      }
    }
  }
  m_tempCtl.set_sensitive(bEnableTemp);
  m_voltCtl.set_sensitive(bEnableVolt);
  m_ampCtl.set_sensitive(bEnableAmp);
  m_boolCtl.set_sensitive(bEnableBool);
}

void
CGraphWndToolbar::keyClosed()
{
  if (m_keyCtl.get_active()) {
    m_keyCtl.set_active(false);
  }
}

void
CGraphWndToolbar::numGraphWndChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  unsigned int n;

  n = pMainWnd->getNumGraphWnd();
  m_plusCtl.set_sensitive(n < 4);
  m_closeCtl.set_sensitive(n >= 2);
}

void
CGraphWndToolbar::numTraceChanged()
{
  unsigned int n = m_pParent->getScope()->getNumTrace();

  if (n > 0) {
    m_keyCtl.set_sensitive(true);
    m_pauseCtl.set_sensitive(true);
  } else {
    if (m_keyCtl.get_active()) {
      m_keyCtl.set_active(false);
    }
    m_pauseCtl.set_sensitive(false);
    m_keyCtl.set_sensitive(false);
  }
}

void
CGraphWndToolbar::setIndex(
  unsigned int index)
{
  std::ostringstream ss;

  m_index = index;
  ss << (m_index + 1);
  m_labelCtl.set_text(ss.str());
}

void
CGraphWndToolbar::tracesChanged()
{
  m_pScopeKey->tracesChanged();
}

// -------------------------------------------------------------------------
// End of IGraphWndToolbar interface
// -------------------------------------------------------------------------

CGraphWndToolbar::CGraphWndToolbar(
  IGraphWnd* pParent,
  unsigned int index) :
  m_ampCtl(sizeof(g_raw_amp_sm), g_raw_amp_sm),
  m_boolCtl(sizeof(g_raw_bool_sm), g_raw_bool_sm),
  m_closeCtl(sizeof(g_raw_close_sm), g_raw_close_sm),
  m_pauseCtl(sizeof(g_raw_go_sm), g_raw_go_sm, sizeof(g_raw_pause_sm), g_raw_pause_sm),
  m_plusCtl(sizeof(g_raw_plus_sm), g_raw_plus_sm),
  m_tempCtl(sizeof(g_raw_temp_sm), g_raw_temp_sm),
  m_voltCtl(sizeof(g_raw_volt_sm), g_raw_volt_sm)
{
  m_pParent = pParent;
  m_index = index;
  m_bPaused = FALSE;

  m_labelCtl.set_padding(10, 0);

  m_pScopeKey = getFactory().createScopeKey(this);

  m_keyCtl.set_sensitive(FALSE);
  m_keyCtl.set_label("Key...");
  m_keyCtl.signal_toggled().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onShowKey));

  m_hBoxCtl.pack_start(m_tempCtl, FALSE, FALSE);
  m_hBoxCtl.pack_start(m_voltCtl, FALSE, FALSE);
  m_hBoxCtl.pack_start(m_ampCtl, FALSE, FALSE);
  m_hBoxCtl.pack_start(m_keyCtl, FALSE, FALSE);
  m_hBoxCtl.pack_start(m_pauseCtl, FALSE, FALSE);
  pack_start(m_hBoxCtl, TRUE, TRUE);
  m_hBoxCtl.set_spacing(10);
  pack_start(m_plusCtl, FALSE, FALSE);
  pack_start(m_closeCtl, FALSE, FALSE);

  set_spacing(10);
  set_border_width(1);

  m_tempCtl.set_tooltip_text("Show all temperature sensors in scope");
  m_tempCtl.set_sensitive(FALSE);
  m_tempCtl.signal_clicked().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonTemp));
  m_voltCtl.set_tooltip_text("Show all voltage sensors in scope");
  m_voltCtl.set_sensitive(FALSE);
  m_voltCtl.signal_clicked().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonVolt));
  m_ampCtl.set_tooltip_text("Show all current sensors in scope");
  m_ampCtl.set_sensitive(FALSE);
  m_ampCtl.signal_clicked().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonAmp));
  m_boolCtl.set_tooltip_text("Show all boolean sensors in scope");
  m_boolCtl.set_sensitive(FALSE);
  m_pauseCtl.set_tooltip_text("Pause");
  m_pauseCtl.set_sensitive(FALSE);
  m_pauseCtl.signal_toggled().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonPauseToggle));
  m_closeCtl.set_tooltip_text("Close this scope");
  m_closeCtl.set_sensitive(FALSE);
  m_closeCtl.signal_clicked().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonClose));
  m_plusCtl.set_tooltip_text("Add another scope");
  m_plusCtl.set_sensitive(FALSE);
  m_plusCtl.signal_clicked().connect(sigc::mem_fun(*this, &CGraphWndToolbar::onButtonPlus));
}

CGraphWndToolbar::~CGraphWndToolbar()
{
  delete m_pScopeKey;
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_UNIT_TYPE unit)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.Unit == unit) {
      pScope->addTrace(i);
    }
  }
  pScope->addTraceDone();
  m_pParent->started();
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_DATA_TYPE datatype)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.DataType == datatype) {
      pScope->addTrace(i);
    }
  }
  pScope->addTraceDone();
  m_pParent->started();
}

void
CGraphWndToolbar::adjustScale(
  float value)
{
  IScopeWnd* pScope = m_pParent->getScope();

  pScope->adjustScale(value);
}

void
CGraphWndToolbar::onButtonClose(
  void)
{
  m_pParent->getParent()->removeGraphWnd(m_pParent);
}

void
CGraphWndToolbar::onButtonAmp()
{
  addSensors(ADMXRC3_UNIT_A);
  adjustScale(0.0f);
  adjustScale(0.01f);
}

void
CGraphWndToolbar::onButtonBool()
{
  addSensors(ADMXRC3_DATA_BOOL);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

void
CGraphWndToolbar::onButtonTemp()
{
  addSensors(ADMXRC3_UNIT_C);
  adjustScale(0.0f);
  adjustScale(100.0f);
}

void
CGraphWndToolbar::onButtonVolt()
{
  addSensors(ADMXRC3_UNIT_V);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

void
CGraphWndToolbar::onButtonPauseToggle()
{
  if (m_pauseCtl.get_active()) {
    m_bPaused = TRUE;
    m_pauseCtl.set_tooltip_text("Resume");
  } else {
    m_bPaused = FALSE;
    m_pauseCtl.set_tooltip_text("Pause");
  }
}

void
CGraphWndToolbar::onButtonPlus(
  void)
{
  m_pParent->getParent()->addGraphWnd();
}

void
CGraphWndToolbar::onShowKey(
  void)
{
  if (m_keyCtl.get_active()) {
    m_pScopeKey->getNativeWindow().show();
  } else {
    m_pScopeKey->getNativeWindow().hide();
  }
}
