#include "scopekey.h"

CScopeKey::CScopeKey(
  IGraphWndToolbar* pParent)
{
  m_pParent = pParent;
  m_numTrace = 0;
  m_pCheckButtons = NULL;
  m_bFirstShow = TRUE;

  set_type_hint(Gdk::WINDOW_TYPE_HINT_UTILITY);
  set_position(Gtk::WIN_POS_MOUSE);
  set_title("Key");
  set_transient_for(m_pParent->getParent()->getParent()->getNativeWindow());
  set_resizable(false);
  set_border_width(10);

  m_allCtl.set_label("Select All");
  m_allCtl.signal_clicked().connect(sigc::mem_fun(*this, &CScopeKey::onButtonAll));
  m_noneCtl.set_label("Select None");
  m_noneCtl.signal_clicked().connect(sigc::mem_fun(*this, &CScopeKey::onButtonNone));
  m_allNoneCtl.set_spacing(20);
  m_allNoneCtl.set_layout(Gtk::BUTTONBOX_START);
  m_allNoneCtl.pack_start(m_allCtl);
  m_allNoneCtl.pack_start(m_noneCtl);
  m_vboxCtl.set_spacing(10);
  m_vboxCtl.pack_start(m_allNoneCtl);
  m_vboxCtl.pack_start(m_tableCtl);
  add(m_vboxCtl);

  show_all_children();
}

CScopeKey::~CScopeKey()
{
  deleteButtons();
}

void
CScopeKey::deleteButtons()
{
  if (m_numTrace) {
    ASSERT(NULL != m_pCheckButtons);
    for (unsigned int i = 0; i < m_numTrace; i++) {
      m_tableCtl.remove(m_pCheckButtons[i]);
    }
    delete[] m_pCheckButtons;
    m_pCheckButtons = NULL;
    m_numTrace = 0;
  }
}

void
CScopeKey::tracesChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent()->getParent();
  IScopeWnd* pScopeWnd = m_pParent->getParent()->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = pScopeWnd->getNumTrace();

  deleteButtons();
  if (n) {
    m_pCheckButtons = new CTraceCheckButton[n];
    for (unsigned int i = 0; i < n; i++) {
      unsigned int sensorIndex;
      TCHAR* pSensorName;

      sensorIndex = pScopeWnd->getTrace(i);
      ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);
      pSensorName = devInfo.m_pSensors[sensorIndex].m_info.Description;
      m_pCheckButtons[i].setColor(devInfo.m_pSensors[sensorIndex].m_traceColor);
      m_pCheckButtons[i].setRenderer(&m_lineRenderer);
      m_pCheckButtons[i].getButton().set_label(pSensorName);
      m_pCheckButtons[i].getButton().set_active(true);
      m_pCheckButtons[i].getButton().signal_clicked().connect(sigc::bind<unsigned int>(sigc::mem_fun(*this, &CScopeKey::onToggle), i));
      m_tableCtl.pack_start(m_pCheckButtons[i], false, false);
    }
    m_tableCtl.show_all_children();
    m_numTrace = n;
  }
}

void
CScopeKey::onButtonAll(
  void)
{
  for (unsigned int i = 0; i < m_numTrace; i++) {
    m_pCheckButtons[i].getButton().set_active(true);
  }
}

void
CScopeKey::onButtonNone(
  void)
{
  for (unsigned int i = 0; i < m_numTrace; i++) {
    m_pCheckButtons[i].getButton().set_active(false);
  }
}

void
CScopeKey::onToggle(
  unsigned int index)
{
  bool bEnable = m_pCheckButtons[index].getButton().get_active();

  m_pParent->getParent()->getScope()->enableTrace(index, bEnable);
}

void
CScopeKey::on_hide()
{
  get_position(m_x, m_y);

  Gtk::Window::on_hide();
  m_pParent->keyClosed();
}

void
CScopeKey::on_show()
{
  Gtk::Window::on_show();

  if (m_bFirstShow) {
    m_bFirstShow = FALSE;
  } else {
    move(m_x, m_y);
  }
}
