using namespace std;

#include <iostream>
#include <sstream>

#include <math.h>
#include <time.h>

#include "../../sdk_version.h"
#include "sysmonwnd.h"
#include "icon/sysmon.h"

#if (GTK_MAJOR_VERSION >= 3)
# define APPEND_TEXT(ctl, text) ((ctl).append(text))
#else
# define APPEND_TEXT(ctl, text) ((ctl).append_text(text))
#endif

// A color wheel; colors 180 deg. apart have best contrast
static const color_t g_colorWheel[] = {
  RGB(255, 0, 0),
  RGB(255, 125, 0),
  RGB(255, 255, 0),
  RGB(125, 255, 0),
  RGB(0, 255, 0),
  RGB(0, 255, 125),
  RGB(0, 255, 255),
  RGB(0, 125, 255),
  RGB(0, 0, 255),
  RGB(125, 0, 255),
  RGB(255, 0, 255),
  RGB(255, 0, 125)
};

/* Indices of pairs of colors in color wheel above. */
static const unsigned int g_traceColorIndices[][2] = {
  /* Single-color lines */
  { 0, 0, },
  { 1, 1, },
  { 2, 2, },
  { 3, 3, },
  { 4, 4, },
  { 5, 5, },
  { 6, 6, },
  { 7, 7, },
  { 8, 8, },
  { 9, 9, },
  { 10, 10, },
  { 11, 11, },
  /* Bi-color lines, 6 apart on color wheel */
  { 0, 6, },
  { 1, 7, },
  { 2, 8, },
  { 3, 9, },
  { 4, 10, },
  { 5, 11, },
  /* Bi-color lines, 5 apart on color wheel */
  { 0, 5, },
  { 1, 6, },
  { 2, 7, },
  { 3, 8, },
  { 4, 9, },
  { 5, 10, },
  { 6, 11, },
  { 7, 0, },
  { 8, 1, },
  { 9, 2, },
  { 10, 3, },
  { 11, 4, },
  /* Bi-color lines, 4 apart on color wheel */
  { 0, 4, },
  { 1, 5, },
  { 2, 6, },
  { 3, 7, },
  { 4, 8, },
  { 5, 9, },
  { 6, 10, },
  { 7, 11, },
  { 8, 0, },
  { 9, 1, },
  { 10, 2, },
  { 11, 3, }
};

// Sensor read periods, selected m_freqSelectCtl
const unsigned int CSysMonWnd::m_updatePeriods[] = {
  100, 200, 500, 1000, 2000, 5000, 10000
};

// -------------------------------------------------------------------------
// Start of ISysMonWnd interface
// -------------------------------------------------------------------------

bool_t
CSysMonWnd::addGraphWnd()
{
  IGraphWnd** pp = NULL;
  IGraphWnd* pNew = NULL;
  unsigned int n = m_numGraphWnd + 1;

  pp = new IGraphWnd*[n];
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    pp[i] = m_ppGraphWnd[i];
  }
  pNew = getFactory().createGraphWnd(this, m_numGraphWnd, m_currentTab == 2);
  pp[n - 1] = pNew;
  switch (n) {
  case 1:
    m_tableCtl.attach(pNew->getNativeControl(), 0, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    break;

  case 2:
    m_tableCtl.remove(m_ppGraphWnd[0]->getNativeControl());
    m_tableCtl.attach(m_ppGraphWnd[0]->getNativeControl(), 0, 1, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    m_tableCtl.attach(pNew->getNativeControl(), 1, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    break;

  case 3:
    m_tableCtl.remove(m_ppGraphWnd[0]->getNativeControl());
    m_tableCtl.remove(m_ppGraphWnd[1]->getNativeControl());
    m_tableCtl.attach(m_ppGraphWnd[0]->getNativeControl(), 0, 1, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    m_tableCtl.attach(m_ppGraphWnd[1]->getNativeControl(), 1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    m_tableCtl.attach(pNew->getNativeControl(), 0, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    break;

  case 4:
    m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
    m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 0, 1, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    m_tableCtl.attach(pNew->getNativeControl(), 1, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    break;
  }
  pNew->getNativeControl().show_all();
  delete[] m_ppGraphWnd;
  m_ppGraphWnd = pp;
  pNew->deviceChanged();
  setNumGraphWnd(n);

  return TRUE;
}

bool_t
CSysMonWnd::doClearDeviceErrors()
{
  return m_pDeviceStatusWnd->doClearDeviceErrors();
}

bool_t
CSysMonWnd::doSelectDeviceIndex(unsigned int deviceIndex)
{
  if (deviceIndex >= m_device.m_numDevice) {
    return FALSE;
  }
  setDevice(deviceIndex, FALSE);
  return TRUE;
}

bool_t
CSysMonWnd::doSelectTab(unsigned int tabIndex)
{
  if (tabIndex >= 4) {
    return FALSE;
  }
  m_notebookCtl.set_current_page((int)tabIndex);
  return TRUE;
}

void
CSysMonWnd::idle()
{
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getScope()->idle();
  }
}

bool_t
CSysMonWnd::removeGraphWnd(
  IGraphWnd* pGraphWnd)
{
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    if (m_ppGraphWnd[i] == pGraphWnd) {
      return removeGraphWnd(i);
    }
  }

  return FALSE;
}

bool_t
CSysMonWnd::removeGraphWnd(
  unsigned int index)
{
  unsigned int n = m_numGraphWnd;
  IGraphWnd* pGraphWnd;

  pGraphWnd = m_ppGraphWnd[index];
  switch (index) {
  case 0:
    m_tableCtl.remove(pGraphWnd->getNativeControl());
    if (m_numGraphWnd == 4) {
      m_tableCtl.remove(m_ppGraphWnd[1]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[3]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[1]->getNativeControl(), 0, 1, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[3]->getNativeControl(), 0, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    } else if (m_numGraphWnd == 3) {
      m_tableCtl.remove(m_ppGraphWnd[1]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[1]->getNativeControl(), 0, 1, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 1, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    } else if (m_numGraphWnd == 2) {
      m_tableCtl.remove(m_ppGraphWnd[1]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[1]->getNativeControl(), 0, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    }
    break;

  case 1:
    m_tableCtl.remove(pGraphWnd->getNativeControl());
    if (m_numGraphWnd == 4) {
      m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[3]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[3]->getNativeControl(), 0, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    } else if (m_numGraphWnd == 3) {
      m_tableCtl.remove(m_ppGraphWnd[0]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[0]->getNativeControl(), 0, 1, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 1, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    } else if (m_numGraphWnd == 2) {
      m_tableCtl.remove(m_ppGraphWnd[0]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[0]->getNativeControl(), 0, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    }
    break;

  case 2:
    m_tableCtl.remove(pGraphWnd->getNativeControl());
    if (m_numGraphWnd == 4) {
      m_tableCtl.remove(m_ppGraphWnd[3]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[3]->getNativeControl(), 0, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    } else if (m_numGraphWnd == 3) {
      m_tableCtl.remove(m_ppGraphWnd[0]->getNativeControl());
      m_tableCtl.remove(m_ppGraphWnd[1]->getNativeControl());
      m_tableCtl.attach(m_ppGraphWnd[0]->getNativeControl(), 0, 1, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
      m_tableCtl.attach(m_ppGraphWnd[1]->getNativeControl(), 1, 2, 0, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    }
    break;

  case 3:
    m_tableCtl.remove(pGraphWnd->getNativeControl());
    m_tableCtl.remove(m_ppGraphWnd[2]->getNativeControl());
    m_tableCtl.attach(m_ppGraphWnd[2]->getNativeControl(), 0, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
    break;
  }
  delete pGraphWnd;

  for (unsigned int i = index + 1; i < n; i++) {
    m_ppGraphWnd[i - 1] = m_ppGraphWnd[i];
  }
  setNumGraphWnd(n - 1);

  return TRUE;
}

void
CSysMonWnd::requestQuit()
{
  if (m_bLogging) {
    stopLogging(true);
  }
  Gtk::Main::quit();
}

void
CSysMonWnd::setNumGraphWnd(
  unsigned int n)
{
  m_numGraphWnd = n;
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getToolbar()->numGraphWndChanged();
    m_ppGraphWnd[i]->getToolbar()->setIndex(i);
  }
}

// -------------------------------------------------------------------------
// End of ISysMonWnd interface
// -------------------------------------------------------------------------

CSysMonWnd::CSysMonWnd()
{
  static const char* pTitle = "ADMXRC3 Diagnostics";
  m_deviceIndex = (unsigned int)-1;
  m_numGraphWnd = 0;
  m_currentTab = (TabIndex)0;
  m_timerNumber = 0;
  m_bTimerRunning = FALSE;
  ASSERT(ARRAY_LENGTH(m_updatePeriods) >= 4);
  m_updatePeriodIndex = -1;
  m_ppGraphWnd = NULL;
  m_pDeviceInfoWnd = NULL;
  m_pSensorInfoWnd = NULL;
  m_pDeviceStatusWnd = NULL;
  m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  m_device.m_numDevice = 0;
  m_device.m_numSensor = 0;
  m_device.m_numAvailableSensor = 0;
  m_device.m_numClockGen = 0;
  m_bLogging = false;

  m_iconPixbuf = Gdk::Pixbuf::create_from_inline(sizeof(g_raw_sysmon), g_raw_sysmon, false);

  set_title(pTitle);
  set_border_width(10);
  set_icon(m_iconPixbuf);
  signal_window_state_event().connect(sigc::mem_fun(*this, &CSysMonWnd::onWindowStateChanged));
  signal_delete_event().connect(sigc::mem_fun(*this, &CSysMonWnd::onDelete));

  m_trayMenuExit.set_label("Exit");
  m_trayMenuExit.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onTrayMenuExit));
  m_trayMenuRestore.set_label("Restore");
  m_trayMenuRestore.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onTrayMenuRestore));
  m_trayMenuStartNewLog.set_label("Start new log file");
  m_trayMenuStartNewLog.set_sensitive(false);
  m_trayMenuStartNewLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onStartNewLog));
  m_trayMenuAppendToLog.set_label("Append to log file");
  m_trayMenuAppendToLog.set_sensitive(false);
  m_trayMenuAppendToLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onAppendToLog));
  m_trayMenuStopLog.set_label("Stop logging");
  m_trayMenuStopLog.set_sensitive(false);
  m_trayMenuStopLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onStopLog));
  m_trayMenu.append(m_trayMenuRestore);
  m_trayMenu.append(m_trayMenuSeparator1);
  m_trayMenu.append(m_trayMenuStartNewLog);
  m_trayMenu.append(m_trayMenuAppendToLog);
  m_trayMenu.append(m_trayMenuStopLog);
  m_trayMenu.append(m_trayMenuSeparator2);
  m_trayMenu.append(m_trayMenuExit);
  m_trayMenu.show_all();

  m_trayIcon = Gtk::StatusIcon::create(m_iconPixbuf);
#if (GTK_MAJOR_VERSION >= 3)
  m_trayIcon->set_tooltip_text(pTitle);
#else
  m_trayIcon->set_tooltip(pTitle);
#endif
  m_trayIcon->set_visible(FALSE);
  m_trayIcon->signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onTrayActivate));
  m_trayIcon->signal_popup_menu().connect(sigc::mem_fun(*this, &CSysMonWnd::onPopupTrayMenu));

  m_hBoxCtl.set_spacing(30);
  m_hBox1Ctl.set_spacing(10);
  m_hBox2Ctl.set_spacing(10);

  m_deviceLabelCtl.set_label("Device");
  m_hBox1Ctl.pack_start(m_deviceLabelCtl, false, false);
  m_hBox1Ctl.pack_start(m_deviceSelectCtl, false, false);

  m_deviceSelectCtl.signal_changed().connect(sigc::mem_fun(*this, &CSysMonWnd::onSelectDevice));

  m_freqLabelCtl.set_label("Update period");
  m_hBox2Ctl.add(m_freqLabelCtl);

  for (unsigned int i = 0; i < ARRAY_LENGTH(m_updatePeriods); i++) {
    std::ostringstream ss;
    if (m_updatePeriods[i] >= 1000) {
      ss << ((double)m_updatePeriods[i] / 1000.0) << " s";
    } else {
      ss << ((double)m_updatePeriods[i]) << " ms";
    }
    APPEND_TEXT(m_freqSelectCtl, ss.str());
  }
  m_freqSelectCtl.signal_changed().connect(sigc::mem_fun(*this, &CSysMonWnd::onSelectUpdatePeriod));
  m_hBox2Ctl.add(m_freqSelectCtl);

  m_actionMenuStartNewLog.set_label("Start new log file");
  m_actionMenuStartNewLog.set_sensitive(false);
  m_actionMenuStartNewLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onStartNewLog));
  m_actionMenuAppendToLog.set_label("Append to log file");
  m_actionMenuAppendToLog.set_sensitive(false);
  m_actionMenuAppendToLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onAppendToLog));
  m_actionMenuStopLog.set_label("Stop logging");
  m_actionMenuStopLog.set_sensitive(false);
  m_actionMenuStopLog.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onStopLog));
  m_actionMenuAbout.set_label("About");
  m_actionMenuAbout.signal_activate().connect(sigc::mem_fun(*this, &CSysMonWnd::onAbout));
  m_actionMenuCtl.append(m_actionMenuStartNewLog);
  m_actionMenuCtl.append(m_actionMenuAppendToLog);
  m_actionMenuCtl.append(m_actionMenuStopLog);
  m_actionMenuCtl.append(m_actionMenuSeparator);
  m_actionMenuCtl.append(m_actionMenuAbout);
  m_actionMenuCtl.show_all();

  m_actionCtl.set_label("Action...");
  m_actionCtl.signal_clicked().connect(sigc::mem_fun(*this, &CSysMonWnd::onAction));

  m_hBoxCtl.pack_start(m_hBox1Ctl, FALSE, FALSE, 0);
  m_hBoxCtl.pack_start(m_hBox2Ctl, FALSE, FALSE, 0);
  m_hBoxCtl.pack_start(m_actionCtl, FALSE, FALSE, 0);

  m_pDeviceInfoWnd = getFactory().createDeviceInfoWnd(this, TRUE);
  m_notebookLabel1Ctl.set_text("Device Information");
  m_notebookCtl.append_page(m_pDeviceInfoWnd->getNativeControl(), m_notebookLabel1Ctl);

  m_pSensorInfoWnd = getFactory().createSensorInfoWnd(this, TRUE);
  m_notebookLabel2Ctl.set_text("Sensor Information");
  m_notebookCtl.append_page(m_pSensorInfoWnd->getNativeControl(), m_notebookLabel2Ctl);

  m_vBox1Ctl.set_spacing(20);
  m_vBox1Ctl.pack_start(m_hBoxCtl,FALSE, FALSE, 0);
  m_vBox1Ctl.pack_end(m_notebookCtl, TRUE, TRUE, 0);

  m_tableCtl.set_col_spacings(5);
  m_tableCtl.set_row_spacings(5);
  m_tableCtl.set_homogeneous(true);

  m_notebookLabel3Ctl.set_text("Sensor Readout");
  m_notebookCtl.append_page(m_tableCtl, m_notebookLabel3Ctl);

  m_pDeviceStatusWnd = getFactory().createDeviceStatusWnd(this, TRUE);
  m_notebookLabel4Ctl.set_text("Device Status");
  m_notebookCtl.append_page(m_pDeviceStatusWnd->getNativeControl(), m_notebookLabel4Ctl);

  m_pClockGenWnd = getFactory().createClockGenWnd(this, TRUE);
  m_notebookLabel5Ctl.set_text("Clock Generators");
  m_notebookCtl.append_page(m_pClockGenWnd->getNativeControl(), m_notebookLabel5Ctl);

  add(m_vBox1Ctl);

  m_timerSlot = sigc::bind(sigc::mem_fun(*this, &CSysMonWnd::onTimer), m_timerNumber);

  addGraphWnd();
  setUpdatePeriod(3);
  enumerateDevices(NULL);
  if (m_device.m_numDevice) {
    setDevice(0);
  }

  show_all();
}

CSysMonWnd::~CSysMonWnd()
{
  if (m_bTimerRunning) {
    m_timer.disconnect();
    m_bTimerRunning = FALSE;
  }
  if (NULL != m_pDeviceInfoWnd) {
    delete m_pDeviceInfoWnd;
    m_pDeviceInfoWnd = NULL;
  }
  if (NULL != m_pSensorInfoWnd) {
    delete m_pSensorInfoWnd;
    m_pSensorInfoWnd = NULL;
  }
  if (NULL != m_pDeviceStatusWnd) {
    delete m_pDeviceStatusWnd;
    m_pDeviceStatusWnd = NULL;
  }
  if (NULL != m_pClockGenWnd) {
    delete m_pDeviceInfoWnd;
    m_pDeviceInfoWnd = NULL;
  }
  if (NULL != m_ppGraphWnd) {
    for (unsigned int i = 0; i < m_numGraphWnd; i++) {
      delete m_ppGraphWnd[i];
      m_ppGraphWnd[i] = NULL;
    }
    delete[] m_ppGraphWnd;
    m_ppGraphWnd = NULL;
  }
  if (NULL != m_device.m_pSensors) {
    delete[] m_device.m_pSensors;
    m_device.m_pSensors = NULL;
  }
  if (ADMXRC3_HANDLE_INVALID_VALUE != m_device.m_hDevice) {
    ADMXRC3_Close(m_device.m_hDevice);
    m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  }
}

void
CSysMonWnd::enumerateDevices(
  ADMXRC3_CARD_INFOEX* pCardInfoEx)
{
  ADMXRC3_CARD_INFOEX cardInfoEx;
  ADMXRC3_STATUS status;
  ADMXRC3_HANDLE hDevice;
  const char* pModelName;
  unsigned int i, index = 0;
  bool_t bFound = FALSE, bNumDeviceChanged;

#if (GTK_MAJOR_VERSION >= 3)
  m_deviceSelectCtl.remove_all();
#else
  m_deviceSelectCtl.clear_items();
#endif
  for (i = 0; i < 0x100U; i++) {
    std::ostringstream ss;
    status = ADMXRC3_OpenEx(i, TRUE, 0, &hDevice);
    if (ADMXRC3_SUCCESS == status) {
      status = ADMXRC3_GetCardInfoEx(hDevice, &cardInfoEx);
      if (ADMXRC3_SUCCESS == status) {
        pModelName = CAdmxrc3Help::getModelName(cardInfoEx.Model);
        if (NULL == pModelName) {
          ss << "Index " << i << " (Unknown model " << cardInfoEx.Model << " (0x" << HEX << cardInfoEx.Model << DEC << "))";
        } else {
          ss << "Index " << i << " " << pModelName << " SN #" << cardInfoEx.SerialNumber;
        }
      } else {
        ss << "Index " << i << " (Could not determine model)";
      }
      ADMXRC3_Close(hDevice);
    } else {
      if (ADMXRC3_DEVICE_NOT_FOUND == status) {
        break;
      } else {
        ss << "Index " << i << " (Could not open)";
      }
    }
    APPEND_TEXT(m_deviceSelectCtl, ss.str());
  }
  bNumDeviceChanged = (m_device.m_numDevice != i);
  m_device.m_numDevice = i;
  if (0 == m_device.m_numDevice) {
    APPEND_TEXT(m_deviceSelectCtl, "No devices available");
    m_bSelectDeviceFromControl = FALSE;
    m_deviceSelectCtl.set_active(0);
    m_bSelectDeviceFromControl = TRUE;
  }
  if (bFound) {
    m_bSelectDeviceFromControl = FALSE;
    m_deviceSelectCtl.set_active(index);
    m_bSelectDeviceFromControl = TRUE;
  } else {
    if (m_bLogging) {
      stopLogging(FALSE);
    }
  }
  m_deviceSelectCtl.set_sensitive(0 != m_device.m_numDevice ? TRUE : FALSE);
  if (bNumDeviceChanged) {
    m_pDeviceInfoWnd->numDeviceChanged();
    m_pDeviceStatusWnd->numDeviceChanged();
    m_pSensorInfoWnd->numDeviceChanged();
    m_pClockGenWnd->numDeviceChanged();
  }
}

void
CSysMonWnd::onAbout()
{
  Gtk::AboutDialog aboutDialog;
  int version[4] = { ADMXRC_SDK_VERSION_NUM };

  aboutDialog.set_name(get_title());
  std::ostringstream ss;
  ss << version[0] << "." << version[1] << "." << version[2];
  aboutDialog.set_version(ss.str());
  aboutDialog.set_copyright("(C) Copyright 2010-2017 Alpha Data");
  aboutDialog.set_website("http://www.alpha-data.com");
  aboutDialog.set_website_label("www.alpha-data.com");
  aboutDialog.set_logo(m_iconPixbuf);
  aboutDialog.run();
}

void
CSysMonWnd::onAction()
{
  m_actionMenuCtl.popup(0, gtk_get_current_event_time());
}

void
CSysMonWnd::onStartNewLog()
{
  int response;

  ASSERT(!m_bLogging);
  Gtk::FileChooserDialog dialog(*this, "Start new log file", Gtk::FILE_CHOOSER_ACTION_SAVE);
#if (GTK_MAJOR_VERSION >= 3)
  Glib::RefPtr<Gtk::FileFilter> filters[2];
#else
  Gtk::FileFilter filters[2];
#endif
  dialog.set_transient_for(*this);
  dialog.set_select_multiple(false);
#if (GTK_MAJOR_VERSION >= 3)
  filters[0]->set_name("Comma separated value file");
  filters[0]->add_pattern("*.csv");
  filters[1]->set_name("All files");
  filters[1]->add_pattern("*");
  filters[1]->add_pattern("*.*");
#else
  filters[0].set_name("Comma separated value file");
  filters[0].add_pattern("*.csv");
  filters[1].set_name("All files");
  filters[1].add_pattern("*");
  filters[1].add_pattern("*.*");
#endif
  dialog.add_filter(filters[0]);
  dialog.add_filter(filters[1]);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button("Save", Gtk::RESPONSE_OK);
  dialog.set_local_only(true);
  response = dialog.run();
  switch (response) {
    case Gtk::RESPONSE_OK:
      startLogging(dialog.get_filename(), false);
      break;

    case Gtk::RESPONSE_CANCEL:
    default:
      break;
  }
}

void
CSysMonWnd::onAppendToLog()
{
  int response;

  ASSERT(!m_bLogging);
  Gtk::FileChooserDialog dialog(*this, "Append to log file", Gtk::FILE_CHOOSER_ACTION_SAVE);
#if (GTK_MAJOR_VERSION >= 3)
  Glib::RefPtr<Gtk::FileFilter> filters[2];
#else
  Gtk::FileFilter filters[2];
#endif
  dialog.set_transient_for(*this);
  dialog.set_select_multiple(false);
#if (GTK_MAJOR_VERSION >= 3)
  filters[0]->set_name("Comma separated value file");
  filters[0]->add_pattern("*.csv");
  filters[1]->set_name("All files");
  filters[1]->add_pattern("*");
  filters[1]->add_pattern("*.*");
#else
  filters[0].set_name("Comma separated value file");
  filters[0].add_pattern("*.csv");
  filters[1].set_name("All files");
  filters[1].add_pattern("*");
  filters[1].add_pattern("*.*");
#endif
  dialog.add_filter(filters[0]);
  dialog.add_filter(filters[1]);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button("Open", Gtk::RESPONSE_OK);
  response = dialog.run();
  switch (response) {
    case Gtk::RESPONSE_OK:
      startLogging(dialog.get_filename(), true);
      break;

    case Gtk::RESPONSE_CANCEL:
    default:
      break;
  }
}

void
CSysMonWnd::onStopLog()
{
  ASSERT(m_bLogging);
  stopLogging(false);
}

bool
CSysMonWnd::onDelete(GdkEventAny* /* pEventInfo */)
{
  if (m_bLogging) {
    stopLogging(true);
  }
  Gtk::Main::quit();
  return TRUE;
}

void
CSysMonWnd::onPopupTrayMenu(guint button, guint32 activateTime)
{
  m_trayIcon->popup_menu_at_position(m_trayMenu, button, activateTime);
}

void
CSysMonWnd::onSelectDevice()
{
  if (!m_bSelectDeviceFromControl) {
    return;
  }
  setDevice(m_deviceSelectCtl.get_active_row_number(), TRUE);
}

void
CSysMonWnd::onSelectUpdatePeriod()
{
  int index = m_freqSelectCtl.get_active_row_number();

  if (m_updatePeriodIndex == index) {
    return;
  }
  if ((m_device.m_numAvailableSensor || m_device.m_numClockGen) && m_bTimerRunning) {
    m_timer.disconnect();
    m_bTimerRunning = FALSE;
  }
  m_updatePeriodIndex = index;
  if (m_device.m_numAvailableSensor || m_device.m_numClockGen) {
    m_timer = Glib::signal_timeout().connect(m_timerSlot, m_updatePeriods[m_updatePeriodIndex]);
    m_bTimerRunning = TRUE;
  }
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getScope()->updatePeriodChanged();
  }
}

bool
CSysMonWnd::onTimer(
  int timerNumber)
{
  if (timerNumber == m_updateTimerNumber) {
    if (m_device.m_numAvailableSensor) {
      readSensors(getTimeNow());
    }
    m_pDeviceStatusWnd->update();
    m_pClockGenWnd->update();
    return true; /* Continue timer */
  } else {
    assert(FALSE);
    return false;
  }
}

bool
CSysMonWnd::onWindowStateChanged(GdkEventWindowState* pEventInfo)
{
  if (pEventInfo->changed_mask & GDK_WINDOW_STATE_ICONIFIED && pEventInfo->new_window_state & GDK_WINDOW_STATE_ICONIFIED) {
    m_trayIcon->set_visible(TRUE);
    hide();
  } else if (pEventInfo->changed_mask & GDK_WINDOW_STATE_WITHDRAWN && !(pEventInfo->new_window_state & GDK_WINDOW_STATE_WITHDRAWN)) {
    m_trayIcon->set_visible(FALSE);
  }
  return TRUE;
}

void
CSysMonWnd::onTrayActivate()
{
  show();
  deiconify();
}

void
CSysMonWnd::onTrayMenuExit()
{
  requestQuit();
}

void
CSysMonWnd::onTrayMenuRestore()
{
  show();
  deiconify();
}

void
CSysMonWnd::readSensors(timestamp_t timestamp)
{
  SensorInfo* pSensorInfo;
  ADMXRC3_STATUS status;
  ADMXRC3_SENSOR_VALUE sensorValue;
  FILE* pFile = NULL;

  if (m_bLogging) {
    pFile = fopen(m_logFilename.c_str(), "r+");
    if (NULL == pFile) {
      stopLogging(true);
      fprintf(stderr, "sysmon: Failed to open the file '%s' in append mode. Stopping logging.", m_logFilename.c_str());
    } else {
      fseek(pFile, 0, SEEK_END);
      fprintf(pFile, "DATA,%lld", (long long)(timestamp - m_logTimestamp));
    }
  }
  for (unsigned int i = 0; i < m_device.m_numSensor; i++) {
    pSensorInfo = &m_device.m_pSensors[i];
    if (pSensorInfo->m_bKnownDataType) {
      status = ADMXRC3_ReadSensor(m_device.m_hDevice, i, &sensorValue);
      if (ADMXRC3_SUCCESS == status) {
        pSensorInfo->m_current = sensorValue;
      }
      if (NULL != pFile) {
        switch (pSensorInfo->m_info.DataType) {
          case ADMXRC3_DATA_BOOL:
            fprintf(pFile, ",%s", pSensorInfo->m_current.Bool ? "TRUE" : "FALSE");
            break;

          case ADMXRC3_DATA_DOUBLE:
            fprintf(pFile, ",%lf", pSensorInfo->m_current.Double);
            break;

          case ADMXRC3_DATA_INT32:
            fprintf(pFile, ",%ld", (long)pSensorInfo->m_current.Int32);
            break;

          case ADMXRC3_DATA_UINT32:
            fprintf(pFile, ",%lu", (long)pSensorInfo->m_current.UInt32);
            break;

          default:
            fprintf(pFile, ",?");
            break;
        }
      }
    }
  }
  if (NULL != pFile) {
    fprintf(pFile, "\n");
    fclose(pFile);
  }
  m_pSensorInfoWnd->update();
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->update(timestamp);
  }
}

void
CSysMonWnd::setDevice(
  unsigned int deviceIndex,
  bool_t bFromControl)
{
  ADMXRC3_STATUS status;

  if (m_device.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE && deviceIndex == m_deviceIndex) {
    return;
  }

  if (m_bLogging) {
    ASSERT(m_device.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE);
    stopLogging(false);
  }

  if (m_device.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    ADMXRC3_Close(m_device.m_hDevice);
    m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
    m_device.m_numSensor = 0;
    m_device.m_numAvailableSensor = 0;
    if (NULL != m_device.m_pSensors) {
      delete[] m_device.m_pSensors;
    }
    m_device.m_numClockGen = 0;
    if (m_bTimerRunning) {
      m_timer.disconnect();
      m_bTimerRunning = FALSE;
    }
  }

  m_deviceIndex = deviceIndex;
  if (!bFromControl) {
    m_bSelectDeviceFromControl = FALSE;
    m_deviceSelectCtl.set_active(deviceIndex);
    m_bSelectDeviceFromControl = TRUE;
  }

  status = ADMXRC3_OpenEx(m_deviceIndex, TRUE, 0, &m_device.m_hDevice);
  if (ADMXRC3_SUCCESS != status) {
    std::ostringstream ss;
    ss << "Failed to open device " << m_deviceIndex << ": " << ADMXRC3_GetStatusString(status, TRUE);
    Gtk::MessageDialog dialog(*this, ss.str());
    dialog.run();
  } else {
    m_device.m_deviceIndex = m_deviceIndex;
    status = ADMXRC3_GetCardInfoEx(m_device.m_hDevice, &m_device.m_deviceInfo);
    if (ADMXRC3_SUCCESS != status) {
      std::ostringstream ss;
      ss << "Failed to get card information for device " << m_deviceIndex << ": " << ADMXRC3_GetStatusString(status, TRUE);
      Gtk::MessageDialog dialog(*this, ss.str());
      dialog.run();
    } else {
      m_device.m_numSensor = m_device.m_deviceInfo.NumSensor;
    }
    m_device.m_pSensors = new SensorInfo[m_device.m_numSensor];
    ASSERT(NULL != m_device.m_pSensors);
    for (unsigned int i = 0; i < m_device.m_numSensor; i++) {
      status = ADMXRC3_GetSensorInfo(m_device.m_hDevice, i, &m_device.m_pSensors[i].m_info);
      if (ADMXRC3_SUCCESS != status) {
        m_device.m_pSensors[i].m_bValid = FALSE;
      } else {
        m_device.m_pSensors[i].m_bValid = TRUE;
        switch (m_device.m_pSensors[i].m_info.DataType) {
        case ADMXRC3_DATA_BOOL:
        case ADMXRC3_DATA_DOUBLE:
        case ADMXRC3_DATA_INT32:
        case ADMXRC3_DATA_UINT32:
          m_device.m_pSensors[i].m_bKnownDataType = TRUE;
          m_device.m_pSensors[i].m_exponent = powf(10.0f, (float)m_device.m_pSensors[i].m_info.Exponent);
          m_device.m_numAvailableSensor++;
          m_device.m_pSensors[i].m_traceColor[0] = RGB(0xFF, 0xFF, 0xFF);
          m_device.m_pSensors[i].m_traceColor[1] = RGB(0xFF, 0xFF, 0xFF);
          if (i < ARRAY_LENGTH(g_traceColorIndices)) {
            m_device.m_pSensors[i].m_traceColor[0] = g_colorWheel[g_traceColorIndices[i][0]];
            m_device.m_pSensors[i].m_traceColor[1] = g_colorWheel[g_traceColorIndices[i][1]];
          }
          break;

        default:
          /* Don't understand this datatype */
          m_device.m_pSensors[i].m_bKnownDataType = FALSE;
        }
      }
    }
    m_device.m_numClockGen = m_device.m_deviceInfo.NumClockGen;
  }

  if (m_device.m_numAvailableSensor) {
    m_trayMenuStartNewLog.set_sensitive(true);
    m_trayMenuAppendToLog.set_sensitive(true);
    m_actionMenuStartNewLog.set_sensitive(true);
    m_actionMenuAppendToLog.set_sensitive(true);
  } else {
    m_trayMenuStartNewLog.set_sensitive(false);
    m_trayMenuAppendToLog.set_sensitive(false);
    m_actionMenuStartNewLog.set_sensitive(false);
    m_actionMenuAppendToLog.set_sensitive(false);
  }
  if ((m_device.m_numAvailableSensor || m_device.m_numClockGen) && !m_bTimerRunning) {
    m_timer = Glib::signal_timeout().connect(m_timerSlot, m_updatePeriods[m_updatePeriodIndex]);
    m_bTimerRunning = TRUE;
  }
  m_pDeviceInfoWnd->deviceChanged();
  m_pDeviceStatusWnd->deviceChanged();
  m_pSensorInfoWnd->deviceChanged();
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->deviceChanged();
  }
  m_pClockGenWnd->deviceChanged();
}

void
CSysMonWnd::setUpdatePeriod(
  unsigned int index)
{
  m_freqSelectCtl.set_active(index);
}

void
CSysMonWnd::startLogging(
  const std::string& filename,
  bool bAppend)
{
  FILE* pFile;

  m_bLogAppend = bAppend;
  m_logFilename.assign(filename);
  if (bAppend) {
    pFile = fopen(m_logFilename.c_str(), "r+");
    if (NULL == pFile) {
      std::ostringstream ss;
      ss.str("");
      ss << "Failed to open the file\n\n";
      ss << m_logFilename;
      Gtk::MessageDialog errorDialog(*this, ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
      errorDialog.set_title("Unable to open file");
      errorDialog.run();
      return;
    } else {
      fseek(pFile, 0, SEEK_END);
    }
  } else {
    pFile = fopen(m_logFilename.c_str(), "w");
    if (NULL == pFile) {
      std::ostringstream ss;
      ss.str("");
      ss << "Failed to create the file\n\n";
      ss << m_logFilename;
      Gtk::MessageDialog errorDialog(*this, ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
      errorDialog.set_title("Unable to create file");
      errorDialog.run();
      return;
    }
  }
  m_bLogging = TRUE;
  emitLogHeader(pFile, m_device, m_logTimestamp);
  fclose(pFile);
  m_trayMenuStartNewLog.set_sensitive(false);
  m_trayMenuAppendToLog.set_sensitive(false);
  m_trayMenuStopLog.set_sensitive(true);
  m_actionMenuStartNewLog.set_sensitive(false);
  m_actionMenuAppendToLog.set_sensitive(false);
  m_actionMenuStopLog.set_sensitive(true);
}

void
CSysMonWnd::stopLogging(bool bQuiet)
{
  FILE* pFile;

  pFile = fopen(m_logFilename.c_str(), "r+");
  if (NULL == pFile) {
    if (!bQuiet) {
      fprintf(stderr, "sysmon: Failed to open the file '%s' in append mode when stopping logging.", m_logFilename.c_str());
    }
  } else {
    struct tm* pDateTime;
    char dateStr[32];
    time_t calendarTime;

    time(&calendarTime);
    pDateTime = localtime(&calendarTime);
    strftime(dateStr, ARRAY_LENGTH(dateStr) - 1, "%H:%M:%S %d %b %Y", pDateTime);
    dateStr[ARRAY_LENGTH(dateStr) - 1] = '\0'; /* Ensure nul-terminated */
    fseek(pFile, 0, SEEK_END);
    fprintf(pFile, "STOP,%s\n", dateStr);
    fclose(pFile);
  }
  m_bLogging = FALSE;
  m_trayMenuStartNewLog.set_sensitive(true);
  m_trayMenuAppendToLog.set_sensitive(true);
  m_trayMenuStopLog.set_sensitive(false);
  m_actionMenuStartNewLog.set_sensitive(true);
  m_actionMenuAppendToLog.set_sensitive(true);
  m_actionMenuStopLog.set_sensitive(false);
}

void
CSysMonWnd::emitLogHeader(
  FILE* pFile,
  const DeviceInfo& devInfo,
  timestamp_t& timestamp)
{
  struct tm* pDateTime;
  char dateStr[32];
  time_t calendarTime;

  timestamp = getTimeNow();
  time(&calendarTime);
  pDateTime = localtime(&calendarTime);
  strftime(dateStr, ARRAY_LENGTH(dateStr) - 1, "%H:%M:%S %d %b %Y", pDateTime);
  dateStr[ARRAY_LENGTH(dateStr) - 1] = '\0'; /* Ensure nul-terminated */
  fprintf(pFile, "START,%s\n", dateStr);
  fprintf(pFile, "COMMENT,MODEL,SERIAL#\n");
  fprintf(pFile, "DEVICE,%s,%lu\n", CAdmxrc3Help::getModelName(devInfo.m_deviceInfo.Model), (unsigned long)devInfo.m_deviceInfo.SerialNumber);
  fprintf(pFile, "COMMENT,SENSOR#,Description,Unit\n");
  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    fprintf(pFile, "SENSOR,%u,%s,%s\n", i, devInfo.m_pSensors[i].m_info.Description, CAdmxrc3Help::getUnitString(devInfo.m_pSensors[i].m_info.Unit));
  }
  fprintf(pFile, "COMMENT,TIMESTAMP");
  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    fprintf(pFile, ",%s", devInfo.m_pSensors[i].m_info.Description);
  }
  fprintf(pFile, "\nCOMMENT,ms");
  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    fprintf(pFile, ",%s", CAdmxrc3Help::getUnitString(devInfo.m_pSensors[i].m_info.Unit));
  }
  fprintf(pFile, "\n");
}
