#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "factory.h"
#include "graphwndtoolbar.h"
#include "resource_button.h"
#include "resource_icon.h"

IMPLEMENT_DYNAMIC(CGraphWndToolbar, CWnd)

BEGIN_MESSAGE_MAP(CGraphWndToolbar, CWnd)
  ON_COMMAND(IDCTL_AMP, OnButtonAmp)
  ON_COMMAND(IDCTL_CLOSE, OnButtonClose)
  ON_COMMAND(IDCTL_TEMP, OnButtonTemp)
  ON_COMMAND(IDCTL_BOOL, OnButtonBool)
  ON_COMMAND(IDCTL_GO, OnButtonGo)
  ON_COMMAND(IDCTL_PAUSE, OnButtonPause)
  ON_COMMAND(IDCTL_PLUS, OnButtonPlus)
  ON_COMMAND(IDCTL_VOLT, OnButtonVolt)
  ON_COMMAND(IDCTL_KEY, OnButtonKey)
  ON_WM_CREATE()
  ON_WM_SIZE()
END_MESSAGE_MAP()

// -------------------------------------------------------------------------
// Start of IGraphWndToolbar interface
// -------------------------------------------------------------------------

void
CGraphWndToolbar::deviceChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  SensorInfo* pSensorInfo;
  bool_t bEnableTemp = FALSE;
  bool_t bEnableVolt = FALSE;
  bool_t bEnableAmp = FALSE;
  bool_t bEnableBool = FALSE;

  m_pParent->getScope()->removeAllTraces();
  m_pParent->getScope()->addTraceDone();

  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    pSensorInfo = &devInfo.m_pSensors[i];
    if (pSensorInfo->m_bValid && pSensorInfo->m_bKnownDataType) {
      switch (pSensorInfo->m_info.Unit) {
      case ADMXRC3_UNIT_C:
        bEnableTemp = TRUE;
        break;

      case ADMXRC3_UNIT_V:
        bEnableVolt = TRUE;
        break;

      case ADMXRC3_UNIT_A:
        bEnableAmp = TRUE;
        break;

      case ADMXRC3_DATA_BOOL:
        bEnableBool = TRUE;
        break;

      default:
        break;
      }
    }
  }
  m_tempCtl.EnableWindow(bEnableTemp);
  m_voltCtl.EnableWindow(bEnableVolt);
  m_ampCtl.EnableWindow(bEnableAmp);
  m_boolCtl.EnableWindow(bEnableBool);
}

void
CGraphWndToolbar::keyClosed()
{
  if (m_keyCtl.GetCheck() == BST_CHECKED) {
    m_keyCtl.SetCheck(BST_UNCHECKED);
  }
}

void
CGraphWndToolbar::numGraphWndChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  unsigned int n;

  n = pMainWnd->getNumGraphWnd();
  m_plusCtl.EnableWindow(n < 4);
  m_closeCtl.EnableWindow(n >= 2);
}

void
CGraphWndToolbar::numTraceChanged()
{
  unsigned int n = m_pParent->getScope()->getNumTrace();

  if (n > 0) {
    m_keyCtl.EnableWindow(TRUE);
    m_goCtl.EnableWindow(TRUE);
    m_pauseCtl.EnableWindow(TRUE);
  } else {
    m_goCtl.EnableWindow(FALSE);
    m_pauseCtl.EnableWindow(FALSE);
    m_keyCtl.SetCheck(BST_UNCHECKED);
    m_pKey->getNativeWindow().ShowWindow(SW_HIDE);
    m_keyCtl.EnableWindow(FALSE);
  }
}

void
CGraphWndToolbar::setIndex(
  unsigned int index)
{
  m_index = index;
}

void
CGraphWndToolbar::tracesChanged()
{
  m_pKey->tracesChanged();
}

// -------------------------------------------------------------------------
// End of IGraphWndToolbar interface
// -------------------------------------------------------------------------

CGraphWndToolbar::CGraphWndToolbar(
  IGraphWnd* pParent,
  unsigned int index) :
  m_tempCtl(IDI_TEMP),
  m_voltCtl(IDI_VOLT),
  m_ampCtl(IDI_AMP),
  m_boolCtl(IDI_BOOL),
  m_pauseCtl(IDI_PAUSE),
  m_goCtl(IDI_GO),
  m_plusCtl(IDI_PLUS),
  m_closeCtl(IDI_CLOSE)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_index = index;
  m_bPaused = FALSE;
  m_bFirstShowKey = TRUE;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | WS_VISIBLE, 0, 0, 1, 1, m_pParent->getNativeControl(), NULL, NULL);
  ASSERT(bSuccess);
}

CGraphWndToolbar::~CGraphWndToolbar()
{
  delete m_pKey;
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_UNIT_TYPE unit)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.Unit == unit) {
      pScope->addTrace(i);
    }
  }
  pScope->addTraceDone();
  m_pParent->started();
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_DATA_TYPE datatype)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.DataType == datatype) {
      pScope->addTrace(i);
    }
  }
  pScope->addTraceDone();
  m_pParent->started();
}

void
CGraphWndToolbar::adjustScale(
  float value)
{
  IScopeWnd* pScope = m_pParent->getScope();

  pScope->adjustScale(value);
}

bool_t
CGraphWndToolbar::PreTranslateMessage(
  MSG* pMsg)
{
  m_toolTipCtl.RelayEvent(pMsg);

  return CWnd::PreTranslateMessage(pMsg);
}

void
CGraphWndToolbar::OnButtonAmp()
{
  addSensors(ADMXRC3_UNIT_A);
  adjustScale(0.0f);
  adjustScale(0.01f);
}

void
CGraphWndToolbar::OnButtonBool()
{
  addSensors(ADMXRC3_DATA_BOOL);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

void
CGraphWndToolbar::OnButtonClose(
  void)
{
  m_pParent->getParent()->removeGraphWnd(m_pParent);
}

void
CGraphWndToolbar::OnButtonGo(
  void)
{
  bool_t bSetFocus = FALSE;

  bSetFocus = (GetFocus() == &m_goCtl);
  m_goCtl.ShowWindow(FALSE);
  m_pauseCtl.ShowWindow(TRUE);
  if (bSetFocus) {
    m_pauseCtl.SetFocus();
  }
  m_bPaused = FALSE;
}

void
CGraphWndToolbar::OnButtonKey(
  void)
{
  int nCmdShow = (m_keyCtl.GetCheck() == BST_CHECKED) ? SW_SHOWNORMAL : SW_HIDE;
  POINT p;

  if (nCmdShow == SW_SHOWNORMAL) {
    if (m_bFirstShowKey) {
      GetCursorPos(&p);
      p.x -= 10;
      p.y -= 10;
      m_pKey->getNativeWindow().SetWindowPos(NULL, p.x, p.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
      m_bFirstShowKey = FALSE;
    }
  }
  m_pKey->getNativeWindow().ShowWindow(nCmdShow);
}

void
CGraphWndToolbar::OnButtonPause(
  void)
{
  bool_t bSetFocus = FALSE;

  bSetFocus = (GetFocus() == &m_pauseCtl);
  m_pauseCtl.ShowWindow(FALSE);
  m_goCtl.ShowWindow(TRUE);
  if (bSetFocus) {
    m_goCtl.SetFocus();
  }
  m_bPaused = TRUE;
}

void
CGraphWndToolbar::OnButtonPlus(
  void)
{
  m_pParent->getParent()->addGraphWnd();
}

void
CGraphWndToolbar::OnButtonTemp()
{
  addSensors(ADMXRC3_UNIT_C);
  adjustScale(0.0f);
  adjustScale(100.0f);
}

void
CGraphWndToolbar::OnButtonVolt()
{
  addSensors(ADMXRC3_UNIT_V);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

int
CGraphWndToolbar::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  RECT r = { 0, 0, 0, 0 };
  CString caption;

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  m_tempCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_TEMP);
  m_tempCtl.EnableWindow(FALSE);
  m_voltCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_VOLT);
  m_voltCtl.EnableWindow(FALSE);
  m_ampCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_AMP);
  m_ampCtl.EnableWindow(FALSE);
  m_boolCtl.Create(_T(""), WS_CHILD | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_BOOL);
  m_boolCtl.EnableWindow(FALSE);
  m_keyCtl.Create(_T("Key..."), WS_CHILD | WS_VISIBLE | BS_PUSHLIKE | BS_AUTOCHECKBOX, r, this, (UINT)IDCTL_KEY);
  m_keyCtl.SetFont(&m_font);
  m_keyCtl.EnableWindow(FALSE);
  m_pauseCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_PAUSE);
  m_pauseCtl.EnableWindow(FALSE);
  m_goCtl.Create(_T(""), WS_CHILD | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_GO);
  m_goCtl.EnableWindow(FALSE);
  m_plusCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_PLUS);
  m_plusCtl.EnableWindow(FALSE);
  m_closeCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_CLOSE);
  m_closeCtl.EnableWindow(FALSE);

  m_toolTipCtl.CreateEx(this, TTS_ALWAYSTIP);
  m_toolTipCtl.SetFont(&m_font);
  m_toolTipCtl.AddTool(&m_tempCtl, _T("Show all temperature sensors in scope"));
  m_toolTipCtl.AddTool(&m_voltCtl, _T("Show all voltage sensors in scope"));
  m_toolTipCtl.AddTool(&m_ampCtl, _T("Show all current sensors in scope"));
  m_toolTipCtl.AddTool(&m_boolCtl, _T("Show all boolean sensors in scope"));
  m_toolTipCtl.AddTool(&m_pauseCtl, _T("Pause"));
  m_toolTipCtl.AddTool(&m_goCtl, _T("Resume"));
  m_toolTipCtl.AddTool(&m_closeCtl, _T("Close this scope"));
  m_toolTipCtl.AddTool(&m_plusCtl, _T("Add another scope"));
  m_toolTipCtl.Activate(TRUE);

  m_pKey =  getFactory().createScopeKey(this);

  return 0;
}

void
CGraphWndToolbar::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  const unsigned int buttonSize = 28;
  int x, dy;

  CWnd::OnSize(nType, cx, cy);

  dy = (cy - buttonSize) >> 1;
  x = dy;
  m_tempCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_voltCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_ampCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_keyCtl.MoveWindow(x, dy, 40, buttonSize);
  x += 46;
  m_pauseCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  m_goCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  dy = ((cy - buttonSize) >> 1);
  m_plusCtl.MoveWindow(cx - (2 * buttonSize + 6 + dy), dy, buttonSize, buttonSize);
  m_closeCtl.MoveWindow(cx - (buttonSize + dy), dy, buttonSize, buttonSize);
}
